c23456789a123456789b123456789c123456789d123456789e123456789f123456789g12
c  Program SIMULPS14 (24 sept 1998)
c  Published version April 1999
c
c  based on simulps13q from donna eberhart-phillips, march 1998
c  
c  florian haslinger, eth zurich
c  implement RKP (RungeKutta +  perturbation)  raytracer (J. Virieux, see Virieux et al,-
c  Ray tracing For Earthquake location In Laterally Heterogeneous Media). 
c 
c  the RKP raytracer will be used if i3d=4. an input file
c  RAYTRAC must then be supplied containing the parameters for the
c  raytracer.
c  The b-spline interpolation used in RKP-raytracing requires a grid of
c  velocity values with constant spacing in each direction.
c  Trilinear interpolation from the input velocity grid in MOD down to
c  a grid with spacing dxrt,dyrt,dzrt is done in routine bldrtgrd.
c  The rt-grid does not encompass the boundaries of the input velocity grid
c  but is hard-wired to a region [(.km(2)-2*d.rt) , (.km(n-1) + 2*d.rt)].
c  (this is for x and y, for z the grids have the same region)
c  Make sure that all gridpoints of the original grid are hit by 
c  gridpoints of the rt-grid. For the effects of the parametrization
c  see the Ph.D of F. Haslinger.
c 
c  following comments from simulps13q - also valid for this program
c 
c   This subroutine has been modified for Q inversion 15-March-1998.
c   Following Andreas Rietbrock's version.
c   The input velocity file has Vp then Q.
c   The input data is tstar with the hypocentre from the 3-D Vp
c   solution (which must be already done for the model region).
c   The input hypocentres are treated as shots and so are not
c   perturbed in the Q inversion.
c
c   Then (when iuseq=1), the second part of the model array
c   is used to store Vp*Qp (not Vp/Vs).
c   The solution is ONLY for Q, not for Vp though.  
c   THUS the solution arrays that would otherwise be related
c   to the first part of the model array (Vp) are used.
c   The fixed points, damping and dvmax otherwise related to 
c   Vp are related to Q or V*Q for iuseq=1.  
c   Fixed node indices for Q are as for Vp otherwise (ie: z indices 2 to nz-1).
c   As this is slightly confusing to the user, qdamp and dqmax are also 
c   included as input values.
c
c   The Q option has only been used for Qp by Andreas and Donna.  Note
c   however that it should just as well work for Qs if the input data
c   were t* for S and the input velocity model was Vs and Qs.
c
c
c   This version  inverts for Vp and Vp/Vs (not
c   for Vp and Vs).  The input data are P travel-times and S-P times
c   (previous versions used P travel-times and S travel-times.  See
c   Thurber (ref 1 below) for discussion of solution for Vp/Vs
c   using S-P data.
c   This version also allows the user to vary the weighting, in the 
c   hypocenter solution, of the S-P data relative to the P data. (see wtsp)
c   Vp and Vp/Vs are input and are used to compute Vs at each iteration.
c   The S velocities are on the same grid as the p velocities and are
c   stored in additional nz layer positions.  
c   Uses up to 'maxpar' velocity nodes (and station parameters), but only invert for up to 'mxpari'.
c   'maxpar' and 'mxpari' are set in ' simulps_common.inc'.
c   Has the option of outputing the raypaths.
c   Has Thurber's psuedo-bending ray-tracing from his SIMUL3M version
c   Allows less curvature for initial arcuate rays below "moho"
c   (see i3d input parameter).
c
c        Donna Eberhart-Phillips, Ph.D.
c        Inst. of Geological and Nuclear Sciences
c        Earth & Ocean Sciences Research
c        P. O. Box 56
c        Dunedin, New Zealand
c 
c        email:  donna@waves.otago.ac.nz
c 
c        Telephone: 64-3-479-9051
c        Fax: 64-3-479-7586
c
c
c  If you are using this program please read and reference the following
c     chapters by myself & Cliff Thurber:
c
c  1)  Thurber, C. H., Local earthquake tomography: velocities and Vp/Vs - theory,
c            in Seismic Tomography: Theory and Practice, edited by
c            H. M. Iyer and K. Hirahara, 1993. 
c
c  2)  Eberhart-Phillips, D., Local earthquake tomography: earthquake source regions,
c            in Seismic Tomography: Theory and Practice, edited by
c            H. M. Iyer and K. Hirahara, 1993. 
c
c  3)  Thurber, C. H., Earthquake locations and three-dimensional crustal structure
c             in the Coyote Lake area, central California, J. Geophys. Res., 
c             v. 88, p. 8226-8236, 1983.
c
c  4) If you are doing a Q inversion, reference:
c      Rietbrock, A., Entwicklung eines Programmsystems zure konsistenten
c            Auswertung grosser seismologischer Datensatze mit Anwendung auf
c            die Untersuchung der Absorptions-struktur der Loma-Prieta-Region
c            Kalifornien, Ph.D. Thesis, Ludwig-Maximilians-Universitat Munchen,
c            Munchen, Germany, 1996.
c
c
c  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
c  USGS disclaimer statement:
c  ALTHOUGH THIS PROGRAM HAS BEEN USED BY THE USGS, NO WARRANTY,
c  EXPRESSED OR IMPLIED, IS MADE BY THE USGS OR THE UNITED
c  STATES GOVERNMENT AS TO THE ACCURACY AND FUNCTIONING OF THE
c  PROGRAM AND RELATED PROGRAM MATERIAL NOR SHALL THE FACT OF
c  DISTRIBUTION CONSTITUTE ANY SUCH WARRANTY, AND NO 
c  RESPONSIBILITY IS ASSUMED BY THE USGS IN CONNECTION THEREWITH.
c
c  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
c
cdmep 1-Oct-98  Update dlta in loceqk, which is used in distance weighting
c
cfh suggestion: incorporate  file-naming for input and output:
cfh this version:
cfh    input:     file 01: CNTL
cfh               file 02: STNS
cfh               file 03: MOD
cfh               file 04: EQKS
cfh               file 07: SHOT
cfh               file 08: BLAS
cfh    output:    file 12: summary
cfh               file 13: finalsmpout
cfh               file 15: ev'xxxx'.rp
cfh               file 16: output
cfh               file 17: resol.out
cfh               file 18: nodes.out
cfh               file 19: f19
cfh               file 20: residuals
cfh               file 22: newstns
cfh               file 23: velomod.out
cfh               file 24: f24
cfh               file 25: results.for_plot
cfh               file 26: pseudo
cfh               file 28: f28
cfh               file 34: hypo71list
cfh               file 36: itersum
cfh               file 45: f45
c  some additional changes by f.haslinger, eth zurich
c  17-feb-99
c  - format of final output to file 16 (output), meant to be 'resolution in more 
c    readable format' commented out, as large grids make this unreadable
c  24-sept-98
c   - in subroutine aztoa take first 2 raypoints also for azimuth calculation
c     (previously first and last rp)
c  15-june-98
c  - incorporate changes suggested by donna to get number of
c    nodes inverted for in file 18 (full resolution output)
c    (marked cfhdmep)
c    added nparpi, nparsi, nrowp, nrows, nrowst to common
c  - incorporate changes (from e.kissling) to deal with N/S , E/W
c    problem. requires correct coordinate qualifier (N/S or E/W) in
c    station and event coordinates and uses cns, cew to get correct
c    values (N, W are positive; S, E are negative - U.S centered world...)
c    !! for output to file 25 in case of inversion for q, the coordinate output
c       has not been changed
c  - small changes in the output formats to file 16 (output) for compatibility
c    with sim2gmt
c
c 
c  Record of changes I have made in program (DMEP).
c  15-Mar-98 Various changes to solve for Q, following the code by 
c     Andreas Rietbrock.
c  20-Oct-97 For INPUT2, read in all stations to end of file rather than
c  stopping at nsts
c  12-sep-96 Add more numbers to output summary file 36
c  8-may-96  Make delta epicentral distance to work better for deep 
c      events and to be correct hypo71 output for aztoa
c  22-aug-95 Added nzco to input2 to get azimuth correct for New Zealand
c  15-mar-95 Minor output change (input1)
c  30-jun-94 Also save ndof1,wndof1 in OUTADJ
c  22-jun-94 Save var1,varw1 in OUTADJ instead of recomputing
c     in DECIDE
c  10-jun-94 Clarified ftest variables.
c 21-apr-94 Changed 'a' to 'ac' for hypocenter error ellipse
c   calculation in LOCEQK
c 15-mar-94 Added output file 36 that has summary output.
c 11-mar-94 Changes throughout program to have common blocks
c    in a separate file that is included: simulps_common.inc
c 7-mar-94 Change to TTMDER.  Improved check for writing to
c    boundary nodes.
c 27-nov-93  Several changes to take care of "uninitialized variables".
c    Most of these were in HYPOINVERSE write statements in subroutine OUT
c 25-nov-93  This version has been modified by J. Evans so that
c    it will compile better on sun machines.  (changes to do loops
c  4-oct-93 sdl was not being used (in s-p version).  Change to TTMDER so that sdl is
c      used.
c  12-jul-93 Minor change to OUTEND in format for writing nparvi to nodes output file
c  08-jul-93 Do not allow damping to become smaller (with idmp)
c     Also do not calculate new damp if zero nodes observed
c  07-jul-93 Change npar dimension to 5200
c  03-jul-93 Change to OUTEND for writing station output
c  23-jun-93 Minor change to OUTEND for writing number of stations
c  22-jun-93 Change number of stations allowed to 1800.
c  21-jun-93 Change to INPUT4 to note observed stations that are not in station list
c            if kout2 eq 0 or 1
c  16-jun-93 Change to BEND to avoid divide by zero for constant velocity (zero gradient)
c  03-jun-93 Change to FORWRD.  Corrected way synthetic data was computed for
c      S-P observations.
c  27-apr-93 Change to MEDDER.  Deleted old wr= ... dres that should have
c       been taken out when res3 was added to residual downweighting.
c  23-apr-93 Changes to VELADJ so that solution statistics will be correct
c      for the case where there are relatively few velocity nodes relative
c      to the number of station corrections.
c  19-mar-93 Minor change in INPUT4. 
c  23-feb-93 Change to subroutine OUTEND so that ttobs for S-P is correct
c      for file024.
c  10-feb-93 Change to subroutine OUTRES so that ttobs for S-P is correct
c  1-feb-93 Various minor changes to make more compatible with 
c      sun compiler
c      (Note compile on sun with f77 -lV77 simulps12.for -o simulps12)
c  7-jan-93 Also changed VELBKU for vp/vs
c  6-jan-93 Changes of OUTADJ, OUTEND so do not print out grids that
c      have dws=0.0
c      Changes to VELADJ for case of vp and sta corr only (no vp/vs).
c  5-jan-93 Changes to VELADJ to output sol norm and damping
c      for both vp and vp/vs.
c      Added "wtsp" to allow user to change relative weighting of
c      S-P observations in hypocentral solution.  Changes to WTHYP.
c  30-dec-92 Made changes so that vpvs array is used directly. (Cliff
c       had solved for vp/vs but then used the vp/vs perturbation in
c       veladj to perturb the associated vs element.)  Now vp/vs is 
c       input instead of vs, and vs is calculated from vp and vp/vs
c       in input3 and at end of veladj.
c  5-nov-92 Now have option to create synthetic travel-time data.
c           Use nitmax= -1, use file007 for input hypocenters and
c           stations.  Calculated travel-times will be output to
c           file028.  Made changes to Main and Outend.
c  3-nov-92 Made various changes suggested by Cliff Thurber in order to
c     use S-P times to invert for Vp/Vs instead of using S times to
c     invert for Vs.
c  12-mar-92 Changes to RAYWEB to have different curvature for rays below "moho"
c  24-feb-92 as noted by L. Hutchings, added nbls to variance calculation in RESCOV
c  24-feb-92 Added "bld" to input3.  Bld is factor for setting up velocity
c      interpolation arrays (1.0 or 0.1). Now velocity nodes can be defined to 1/10th km
c      if desired.  Changes to BLDMAP, INPUT3, INTMAP, OUTEND.
c 15-mar-91 In VELADJ, DECIDE, RESCOV, added khit=0 check to all hitct checks 
c    (and deleted hit=0 checks).  Parsep uses khit=0 to setup solution matrix so it
c    should also be used here.
c 22-feb-91 Changes from Egill Hauksson. Corrected variable name in 
c    zeroing out section of RESCOV.  Added hitct check to rhm summation
c    in VELADJ. Also minor printout change in INPUT3
c 9-oct-90 In DECIDE use rmsw instead of rms for comparison to rmstop
c 4-oct-90 In OUTEND use ssqrw instead of ssqr in calculating 'rat' for
c      use in stderr.
c  7-sep-90 Write out 1/covariance(diag) if kout2.eq.5 and ires.gt.0
c      to for016 and for045; changes for OUTEND, RESCOV.
c  3-aug-90 Made changes suggested by Goran Ekstrom for convergence 
c      check in LOCEQK.
c  12-jul-90 Put in a second linear residual weighting, 98% of downweighting
c      is done res1 to res2, last 2% of downweighting is done 
c      res2 to res3.  This is useful in case you happen to get a poor
c      hypocenter with mostly very high residuals.  If res3=res2,
c      100% of downweighting is done res1 to res2, as before.
c      Remove "inew" from MAIN since it's never used.
c      In MAIN, if nitmax.eq.0, skip parsep for blasts & shots also.
c  11-jul-90 Use hit array instead of khit to do cutoff in VELADJ,
c      also changing "nhitct" to "hitct".  A peripheral node can have
c      a high enough khit even though it has no rays near it.
c  27-jun-90 Change input4 so it can also read Alberto's format with
c      millisec. travel-times
c  15-jun-90 Output data variance as rms**2 also.  Correct weighted 
c      variance which should use wnobt in calculating wnodf.  Changes
c      to DECIDE.
c  11-apr-90 Allow ires=3 to only compute resolution on 1st iteration.
c      Saves cpu and is appropriate when idmp=1(damping increases,
c      resolution decreases on later iterations.)
c      Changes to MAIN.
c  6-oct-89 Add "snrmct" to stop iterating if solution norm is
c      less than the specified cutoff value.
c      Changes to MAIN,INPUT1,VELADJ.
c  4-oct-89 Can now invert for station corrections at selected stations
c      by fixing delays (to 0) at other stations.  Changes to common
c      OBSERVE, routines INPUT2, INPUT3, TTMDER, VELADJ.
c  6-jun-89 Fixed hypocentral error (diagonal elements were incorrectly
c      coded as noted by Goran Ekstrom), and error ellipse (for 
c      coordinate rotation) in LOCEQK
c     Have LOCEQK calculate error ellipses and print out
c      HYPOINVERSE-style summary cards, if kout=5.  OUT now called
c      from LOCEQK.
c     In LOCEQK, set ster to 0 before calc. so correct for blasts
c     In SETUP,TTMDER calculation of number of raypath segments, changed
c      from IFIX to NINT, should make it more accurate for short paths
c      where few segments are used
c  2-mar-89 Minor changes to RESCOV so more appropriate for ires=1
c     Changes to TTMDER so that it checks for writing outside defined
c     dtm-array.  This may have caused errors before (numerous 'poorly
c     constrained depth') if deepest grid was not far enough away.
c  22-sep-87 Made a variety of changes to make the program work properly
c     when station delays are included in the inversion (also for the
c     case when all velocity gridpoints are fixed).  Removed line in 
c     Parsep that was wrong for station corrections in inversion
c     Set index arrays properly for station delays, in Input3.
c     Skip sections of ttmder, veladj, velbku, outadj, outend if all velocity nodes are fixed
c     Don't calculate station delay partial derivatives when Ttmder 
c     is called from Loceqk.
c     Fixed array sizes in Main, Strt.
c     Have 3 damping parameters (3rd for station delays), changes to Input1
c  20-feb-87 Changed program so that when earthquake locations are fixed on the
c     first iterations, they are not weighted like shots or blasts.
c  12-jul-86  Fixed dr5 calculation in Path.
c  10-jul-86  Fixed initial value of xmov, pl(no) in LOCEQK.
c  08-jul-86  Also tally station observations by P and S.
c     In Loceqk have last 3d raypath remembered and used as the initial
c     path in Minima, if small change in hypocenter location on last
c     iteration.
c  05-jul-86  Fixed jndex array (which is used for p-or-s decisions).  
c     It had not been updated after poorly sampled nodes were deleted.
c  30-jun-86  Fixed a counter error in MEDDER.  Added write
c     statements to parsep to where resolution problem is
c     arising.
c
c
c  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
c
c
c  (In addition to these notes, see also the handout written by
c   Cliff)
c
c  Written by Cliff Thurber  as part of his PhD thesis.
c  Modified by W. Prothero to include station delays.
c  Parts of this were programmed by Steve Taylor of LLL.
c  Obtained from Prothero&Thurber in 1983 and subsequently
c  modified by Donna Eberhart-Phillips, U.S. Geological Survey.
c
c
c  Input data file list:
c     File 01 - Control Parameters  (read in subroutine INPUT1)
c        line 1- (free format)
c        neqs - number of earthquakes
c        nsht  - number of shots
c        nbls  - number of blasts with known location, but unknown
c                origin time
c        wtsht - weighting of shots relative to quake weighting
c        kout - output control parameter
c               value    files created
c                 0      16,36
c                 1      16,13,36
c                 2      16,13,22,23,24,36
c                 3      16,13,22,23,24,34,36
c                 4      16,13,22,23,24,25,34,36
c                 5      16,12,13,22,23,24,25,34,36
c        kout2 - Printout control parameter
c                0 = full printout including station residuals and
c                    location steps
c                1 = printout station residuals
c                2 = printout location steps
c                3 = don't printout location steps or station residuals
c                4 = as 3 above, also don't printout stations in input2
c                5 = as 4. Also printout 1/(diag. covariance) to for016 
c                    and for045, if ires.gt.0.
c        kout3 - Yet another output control parameter
c                0 = Don't output raypath points or tt differences
c                1 = Output raypath points to file 15, for all raypaths
c                    Print out travel-time differences between ART and
c                    psuedo-bending to File 19.
c                    This is useful for making plots of raypaths.
c                    (For instance, if you want to test a range of psuedo-
c                    bending parameters (xfac,tlim,nitpb) )
c                    **NOTE that this option should not be used regularly,
c                    ** but only to check a few selected events, since
c                    ** it creates a lot of output.
c
c        line 2 - (free format)
c        nitloc - max of iterations for hypocenter location.
c        wtsp   - for hypocenter solution, weight of S-P residual 
c                 relative to P residual (ie:wtsp=1.0 gives equal wt,
c                 wtsp<1 downweights S-P)
c        eigtol - SVD cutoff in hypocentral adjustments. If smallest
c                 eigenvalue in geiger's matrix is < eigtol, the depth
c                 is not adjusted, and a message is printed.
c        rmscut - value for rms residual below which hypocentral adjustments
c                 are terminated.
c        zmin - minimum hypocenter depth
c        dxmax - maximum horizontal hypocentral adjustment allowed in
c                each hypocenter iteration.
c
c        rderr - estimate of reading error, used to estimate hypocenter error
c        ercof - for hypoinverse-like error calculations. Set > 0 and
c                < 1 if you want to include rms.res in hypocenter error
c                estimate. (sigsq=rderr**2 + ercof * rms**2)
c        line 3 - (free format)
c        nhitct - of observations for a parameter to be included
c                 in the inversion. This uses the variable khit.
c        dvpmx - maximum P-velocity adjustment allowed per iteration. 
c        dvsmx - maximum S-velocity adjustment allowed per iteration.
c        idmp -  set to 1 to recalculate the damping value for succeeding
c                iterations.  set to 0 to have constant damping
c        vdamp - damping parameter used in velocity inversion.
c                 vdamp(1)=damping for p-velocity
c                 vdamp(2)=damping for Vp/Vs
c                 vdamp(3)=damping for station delays
c        stepl - (km) used for calculation of partial derivatives along
c                the raypath.
c
c        line 4 - (free format)
c        ires - set to 1 to compute the resolution and print diagonal elements,  
c             also prints out 1/(diagonal covariance)
c           2 to print full resolution to file 17 (recomputed on each
c              iteration.)
c           3 to calculate full resolution on 1st iteration only.
c           0 no resolution calculations.
c        i3d - flag for using psuedo-bending:
c              0=no psuedo-bending
c              1=use in forward problem to compute velocity partial derivatives
c              2=also use in earthquake location subroutine
c              3=psuedo-bending; also use less curvature for initial arcuate
c                rays below "moho".  Assumes last z grid (k=nz-1) is "moho".
c              4=use paraxial raytracer (under construction - fh, 980331)
c                therefore file RAYTRAC must be supplied
c        nitmax - max of iterations of the velocity inversion-hypocenter
c                 relocation loop.
c                 For locations only, set nitmax=0.
c                 For creating synthetic data, set nitmax= -1, and
c                   use file007 for hypocenters and stations.
c                   ***NOTE that synthetic data option has not been ***
c                   ***changed for S-P.  It computes S travel-times. ***
c        snrmct - cutoff value for solution norm. Program will stop
c                 iterating if the solution norm is less than snrmct.
c        ihomo - flag for using 1-d starting model (1=yes,0=no)
c               if flagged, on first ihomo iterations do 2-d art
c        rmstop - overall rms residual for termination of program.
c         
c        ifixl - number of velocity inversion steps to keep hypocenters
c                fixed at start of progressive inversion
c
c        line 5 - (free format)
c        delt1, delt2 - distance weighting factors. The weight is 1 for
c               x<delt1, but tapers linearly to 0 between delt1 and delt2.
c                Note that this is epicentral distance in simulps13
c        res1,res2 - same pattern as above, but for residual weighting.
c
c        line 6 - (free format)
c        ndip - of rotation angles of the plane of the ray, which will
c               be computed in the exhaustive search for the fastest time.
c        iskip - of rotation angles which will be skipped.
c                ndip=9, iskip=3 will give a vertical plane, and 2 swung
c                at angles of 22.5 degrees on each side.
c                ndip=9, iskip=4 will give only the vertical plane, and should
c                be used for 1 dim models, to save computer time.
c           *** Most of the computer time is spent in the raytracing. Careful
c               selection of the starting model and use of iskip can save
c               a lot.
c         scale1  - set scale1 to the step length for the travel
c                           time computation. Set no larger than the grid
c                           spacing.
c         scale2 - scale for the number of paths tried in the raytracing.
c                  Cliff uses a value of 1, and this seems ok, but would
c                  need to be tested in detail. If scale2 is smaller, the
c                  number of paths increases, and the computation time also
c                  goes up.
c
c
c        line 7 - (free format)
c        xfac - Convergence enhancement factor for psuedo-bending
c               Cliff suggests 1.2 to 1.5
c        tlim - Cutoff value for travel time difference to terminate
c               iteration. (0.0005 to 0.002 s)
c               Make lower tlim for t* (iuseq=1) than for t.
c        nitpb - maximum permitted iterations for psuedo-bending (5 to 10)
c                nitpb(1) for shorter raypaths
c                nitpb(2) for raypaths > delt1
c              
c        line 8 - (free format)
c        iusep - flag to tell whether or not to use P arrivals (and invert for
c                P velocities).  0 = no, 1 = yes.
c        iuses - flag to tell whether or not to use S arrivals (and invert for
c                S velocities).  0 = no, 1 = yes.
c        invdel - flag to control inclusion of station delays in the inversion.
c                 =0 to not include stn delays.
c                 =1 to include stn delays.
c
c        line 9 - (free format)
c        iuseq - flag to tell whether inversion is for velocity or Q
c                0 = velocity,  1 = Q 
c        dqmax - maximum Q adjustment allowed per iteration
c        qdamp - damping parameter for Q inversion
c                actually damps solution for Q*V
c
c     File 02 - Station Data  (read in subroutine INPUT2)
c        line 1 - (free format)
c         ltdo, oltm, lndo, olnm, rota, nzco - sets origin and rotation of the
c              coordinate system. Choose the origin at the lower right corner
c              of the region.
c              Y points in North direction.
c              X points West.
c              rota - angle of rotation counterclockwise (degrees). This is
c                     used to rotate the entire coordinate system.
c              nzco - For New Zealand (lat S and long E) make nzco=1,
c                     For US (lat N and long W) make nzco=0
c                     Note that this is only used in AZTOA as a quick
c                     fix for getting azimuths correct
c
c         line 2 - (free format)
c         nsts - number of stations in station list to follow.
c
c         station list: see documentation.
c
c     File 03 - velocity model (read in subroutine INPUT3)
c         This contains the number of nodes in the x, y , and z directions.
c         Then the velocity model. See the documentation for a complete
c         description.  In simulps12 this has Vp, followed by Vp/Vs.
c         For iuseq=1, has Vp, followed by Qp.
c         Specify which nodes (if any) you want to fix velocity for:
c           2  3  4   means x2(2nd column), y3(3rd row), z4(4th layer)
c
c     File 04 - travel-time data for earthquakes. (read in subroutine INPUT4)
c         Note that S data should be made into S-P data.
c         Use program 'convert6' to convert hypo71 summary and phase data
c         to this format.
c         (should still work with old 'convert3' files also, as well as
c         Michelini's format)
c
cfhRT
c     File 55 - control file for paraxial raytracing, read in subroutine input5
c         File 'RAYTRAC' is only read if i3d = 4
c         line 1 - free format
c            iheter, epsob, epsca, ides, ampr, iterrai
c              iheter: max. number of iterations to reach the station from a source
c                      with the raytracing (Jean suggests 10 - 30)
c                      iheter = 3, 13, 23: calculate synthetics (together with nitmax = -1)
c                         use 3 for very smooth, 13 for medium and 23 for very heterogeneous models
c                         (test it out)
c              epsobs: precision in km to reach the station within iheter iterations.
c                      Jean suggests 0.05 for local eq. data. If distance between ray surfacing
c                      point and station is less than epsobs before iheter iterations are done,
c                      iterating is stopped (successful raytracing)
c              epsca:  tolerance in km to keep the result after iheter iterations.
c                      Jean suggests 2.0. If after iheter iterations, the ray still is more than
c                      epsobs away from the station, but less than epsca, the raytracing is
c                      taken to be successful (travel time is anyway adjusted by continuation)
c              ides:   ides .eq. 0: brief output to file rkplog
c                      ides .eq. 1: descriptive output to file rkplog 
c                      ides .eq. 2: VERY detailed output to file rkplog - huge file, only use for 
c                                   testing with a few events!!!
c              ampr:   parameter for the integration step along the ray (tau in the hamiltonian).
c                      Units distance * velocity [km**2/s]. Jean suggests ampr = 1 for local data.
c              iterrai: flag to read initial angles (1= calculate angles on first iteration, 
c                       2= angles have to exist in file ./angle/angle.ne (ne: number of eq)).
c
c         line 2 - free format
c            dxrt, dyrt, dzrt
c                       grid spacing for the ray-tracing grid (linearly interpolated down 
c                       from the input velocity grid). In units of bld (whole km or .1 km)     
c
c
c     File 07 - travel-time data for shots
c         For iuseq=1, read in t* data as shots.
c         (For iuseq=1, neqs and nbls should be zero.)
c
c     File 08 - travel-time data for blasts
c
c  OUTPUT FILES:
c     File 16 - printed output, send to line printer
c
c     File 12 - hypocenters from each iteration, hypoinverse format with error ellipse
c
c     File 13 - Final hypocenters in hypo71 summary card format
c     File 15 - Written when kout3=1, A separate file for each event
c         containing all the raypath points, useful for plotting
c
c     File 17 - Full resolution matrix.  Created if ires.ge.2.
c
c     File 18 - Pointer from full nodes to inversion nodes.
c          Output when fixed nodes are used.
c     File 19 - Written when kout3=1, Contains the difference in travel 
c          time between ART and psuedo-bending
c     File 22 - Station data with new P and S delays.  This
c          can be used as input to future runs. NOT created if
c          invdel=0 (no station delays inverted for).
c
c     File 20 - Station residual output (created for kout2=0 or 1)
c
c     File 23 - Final velocity model.  This can be used as input
c          to future runs.
c
c     File 24 - Earthquake travel-time data for new hypocenters.
c          This can be used as input in future runs.
c
c     File 25 - New velocities in station format so can be plotted 
c          with qplot.
c
c     File 26 - List of observations that used maximum allowed 
c          psuedo-bending iterations (nitpb)
c
c     File 28 - Blast travel-time data with new origin times.
c
c     File 34 - Output file similar to HYPO71 listing file, which can
c          be used as input to FPFIT fault plane solution program.
c          Only for earthquakes and shots since called from Loceqk.
c          (Note that DIST is the hypocentral distance, whereas HYPO71
c          outputs the epicentral distance.)  Written on last (nitmax) iteration.
c
c     File 36 - Summary output file that contains key solution
c          statistics.  This is useful when doing numerous
c          damping runs and wanting to compare variances.
c
c     File 45 - 1/diagonal elements of covariance matrix.  In same
c          format as velocity model input.  Created if ires.ge.2.
c
c     
c     file 80 - output of rays for which RKP raytracing failed (ART-PB ray used then)
c    
c     file 121 - different files for shooting angles for paraxial raytracing
c
cfhRTend
c
c   HINTS:
c
c       1. Invert for a one-dimensional model first. Use nx=3, ny=3, nz=
c          whatever you want in the final model. Set ndip=9, iskip=4.
c
c          Or, better yet, use VELEST, locate your events with the VELEST
c          model, then fix the earthquake locations on the 1st iteration.
c
c       2. Run the inversion on calculated data. This will tell you what
c          kind of averaging is inherent in the inversion process, since
c          the result will probably be different from your original input
c          model.
c
c       3. When the program crashes due to divide by 0, etc, the cause can
c          most often be traced to errors in the setup of the velocity
c          model. No part of a ray must reach over halfway to an outer node.
c          Try increasing the distance of the outer node.
c
c
c  ARRAY DIMENSIONING:
c      11-mar-94 These notes are now in the simulps_common.inc file.
c      PLEASE READ that file to understand the array dimensions
c      and how many parameters you are allowed to invert for.
c
c
c  declaration statements:
      character*26 dash
cfa   character*8 tm
      Character DateToday*11,TimeNow*8  ! PC version: FA.

c
c  common block variables:
      include 'simulps_common.inc'
      include 'rayparax_common.inc'
cfhRT
c
      dash='--------------------------'
c
cek        open(unit=16,status='new',recl=132,file='output')
cfa     open(unit=16,status='new',file='output.txt')
        open(unit=16,file='output.txt')
        rewind (16)
cek        open(unit=36,carriagecontrol='list',status='new')
cfa     open(unit=36,status='new',file='itersum')
        open(unit=36,file='itersum.txt')
        rewind (36)
cfhdmep
cfa     open(unit=18,file='nodes.out',status='new')
        open(unit=18,file='nodes.out')
c    file to check pseudo-bending
cek        open(unit=26,status='new',recl=132,file='pseudo')
cfa     open(unit=26,status='new',file='pseudo.txt')
        open(unit=26,file='pseudo.txt')
cfa     
        rewind (18)  !!!
        rewind (26)
c
c  input routines
c
c  input control parameters
cfa   open(unit=01,file='CNTL',status='old',form='formatted',
cfa  2     action = 'read')
      open(unit=01,file='CNTL')
      rewind (01)
      call input1
      close(01)
c  open resolution file if ires=2
cfa   if(ires.ge.2) open(unit=17,file='resol.out',
cfa  2 status='new')
      if(ires.ge.2) open(unit=17,file='resol.out')
c  open residual output file if kout2=0,1
cfa   if(kout2.le.1) open(unit=20,file='residuals.txt',
cfa  2 status='new')
      if(kout2.le.1) open(unit=20,file='residuals.txt')
c  open files to write raypath points, tt differences to if kout3=1
      if(kout3.eq.0) goto 70
cfa   open(unit=19,file='f19.txt',
cfa  2 status='new')
      open(unit=19,file='f19.txt')
      if(i3d.eq.4) then
        write(19,1902)
      else
        write(19,1901)
      endif
 1901 format('  ne  stn  delta  fstime  ttime   tdif')
 1902 format('  ne  stn  delta  pbtime  ttime   distance[km]')
c
   70 continue 
cfhRT open and read RAYTRAC and open rkpfail for paraxial raytracing
      if(i3d.eq.4) then
         call input5
cfa      open(unit=80,file='rkpfail.txt',form='formatted',status='new')
         open(unit=80,file='rkpfail.txt',form='formatted')
         write(80,'(a)')'RKP-raytracing failed for the following
     .  stn-evt pairs'
      endif
cfhRT end
c
c  initializations
      call strt(0)
      nit=0
c  input station list, set up center of coordinates, calculate
c  cartesian coordinates
cfa   open(unit=02,status='old',form='formatted',file='stns',
cfa  2     action = 'read')      
      open(unit=02,file='stns')      
      rewind (02)
      call input2
      close(02)
c  input medium model
cfa   open(unit=03,status='old',form='formatted',file='MOD',
cfa  2     action = 'read')      
      open(unit=03,file='MOD')      
      rewind (03)
      call input3
      close(03)
      if(neqs.eq.0) goto 71
cfa   open(unit=04,status='old',form='formatted',file='EQKS',
cfa  2      action = 'read')      
      open(unit=04,file='EQKS')      
   71 if(nsht.eq.0) goto 72
cfa   open(unit=07,status='old',form='formatted',file='SHOT',
cfa  2      action = 'read')      
      open(unit=07,file='SHOT')      
   72 if(nbls.eq.0) goto 73
cfa   open(unit=08,status='old',form='formatted',file='BLAS',
cfa  2      action = 'read')      
      open(unit=08,file='BLAS')      
c
   73 if(kout.lt.3) goto 74
cfa   open(unit=34,file='hypo71list.txt',status='new')
      open(unit=34,file='hypo71list.txt')
      rewind(34)
      if(kout.lt.4) goto 74
c       open file for hypocenter summary card output on each iteration
cfa   open(unit=12,file='summary.txt',status='new')
      open(unit=12,file='summary.txt')
      rewind (12)
c
c
   74 istop=0
      istop1=0
c
      neb=neqs+nbls
      nevt=neb+nsht
c
c  iterative inversion loop
c
    1 continue
c
c
      netemp=neqs
      nbtemp=nbls
c  if flagged, on first ifixl iterations, treat all quakes as blasts
      if (ifixl.le.nit) go to 110
      nbls=nbls+neqs
      neqs=0
  110 continue
c
      istemp=iskip
      ndtemp=ndip
c
cfhRT the following makes only sense when not using paraxial raytracer
      if(i3d.ne.4) then
c  if flagged, on first ihomo iterations do 2-d art
      if (ihomo.le.nit) go to 111
      iskip = 0
      ndip = 1
  111 continue
      endif
cfh end
c
cfhRT - now don't use a squared slowness grid but interpolate
cfh     on the velocity grid to get velocities and derivatives
cfh     from there and then convert those to squared slowness and
cfh     derivatives when needed
cfh    still need to build an evenly spaced velocity grid for
cfh    the interpolation to work...
cfhRT for paraxial raytracer we have to create the raytracing grid
cfhRT (regular, with squared slowness values on nodes) from the
cfhRT inversion grid before each new iteration
      if(i3d.eq.4) then
cfa     open(unit=61,file='rkplog.txt',status='new')
        open(unit=61,file='rkplog.txt')
        write(61,*) 'calling bldtrgrd'
        call bldrtgrd
cfh   on first iter build lookup table for rt-grid so we can use intmaprt 
cfh    to get indices for coordinate kms
        if(nit.eq.0) call bldmaprt
        write(80,*)'iteration: ',nit
      endif
cfh end 
c
c
      write(16,1005) nit
 1005 format(///,' iteration step',i3,'; hypocenter adjustments')
cfh check run time
cfa      call time(tm)
cfa      write(16,1006) tm
      Call DateTime(DateToday,TimeNow)
      write(16,1006) TimeNow   ! PC version: FA.
 1006 format('time: ',a8)
      if (neqs.eq.0) go to 11
c
c  loop over all earthquakes
      ne=1
    9 continue
c  input observations of event at first iteration
      if((nit.eq.0).or.(kout2.eq.0)) write(16,130)
     2 dash,dash,dash,dash,dash
  130 format(1x,5a26)
      if (nit.eq.0) call input4(ne,4)
      if(ne.gt.neqs) goto 10
c  locate individual earthquake to reduce residuals
      call loceqk(ne,nit,nwr)
      if((nit.eq.0).and.(nwr.lt.4)) goto 9
      if((nit.gt.0).and.(nwr.lt.4)) goto 9998
c  skip if only locating
      if((nitmax.le.0).and.(kout3.eq.0).and.(kout2.gt.1)) goto 208
c  do forward problem and calculate partial derivatives
      call forwrd(ne)
      if((nitmax.le.0).and.(kout2.gt.1)) goto 208
c  perform parameter separation
      call parsep(ne,nwr)
c  if last iteration, write out station residuals
  208 if((nit.eq.nitmax).and.(kout2.lt.2)) call outres(ne)
      if((nit.eq.nitmax).and.(kout.ge.3)) call outlis(ne)
c  skip this event if not enough good readings,
c        stop if not first iteration
      if((nit.gt.0).and.(nwr.lt.4)) goto 9998
      if(nwr.ge.4) ne=ne+1
      if(ne.le.neqs) goto 9
   10 continue
c
c  loop over all blasts
   11 if(nbls.eq.0) goto 15
      write(16,1613) nbls
 1613 format(/,2x,'Following ',i5,' Events are Blasts with Unknown',
     2 ' Origin Time')
      nb=1
   12 ne=nb+neqs
   13 infile=8
      if((nit.eq.0).or.(kout2.eq.0)) write(16,130)
     2 dash,dash,dash,dash,dash
      if((ifixl.gt.nit).and.(ne.le.netemp)) infile=4
      if(nit.eq.0) then
        call input4(ne,infile)
        if(ne.gt.neb) goto 14
        if((ne.gt.netemp).and.(infile.eq.4)) goto 13
      endif
      call loceqk(ne,nit,nwr)
      if((nitmax.le.0).and.(kout3.eq.0).and.(kout2.gt.1)) goto 308
      call forwrd(ne)
      if((nitmax.le.0).and.(kout2.gt.1)) goto 308
      call parsep(ne,nwr)
  308 if((nit.eq.nitmax).and.(kout2.lt.2)) call outres(ne)
      if((nit.eq.nitmax).and.(kout.ge.3)) call outlis(ne)
      if(nwr.ge.4) nb=nb+1
      if(nb.le.nbls) goto 12
   14 continue
c
c  loop over all shots
   15 continue
      if (nsht.eq.0) go to 25
      write(16,1614) nsht
 1614 format(/,2x,'Following ',i5,' Events are Shots with Known',
     2 ' Origin Time')
      ns=1
c  input observations of shot
   16 ne=ns+neqs+nbls
      if((nit.eq.0).or.(kout2.eq.0)) write(16,130)
     2 dash,dash,dash,dash,dash
      if (nit.gt.0) goto 17
      call input4(ne,7)
c  do forward problem and calculate partial derivatives
      if(nit.eq.0) then
        jfl=0
      else
        jfl=2
        if(ihomo.eq.nit) jfl=1
      end if
   17 call forwrd(ne)
c  add medium derivatives from shot to medium matrix
      call medder(ne)
c** Change for synthetic data, nitmax= -1
      if((nitmax.le.0).and.(kout2.gt.1)) goto 408
c  put derivatives into g matrix
      call parsep(ne,nwr)
  408 ns=ns+1
      if((nit.eq.nitmax).and.(kout2.lt.2)) call outres(ne)
      if(ns.le.nsht) goto 16
   20 continue
c
   25 continue
c** Change for synthetic data, nitmax= -1
      if(nitmax.le.0) goto 9999
c  continue iterating or terminate?
      call decide(istop,nit)
c     if (nit.gt.0) call decide(istop)
        if(istop1.gt.2) goto 9999  !If backed-up twice, don't adjust again,end
        if(istop.eq.2) goto 28     !If variance ratio bad, backup
        if(istop1.eq.2) goto 9999  !If backed-up once and now okay, end
      if(nit.ge.nitmax) goto 9999  !As in CT version, do hyp last
c
 900  continue
      nit=nit+1
      write(16,1015) nit
 1015 format(///,' iteration step',i3,'; simultaneous inversion')
cfh check run time
cfa   call time(tm)
cfa   write(16,1006) tm
      Call DateTime(DateToday,TimeNow)
      write(16,1006) TimeNow   ! PC version: FA.
c  invert for velocity model adjustments
      call veladj(nit)
      if(nit.eq.99) goto 9999
c  output results of iteration
      call outadj(nit,istop,istop1)
c  compute resolution?
      if(ires.eq.0) goto 901
      if((ires.ne.3).or.(nit.eq.1)) call rescov
  901 if(istop.eq.1) go to 9999
      if(istop.eq.0) goto 34
c
   28   write(16,902)
  902   format(' ******* Variance Ratio is less than Critical Ratio',
     2 /,12x,' ******* Backup Parameters Halfway *******',/)
        call velbku(istop1)
        istop1=istop1+istop
        call outadj(nit,istop,istop1)
c
   34 continue
c
c  restore neqs,nbls,iskip,ndip
      neqs=netemp
      nbls=nbtemp
      iskip=istemp
      ndip=ndtemp
c
      call strt(nit)
      rewind(26)
c
cfh this should be done right after loceqk.
cfh      if (i3d.eq.4) iterrai=2        ! use anlge files for next run
      go to 1
c
 9998 continue
      write(16,1620)
 1620 format(//,'  ********** STOP **********')
 9999 continue
c
      call outend
c
      close(04)
      close(07)
      close(12)
      close(16)
      close(26)
      close(36)
      if(ires.eq.2) close(17)
      if(kout3.eq.0) stop
      close(19)
      stop
c***** end of main program *****
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine avsd(cnull,x,nx,sd,av,devtot)
c  program to find the average and standard deviation of a 
c  list of numbers.  Those with value cnull are not included.
c
      real x(5000)
      sum=0
      i=0
      do 50 ix=1,nx
         if(x(ix).eq.cnull) goto 50
         sum=sum+x(ix)
         i=i+1
   50 continue
      nx1=nx
      nx=i
      if(nx.eq.0) goto 800
      av=sum/nx
      devtot=0
      do 260 i=1,nx1
         if(x(i).eq.cnull) goto 260 
         dev=x(i)-av
         devtot=devtot+dev*dev
  260 continue
      sd=sqrt(devtot/nx)
c     write(61,620) nx,av,sd
  620 format(' nx=',i5,', average=',e11.4,', sd=',e11.4)
      return
  800 continue
      sd=0.00
      devtot=0.00
      av=0.00
      nx=0.00
      return
c ***** end of subroutine avsd *****
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine aztoa(x1,x2,y1,y2,z1,z2,xr,yr,azim,tkofan)
c  this subroutine computes the azimuth and take-off-angle
c  for an individual observation
c  (called from Loceqk)
c  pr is station, p1 and p2 are hypocenter and adjoining point
c  on the raypath
c
      common/shortd/ xltkm,xlnkm,rota,nzco,xlt,xln,snr,csr
      parameter (drad=1.7453292d-02)
      parameter (pi=3.1415926536)
      parameter (twopi=6.2831853072)
c
c  Azimuth
cfh quick fix to compute azimuth also on first 2 raypoints
cfh  maybe check whether to use a certain minimum raylength at
cfh  the start of the ray (to average out ray-tracer fluctuations, if there are)
cfh      xd=xr-x1
cfh      yd=yr-y1
      xd=x2-x1
      yd=y2-y1
      xda=abs(xd)
      yda=abs(yd)
      phi=atan(xda/yda)
c  compute correct azimuth depending on quadrant
      if(xd.ge.0.0) then
        if(yd.ge.0.0) then
          theta=twopi-phi
        else
         theta=pi+phi
        endif
      else
        if(yd.ge.0.0) then
          theta=phi
        else
          theta=pi-phi
        endif
      endif
c  rotate back to real north, convert to degrees
      azim=(theta-rota)/drad
c  22aug95 quick change to get correct azimuths for New Zealand
c          S lat and E long
      if(nzco.eq.1) azim=azim+180
      if(azim.gt.360.0) azim=azim-360.0
      if(azim.lt.0.0) azim=azim+360.0
c
c  Take-off-angle
      xd=x2-x1
      yd=y2-y1
      r=sqrt(xd*xd+yd*yd)
      zd=z2-z1
      zda=abs(zd)
      phi=atan(r/zda)
      if(zd.lt.0.0) phi=pi-phi
      tkofan=phi/drad
c
      return
c ***** end of subroutine aztoa *****
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine bend(isp,xfac)
c*****this routine perturbs the initial path in the direction
c      of the normal to the ray path tangent at each point
c      by the optimal distance r
c
      common/pathm/x(130),y(130),z(130),v(130),vq(130),tra,qtra,n,nn
      common/temp/xtemp(130),ytemp(130),ztemp(130),rtemp(130),ttemp(130)
c
c ***
      xtemp(1)=x(1)
      ytemp(1)=y(1)
      ztemp(1)=z(1)
c ***
      do 200 k=2,nn
c
         kk=k-1
         kkk=k+1
c
c*****compute the normal direction of maximum gradient of velocity
c
         dx=x(kkk)-xtemp(kk)
         dy=y(kkk)-ytemp(kk)
         dz=z(kkk)-ztemp(kk)
         dn=dx*dx+dy*dy+dz*dz
         ddn=sqrt(dn)
         rdx=dx/ddn
         rdy=dy/ddn
         rdz=dz/ddn
c
         xk=0.5*dx+xtemp(kk)
         yk=0.5*dy+ytemp(kk)
         zk=0.5*dz+ztemp(kk)
c ***
         call vel3(isp,xk,yk,zk,vk)
         call veld(isp,xk,yk,zk,vx,vy,vz)
c
c ***
         vrd=vx*rdx+vy*rdy+vz*rdz
         rvx=vx-vrd*rdx
         rvy=vy-vrd*rdy
         rvz=vz-vrd*rdz
c
         rvs=sqrt(rvx*rvx+rvy*rvy+rvz*rvz)
         if(rvs.eq.0.0) goto 200
         rvx=rvx/rvs
         rvy=rvy/rvs
         rvz=rvz/rvs
c
c*****compute the optimal distance r
         rcur=vk/rvs
         rtemp(k)=rcur-sqrt(rcur*rcur-0.25*dn)
c
c*****compute the new points and distance of perturbations
c
         xxk=xk+rvx*rtemp(k)
         yyk=yk+rvy*rtemp(k)
         zzk=zk+rvz*rtemp(k)
c
c  convergence enhancement
         xxk=xfac*(xxk-x(k))+x(k)
         yyk=xfac*(yyk-y(k))+y(k)
         zzk=xfac*(zzk-z(k))+z(k)
c
         ttemp(k)=sqrt((x(k)-xxk)**2+(y(k)-yyk)**2+(z(k)-zzk)**2)
         xtemp(k)=xxk
         ytemp(k)=yyk
         ztemp(k)=zzk
         call vel3(isp,xxk,yyk,zzk,vk)
         v(k)=vk
         call vel3(1,xxk,yyk,zzk,vkk)
         vq(k)=vkk
c ***
200   continue
c
      return
c ***** end of subroutine bend *****
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine bldmap
c  common block variables:
      include 'simulps_common.inc'
c
c     array size limits
c     ixkms=iykms=izkms=1500
c
c     write(61,400)
c 400 format(' subroutine bldmap')
      xl=bld-xn(1)
      ixmax=(xn(nx)+xl)/bld
      yl=bld-yn(1)
      iymax=(yn(ny)+yl)/bld
      zl=bld-zn(1)
      izmax=(zn(nz)+zl)/bld
c     write(61,402)ixmax,iymax,izmax
c 402 format(' array sizes: ',3i5)
c
c  Check for array size overflow
      if(ixmax.gt.ixkms.or.iymax.gt.iykms.or.izmax.gt.izkms)goto 330
      ix=1
      do 10 i=1,ixmax
c
         ix1=ix+1
c
         xnow=float(i)*bld-xl
         if (xnow.ge.xn(ix1)) ix=ix1
c
         ixloc(i)=ix
   10 continue
c  Fill remainder of array with zeroes.
cfh ?? why does he start from i=ixmax (this is the last element above) 
cfh      do 12 i=ixmax,ixkms
      do 12 i=ixmax+1,ixkms
         ixloc(i)=0
   12 continue
c
c
      iy=1
      do 15 i=1,iymax
c
         iy1=iy+1
c
         ynow=float(i)*bld-yl
         if (ynow.ge.yn(iy1)) iy=iy1
c
         iyloc(i)=iy
   15 continue
c
c  Fill rest of array with zeroes.
cfh      do 17 i=iymax,iykms
      do 17 i=iymax+1,iykms
         iyloc(i)=0
 17   continue
c
      iz=1
      do 20 i=1,izmax
c
         iz1=iz+1
c
         znow=float(i)*bld-zl
         if (znow.ge.zn(iz1)) iz=iz1
c
         izloc(i)=iz
   20 continue
c
c  Fill remainder of array with zeroes.
cfh      do 22 i=izmax,izkms
      do 22 i=izmax+1,izkms
         izloc(i)=0
  22  continue
      return
 330   continue
      write(16,331)ixkms,iykms,izkms
 331  format(' ***** error in array size in common/locate/',/,
     *' maximum map dimensions (km)=',/,' x=',i5,' y=',i5,' z=',i5)
      write(16,332)ixmax,iymax,izmax
  332 format(' Actual map size (km): ',/,' x=',i5,' y=',i5,' z=',i5)
      stop
c***** end of subroutine bldmap *****
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine cmpdpv(xe,ye,ze,xr,yr,zr,scale2,ndip,dipvec)
c
c  parameters
      real xe,ye,ze,xr,yr,zr,scale2,dipvec(3,9)
c
      integer ndip
c  local variables
      real dx,dy,dz,xh1,yh1,zh1,xh2,yh2,zh2,size,xv,yv,zv,
     *     rescal,x451,y451,z451,x452,y452,z452
c
      integer nv
c
      dx=xr-xe
      dy=yr-ye
      dz=zr-ze
c
c  near-vertical vector
      xv=-dx*dz
      yv=-dy*dz
      zv=dx*dx+dy*dy
c  rescale vector to length scale2
      size=sqrt(xv*xv+yv*yv+zv*zv)
      rescal=scale2/size
c
      xv=xv*rescal
      yv=yv*rescal
      zv=zv*rescal
c
c  store this vector
      nv=(ndip+1)/2
      dipvec(1,nv)=xv
      dipvec(2,nv)=yv
      dipvec(3,nv)=zv
c
      if (ndip.eq.1) return
c
c  horizontal vectors
      xh1=dy
      yh1=-dx
      zh1=0.0
      xh2=-dy
      yh2=dx
      zh2=0.0
c  rescale the vectors to length scale2
      size=sqrt(xh1*xh1+yh1*yh1)
      rescal=scale2/size
c
      xh1=xh1*rescal
      yh1=yh1*rescal
      xh2=xh2*rescal
      yh2=yh2*rescal
c
c  store these two vectors
      dipvec(1,1)=xh1
      dipvec(2,1)=yh1
      dipvec(3,1)=zh1
c
      dipvec(1,ndip)=xh2
      dipvec(2,ndip)=yh2
      dipvec(3,ndip)=zh2
c
      if (ndip.eq.3) return
c
c  determine two 45 degree dip vectors
      rescal=0.7071068
c
      n1=(1+nv)/2
      n2=(nv+ndip)/2
c
      x451=(xh1+xv)*rescal
      y451=(yh1+yv)*rescal
      z451=(zh1+zv)*rescal
c
      x452=(xh2+xv)*rescal
      y452=(yh2+yv)*rescal
      z452=(zh2+zv)*rescal
c
      dipvec(1,n1)=x451
      dipvec(2,n1)=y451
      dipvec(3,n1)=z451
c
      dipvec(1,n2)=x452
      dipvec(2,n2)=y452
      dipvec(3,n2)=z452
c
      if (ndip.eq.5) return
c
c  determine four 22.5 degree dip vectors
      rescal=0.5411961
c
      dipvec(1,2)=(xh1+x451)*rescal
      dipvec(2,2)=(yh1+y451)*rescal
      dipvec(3,2)=(zh1+z451)*rescal
c
      dipvec(1,4)=(x451+xv)*rescal
      dipvec(2,4)=(y451+yv)*rescal
      dipvec(3,4)=(z451+zv)*rescal
c
      dipvec(1,6)=(xv+x452)*rescal
      dipvec(2,6)=(yv+y452)*rescal
      dipvec(3,6)=(zv+z452)*rescal
c
      dipvec(1,8)=(x452+xh2)*rescal
      dipvec(2,8)=(y452+yh2)*rescal
      dipvec(3,8)=(z452+zh2)*rescal
c
c***** end of subroutine cmpdpv *****
      return
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine cmpdsv(ndip,iskip,ns,dipvec,disvec)
c
c  parameters
      real dipvec(3,9),disvec(390,9)
c
      integer ndip,iskip,ns
c  local variables
      real darc(129)
c
      integer inc,narc,ndp,np,n,nd
c  coefficients of standard arc
c
       data darc/0.,0.0342346,0.0676860,0.1003707,0.1323047,0.1635029,
     *.1939796,.2237485,.2528226,.2812141,.3089350,.3359963,
     *.3624091,.3881833,.4133289,.4378553,.4617713,.4850857,
     *.5078065,.5299417,.5514988,.5724850,.5929070,.6127717,
     *.6320853,.6508538,.6690831,.6867787,.7039459,.7205898,
     *.7367154,.7523272,.7674298,.7820274,.7961241,.8097238,
     *.8228301,.8354468,.8475771,.8592244,.8703916,.8810817,
     *.8912975,.9010416,.9103164,.9191245,.9274679,.9353487,
     *.9427691,.9497307,.9562353,.9622845,.9678797,.9730224,
     *.9777138,.9819550,.9857470,.9890908,.9919872,.9944367,.9964401,
     *.9979979,.9991102,.9997776,1.0000000,.9997776,.9991102,.9979979,
     *.9964401,.9944367,.9919872,.9890908,.9857470,.9819550,.9777138,
     *.9730224,.9678797,.9622845,.9562353,.9497307,.9427691,.9353487,
     *.9274679,.9191245,.9103164,.9010416,.8912975,.8810817,.8703916,
     *.8592244,.8475771,.8354468,.8228301,.8097238,.7961241,.7820274,
     *.7674298,.7523272,.7367154,.7205898,.7039459,.6867787,.6690831,
     *.6508538,.6320853,.6127717,.5929070,.5724850,.5514988,.5299417,
     *.5078065,.4850857,.4617713,.4378553,.4133289,.3881833,.3624091,
     *.3359963,.3089350,.2812141,.2528226,.2237485,.1939796,.1635029,
     *.1323047,.1003707,.0676860,.0342346,0.0/
      inc=128/ns
        ndip1=1+iskip
      ndip2=ndip-iskip
c
c  loop over dips
      do 30 ndp=ndip1,ndip2
         narc=1
         nd=3
c  loop over points on the path (skip first and last)
         do 20 np=2,ns
            narc=narc+inc
c
c  loop over x,y,z
            do 10 n=1,3
               nd=nd+1
               disvec(nd,ndp)=darc(narc)*dipvec(n,ndp)
c
   10       continue
   20    continue
   30 continue
c
c***** end of subroutine cmpdsv *****
      return
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine cmpsep(path,pthsep,ns)
c
c  parameters
      real path(390),pthsep(130)
c
      integer ns
c  local variables
      integer nx,ny,nz,nx1,ny1,nz1
c
      nx=-2
c  loop over pairs of points in one set of stored vectors
      do 10 n=1,ns
         nx=nx+3
         ny=nx+1
         nz=nx+2
         nx1=nx+3
         ny1=nx+4
         nz1=nx+5
c
         pthsep(n)=sqrt((path(nx1)-path(nx))**2
     *              +(path(ny1)-path(ny))**2
     *              +(path(nz1)-path(nz))**2)
c
   10 continue
c
c***** end of subroutine cmpsep *****
      return
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine curvdr(isp,nc,ndp,dtt,tt1,tt2,disvec,pthsep,
     *  strpth,trpth1)
c  This subroutine computes the difference in time between the nc
c  and the nc+1 curve with dip = ndp. dtt is the tt difference,
c  tt1 is the tt of the nc curve, and tt2 is the tt of the nc+1 curve.
      dimension disvec(390,9),pthsep(130),strpth(390),trpth1(390)
      common/raytr/trpath(390,9),npt,ns
      ncv=nc
      call curvtm(isp,ncv,ndp,tt,disvec,pthsep,strpth,trpth1)
      tt1=tt
      nc1=nc+1
      call curvtm(isp,nc1,ndp,tt,disvec,pthsep,strpth,trpth1)
      tt2=tt
      dtt=tt2-tt1
      return
c***** end of subroutine curvdr *****
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine curvtm(isp,nc,ndp,ttm,disvec,pthsep,strpth,trpth1)
c  This computes the travel time for curve nc at dip ndp. It is
c  used in the faster search programmed by Prothero.
      dimension disvec(390,9),pthsep(130),strpth(390),trpth1(390)
      common/raytr/trpath(390,9),npt,ns
c  loop to determine points along one path
      npt2=npt-2
      do 45 np=1,npt2
         n1=3*np+1
         n3=n1+2
         do 44 nn=n1,n3
            trpath(nn,ndp)=nc*disvec(nn,ndp)+strpth(nn)
            trpth1(nn)=trpath(nn,ndp)
  44     continue
  45  continue
c  set up pthsep array for travel time calculations
      call cmpsep(trpth1,pthsep,ns)
c  compute travel time along the path
      call ttime(isp,ns,npt,trpth1,pthsep,tt)
      ttm=tt
      return
c***** end of subroutine curvtm *****
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine decide(istop,nit)
c  common block variables:
      include 'simulps_common.inc'
c
      istop=0
      rms=sqrt(ssqr/float(nobt))
      dvar=ssqrw/wnobt
      rmsw=sqrt(dvar)
      write(16,1610) rms,rmsw,dvar
      write(36,1610) rms,rmsw,dvar
 1610 format(//,' unweighted rms=',f8.5,'; weighted rms=',f8.5,
     2 ' data var.(ssqrw/wnobt ie:rms**2)=',f9.6)
      dvarp=ssqrwp/wnobtp
      if(iuses.eq.1) then
        write(16,1611) dvarp
        write(36,1611) dvarp
 1611   format(50x,'P data var.=',f9.6)
      else
        dvars=ssqrws/wnobts
        write(16,1612) dvarp,dvars
        write(36,1612) dvarp,dvars
 1612   format(50x,'P data var.=',f9.6,'  S-P data var.=',f9.6)
      endif
      if (rmsw.lt.rmstop) istop=1
c
c  f-test
      mbl0=0
c  mbl1 was wrong, did not include stations, 1-april-1983, dmep
      do 10 n=1,npar
         if(khit(n).eq.0) goto 10
         if ((hit(n).lt.hitct).or.(nfix(n).eq.1)) go to 10
         mbl0=mbl0+1
   10 continue
      ndof=nobt-4*neqs-mbl0-nbls
      wndof=wnobt-float(4*neqs+mbl0+nbls)
        write(16,1601) mbl0,mbl1
 1601 format(' subroutine decide, mbl0 = ',i6,' mbl1 = ',i6)
      var=ssqr/ndof
      varw=ssqrw/wndof
      write(16,1606) ssqr,var,ssqrw,varw
 1606 format(/,'  Unweighted:  ssqr =',f12.3,'    var. (ssqr/ndof) =',
     2  f9.6,/,
     3  '  Weighted:   ssqrw =',f12.3,'  varw.(ssqrw/wndof) =',f9.6,/)
c
      if(nit.eq.0) then 
        write(36,3606) nit,ssqrw,varw
 3606   format(' Iteration:',i3,', ssqrw =',f12.3,
     *    '  varw.(ssqrw/wndof) =',f9.6)
        return
      endif
c
      call ftest(ndof,ndof1,ratio)
c
      rat=var1/var
      ratw=varw1/varw
c      write(16,6601) nit, var,ndof,var1,ndof1,rat,ratio
 6601 format(//,' f-test iteration',i3,/,
     *  ' new variance and ndof =',f11.6,i6,
     *  /,' old variance and ndof =',f11.6,i6,
     *  /,' variance ratio and critical ratio =',2f10.3)
      write(16,1605) 
 1605 format (//,'*****WEIGHTED****** use for f-test since weighted ',
     2  'throughout inversion',/)
      iwndof=nint(wndof)
      iwndf1=nint(wndof1)
      call ftest(iwndof,iwndf1,ratio)
      write(16,6602) nit,varw,wndof,varw1,wndof1,ratw,ratio
 6602 format(//,' f-test iteration',i3,/,
     *  ' new variance and ndof =',f11.6,f7.0,
     *  /,' old variance and ndof =',f11.6,f7.0,
     *  /,' variance ratio and critical ratio =',2f10.3)
      write(36,6601) nit,varw,ndof,varw1,ndof1,ratw,ratio
c
      if (ratw.lt.ratio) istop=2
c
c***** end of subroutine decide *****
      return
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine disto(j,n,x,y)
c  this routine calculates distance of station or event
c  from given coordinate origin in terms of (possibly
c  rotated) cartesian coords x and y
c  uses short distance conversion factors from setorg
c
c  declaration statements:
      double precision drad,drlt
      parameter (drad=1.7453292d-02)
      parameter (drlt=9.9330647d-01)
c
c  common block variables:
      common/shortd/ xltkm,xlnkm,rota,nzco,xlt,xln,snr,csr
      include 'simulps_common.inc'
c
c  if j=1, calculate coords of station n w.r.t.
c  center of coords, doing rotation if any
c  otherwise, calculate coords of event n
cfhek
cek correction for S and E: minutes must also be taken as negative!
cek
      if (j.ne.1) go to 15
cek        plt=60.*ltds(n)+sltm(n)
cek        pln=60.*lnds(n)+slnm(n)
      plt=abs(60.*ltds(n)+sltm(n))
      pln=abs(60.*lnds(n)+slnm(n))
      if(ltds(n).lt.0) plt=-1.*plt
      if(lnds(n).lt.0) pln=-1.*pln
      go to 25
 15   continue
cek        plt=60.*ltde(n)+eltm(n)
cek        pln=60.*lnde(n)+elnm(n)
       plt=abs(60.*ltde(n)+eltm(n))
       pln=abs(60.*lnde(n)+elnm(n))
       if(ltde(n).lt.0) plt=-1.*plt
       if(lnde(n).lt.0) pln=-1.*pln
   25 continue
c  now convert lat and lon differences to km
      x=pln-xln
      y=plt-xlt
      xlt1=atan(drlt*tan(drad*(plt+xlt)/120.))
      x=x*xlnkm*cos(xlt1)
      y=y*xltkm
c  now do rotation
      if(rota.eq.0.0) return
      ty=csr*y+snr*x
      x=csr*x-snr*y
      y=ty
      return
c***** end of subroutine disto *****
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine fksvd (a, s, v, mmax, nmax, m, n, p, withu, withv)
c
c  common block variables:
      include 'simulps_common.inc'
c
      common/machin/ eta,tol
c
      integer    mmax, nmax, m, n, p
      real       r, w, cs, sn, tol, f, x, eps, gf, t, y
      real       eta, h, q, z
      integer    i, j, k, l, l1, n1, np
      logical    withu, withv
      double precision a(maxobs,maxobs)
      dimension        s(4), v(4,4)
      dimension as1(maxobs),as2(maxobs)
      dimension  t(maxobs)
c
c     ------------------------------------------------------------------
c
c     this is a translation of a cdc 6600 fortran program to ibm 360
c     fortran iv.  this subroutine uses short precision arithmetic.
c     a long precision version is available under the name 'dsvd'.
c
c     this subroutine replaces earlier subroutines with the same name,
c    689   6       &   &0&  s of a complex ar&thmetic program, published
c     as algorithm 358.  this current program is faster, more accurate
c     and less obscure in describing its capabilities.
c
c     original programmer=  r. c. singleton
c     360 version by=       j. g. lewis
c     last revision of this subroutine=  4 december 1973
c
c     ------------------------------------------------------------------
c
c     additional subroutine needed=  rotate
c
c     ------------------------------------------------------------------
c
c
c     this subroutine computes the singular value decomposition
c     of a real m*n matrix a, i.e. it computes matrices u, s, and v
c     such that
c
c                  a = u * s * vt ,
c     where
c              u is an m*n matrix and ut*u = i, (ut=transpose
c                                                    of u),
c              v is an n*n matrix and vt*v = i, (vt=transpose
c                                                    of v),
c        and   s is an n*n diagonal matrix.
c
c     description of parameters=
c
c     a = real array. a contains the matrix to be decomposed.
c         the original data are lost.  if withv=.true., then
c         the matrix u is computed and stored in the array a.
c
c     mmax = integer variable.  the number of rows in the
c            array a.
c
c     nmax = integer variable.  the number of rows in the
c            array v.
c
c     m,n = integer variables.  the number of rows and columns
c           in the matrix stored in a.  (ng=mg=100.  if it is
c           necessary to solve a larger problem, then the
c           amount of storage allocated to the array t must
c           be increased accordingly.)  if mlt n , then either
c           transpose the matrix a or add rows of zeros to
c           increase m to n.
c
c     p = integer variable.  if p'0, then columns n+1, . . . ,
c         n+p of a are assumed to contain the columns of an m*p
c         matrix b.  this matrix is multiplied by ut, and upon
c         exit, a contains in these same columns the n*p matrix
c         ut*b. (p'=0)
c
c     withu, withv = logical variables.  if withu=.true., then
c         the matrix u is computed and stored in the array a.
c         if withv=.true., then the matrix v is computed and
c         stored in the array v.
c
c     s = real array.  s(1), . . . , s(n) contain the diagonal
c         elements of the matrix s ordered so than s(i)>=s(i+1),
c         i=1, . . . , n-1.
c
c     v = real array.  v contains the matrix v.  if withu
c         and withv are not both =.true., then the actual
c         parameter corresponding to a and v may be the same.
c
c     this subroutine is a real version of a fortran subroutine
c     by businger and golub, algorithm 358=  singular value
c     decomposition of a complex matrix, comm. acm, v. 12,
c     no. 10, pp. 564-565 (oct. 1969).
c     with revisions by rc singleton, may 1972.
c     ------------------------------------------------------------------
c
c
c     VAX version:  machine constants calculated internally in strt
c     eta is the machine epsilon (relative accuracy)
c     tol is the smallest representable real divided by eta.
c
      np = n + p
      n1 = n + 1
c
c     householder reduction to bidiagonal form
      gf = 0.0
      eps = 0.0
      l = 1
   10 t(l) = gf
      k = l
      l = l + 1
c
c     elimination of a(i,k), i=k+1, . . . , m
      s(k) = 0.0
      z = 0.0
      do 20 i = k,m
         z = z + a(i,k)**2
   20 continue
      if (z.lt.tol) goto 50
      gf = sqrt(z)
      f = a(k,k)
      if (f.ge.0.0) gf = - gf
      s(k) = gf
      h = gf * (f - gf)
      a(k,k) = f - gf
      if (k.eq.np) goto 50
      do 45 j = l,np
         f = 0
         do 30 i = k,m
            f = f + a(i,k)*a(i,j)
   30    continue
         f = f/h
         do 40 i = k,m
            a(i,j) = a(i,j) + f*a(i,k)
   40    continue
   45 continue
c
c     elimination of a(k,j), j=k+2, . . . , n
   50 eps = amax1(eps,abs(s(k)) + abs(t(k)))
      if (k.eq.n) goto 100
      gf = 0.0
      z = 0.0
      do 60 j = l,n
         z = z + a(k,j)**2
   60 continue
      if (z.lt.tol) goto 10
      gf = sqrt(z)
      f = a(k,l)
      if (f.ge.0.0) gf = - gf
      h = gf * (f - gf)
      a(k,l) = f - gf
      do 70 j = l,n
         t(j) = a(k,j)/h
   70 continue
      do 95 i = l,m
         f = 0
         do 80 j = l,n
            f = f + a(k,j)*a(i,j)
   80    continue
      if (abs(f).lt.1.0e-25) f=0.
         do 90 j = l,n
            a(i,j) = a(i,j) + f*t(j)
   90    continue
   95 continue
c
      goto 10
c
c     tolerance for negligible elements
  100 eps = eps*eta
c
c     accumulation of transformations
      if (.not.withv) goto 160
      k = n
      goto 140
  110    if (t(l).eq.0.0) goto 140
         h = a(k,l)*t(l)
         do 135 j = l,n
            q = 0
            do 120 i = l,n
               q = q + a(k,i)*v(i,j)
  120       continue
            q = q/h
            do 130 i = l,n
               v(i,j) = v(i,j) + q*a(k,i)
  130       continue
  135    continue
  140    do 151 j = 1,n
            v(k,j) = 0
  151    continue
         v(k,k) = 1.0
         l = k
         k = k - 1
         if (k.ne.0) goto 110
c
  160 k = n
      if (.not.withu) goto 230
      gf = s(n)
      if (gf.ne.0.0) gf = 1.0/gf
      go to 210
  170    do 180 j = l,n
            a(k,j) = 0
  180    continue
         gf = s(k)
         if (gf.eq.0.0) goto 210
         h = a(k,k)*gf
         do 205 j = l,n
            q = 0
            do 190 i = l,m
               q = q + a(i,k)*a(i,j)
  190       continue
            q = q/h
            do 200 i = k,m
               a(i,j) = a(i,j) + q*a(i,k)
  200       continue
  205    continue
         gf = 1.0/gf
  210    do 220 j = k,m
            a(j,k) = a(j,k)*gf
  220    continue
         a(k,k) = a(k,k) + 1.0
         l = k
         k = k - 1
         if (k.ne.0) goto 170
c
c     qr diagonalization
      k = n
c
c     test for split
  230    l = k
  240       if (abs(t(l)).le.eps) goto 290
            l = l - 1
            if (abs(s(l)).gt.eps) goto 240
c
c     cancellation
         cs = 0.0
         sn = 1.0
         l1 = l
         l = l + 1
         do 280 i = l,k
            f = sn*t(i)
            t(i) = cs*t(i)
            if (abs(f).le.eps) goto 290
            h = s(i)
            w = sqrt(f*f + h*h)
            s(i) = w
            cs = h/w
            sn = - f/w
            do 221 ia=1,m
               as1(ia) = sngl(a(ia,l1))
               as2(ia) = sngl(a(ia,i))
  221       continue
            if (withu) call hyrot(mmax,as1, as2, cs, sn, m)
            do 222 ia=1,m
               a(ia,l1)=dble(as1(ia))
               a(ia,i)=dble(as2(ia))
  222       continue
            if (np.eq.n) goto 280
            do 270 j = n1,np
               q = a(l1,j)
               r = a(i,j)
               a(l1,j) = q*cs + r*sn
               a(i,j) = r*cs - q*sn
  270       continue
  280    continue
c
c     test for convergence
  290    w = s(k)
         if (l.eq.k) goto 360
c
c     origin shift
         x = s(l)
         y = s(k-1)
         gf = t(k-1)
         h = t(k)
         f = ((y - w)*(y + w) + (gf - h)*(gf + h))/(2.0*h*y)
         gf = sqrt(f*f + 1.0)
         if (f.lt.0.0) gf = - gf
         f = ((x - w)*(x + w) + (y/(f + gf) - h)*h)/x
c
c     qr step
         cs = 1.0
         sn = 1.0
         l1 = l + 1
         do 350 i = l1,k
            gf = t(i)
            y = s(i)
            h = sn*gf
            gf = cs*gf
            w = sqrt(h*h + f*f)
            t(i-1) = w
            cs = f/w
            sn = h/w
            f = x*cs + gf*sn
            gf = gf*cs - x*sn
            h = y*sn
            y = y*cs
            if (withv) call hyrot(nmax,v(1,i-1),v(1,i),cs,sn,n)
            w = sqrt(h*h + f*f)
            s(i-1) = w
            cs = f/w
            sn = h/w
            f = cs*gf + sn*y
            x = cs*y - sn*gf
            do 338 ia=1,m
               as1(ia) = sngl(a(ia,i-1))
               as2(ia) = sngl(a(ia,i))
  338       continue
            if (withu) call hyrot(mmax,as1, as2, cs, sn, m)
            do 339 ia=1,m
               a(ia,i-1)=dble(as1(ia))
               a(ia,i)=dble(as2(ia))
  339       continue
            if (n.eq.np) goto 350
            do 340 j = n1,np
               q = a(i-1,j)
               r = a(i,j)
               a(i-1,j) = q*cs + r*sn
               a(i,j) = r*cs - q*sn
  340       continue
  350    continue
c
         t(l) = 0.0
         t(k) = f
         s(k) = x
         goto 230
c
c     convergence
  360    if (w.ge.0.0) goto 380
         s(k) = - w
         if (.not.withv) goto 380
         do 370 j = 1,n
            v(j,k) = - v(j,k)
  370    continue
  380    k = k - 1
         if (k.ne.0) go to 230
c
c     sort singular values
      do 450 k = 1,n
         gf = -1.0
         do 390 i = k,n
            if (s(i).lt.gf) goto 390
            gf = s(i)
            j = i
  390    continue
         if (j .eq. k) go to 450
         s(j) = s(k)
         s(k) = gf
         if (.not.withv) goto 410
         do 400 i = 1,n
            q = v(i,j)
            v(i,j) = v(i,k)
            v(i,k) = q
  400    continue
  410    if (.not.withu) goto 430
         do 420 i = 1,m
            q = a(i,j)
            a(i,j) = a(i,k)
            a(i,k) = q
  420    continue
  430    if (n.eq.np) goto 450
         do 440 i = n1,np
            q = a(j,i)
            a(j,i) = a(k,i)
            a(k,i) = q
  440    continue
  450 continue
c
      return
c***** end of subroutine fksvd *****
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine forwrd(ne)
c  this routine calculates the forward problem
c  given the stations, and initial hypocenters and
c  velocity model, the routine calculates theoretical
c  travel times in a way appropriate to the assumed
c  form of the velocity model.  three-d ray tracing
c  is performed. travel time derivatives with respect to
c  hypocentral and medium parameters are calculated
c
cfhRT if i3d = 4 uses paraxial raytracing
c
c  declaration statements:
      character*8 fil15
      character*1 phs(2)
c
c  common block variables:
      include 'simulps_common.inc'
c
      data phs/'P','S'/
      zoff=99.0
c  event coordinates
      xe=evc(1,ne)
      ye=evc(2,ne)
      ze=evc(3,ne)
c  loop over all observations of this event
      nobs=kobs(ne)
cfhRT before we loop over the observations we have to read in the shooting
cfhRT angle file if there is one if we use the paraxial raytracer
      if(i3d.eq.4) then
cfhRT prepare for paraxial raytracing, angles are there from loceqk.
cfh   for synthetic data angles have to be computed.
         if(nitmax.eq.-1) then
           call preparax(ne,nobs,0) 
         else
           call preparax(ne,nobs,1)
         endif
      write(80,'(/)')
      write(80,*)'RKP-raytracing for event ',ne,' from forwrd'
      endif
c
      write(26,2601) ne,iyrmo(ne),iday(ne),ihr(ne),mino(ne),
     2 seco(ne),ltde(ne),eltm(ne),lnde(ne),elnm(ne)
 2601 format(3h **,1x,i3,1x,a4,a2,1x,a2,i2,f6.2,i3,'n',f5.2,i4,'w',f6.2,
     2 2f7.2,3f6.2,3x,3i3)
      if(kout3.eq.0) goto 20
      write(fil15(1:8),1500) ne
 1500 format('ev',i03.3,'.rp')
cfa   open(unit=15,file=fil15,status='new')
      open(unit=15,file=fil15)
      write(15,1800) zoff,zoff,zoff
 1800 format('RAYPATH POINTS:',t68,f10.5,/,
     2 ' lat lon z x y -z',t68,f10.5,/,
     3 'format:17x,6f10.5',t68,f10.5)
cfh here starts the loop over all observations
   20 do 77 no=1,nobs
cfhRT ? paraxial raytracing: i3d = 4
         if (i3d.eq.4) then
          if (ides.gt.1) then
             write(61,*)' calling trvhet for event, obs: ',ne,no
          endif
cfh trvhet called with iprep=1 because we should have angles from loceqk if not synt.data
          if (nitmax.eq.-1) then
             call trvhet(ne,no,ttime,0)
          else
             call trvhet(ne,no,ttime,1)
          endif
         else 
          call path(ne,no,xe,ye,ze,ttime)
         endif 
c** Change for synthetic data, nitmax= -1
         if(nitmax.eq.-1) then
            secp(no,ne)=ttime+seco(ne)
c * * Different for S-P synthetic data * *
            isp=intsp(no,ne)
            if(isp.eq.1) then
               intsp(no,ne)=0
cfh calculate path and  trav time for p wave with same rec/stat coords 
cfhRT again use trvhet if i3d = 4
               if(i3d.eq.4) then 
                 call trvhet(ne,no,ptime,1)
               else
                 call path(ne,no,xe,ye,ze,ptime)
               endif
               intsp(no,ne)=1
               smptime=ttime-ptime
               secp(no,ne)=smptime
            endif
         else
            call ttmder(ne,no,1,ttime)
         endif
         if(kout3.eq.0) goto 77
c  write out raypath points
         write(15,1801) ne,no,zoff
 1801    format(' ev=',i4,' obs=',i4,t68,f10.5)
         write(15,1803) iyrmo(ne),iday(ne),ihr(ne),mino(ne),
     2   seco(ne),zoff
 1803    format(a4,a2,1x,a2,i2,1x,f5.2,t68,f10.5)
cfh also print station coordinates in evxxx.rp file 
cfh (shooting may end up somewhere else...)
         ns=isto(no,ne)
         write(15,1814) stn(ns),phs(intsp(no,ne)+1),
     2    stc(1,ns),stc(2,ns),-stc(3,ns),
     3    dlta(no,ne),zoff
 1814    format(a4,a1,3f10.5,2x,f6.2,t68,f10.5)
c         write(15,1804) stn(isto(no,ne)),phs(intsp(no,ne)+1),
c     2   dlta(no,ne),zoff
c 1804    format(a4,a1,f6.2,t68,f10.5)
         do 50 i=1,nrp(no)
            call latlon(rp(1,i,no),rp(2,i,no),lat,xlat,lon,xlon)
            xlat=float(lat)+xlat/60.0
            xlon=float(lon)+xlon/60.0
c  also print out negative z for easy plotting
            zd= -1.0 * rp(3,i,no)
            write(15,1802) xlat,xlon,rp(3,i,no),rp(1,i,no),
     2           rp(2,i,no),zd
 50      continue
 1802    format(17x,6f10.5)
 77   continue                  ! next observation
      if(kout3.eq.1) close(15)
      if(i3d.eq.4) then
cfhRT-------------- we write initial angles for future iterations
cfh -- put it to extra subroutine for cleanliness
         call finparax(ne,nobs)
      endif
      return
c***** end of subroutine forwrd *****
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine ftest(ndf,ndi,ratio)
c
c  interpolate in ftest ratio table
c  ndi is the initial number of degrees of freedom
c  ndf is the final number of degrees of freedom
c
      dimension rattab(4,4),valu(4)
      data rattab/1.69,1.64,1.58,1.51,1.59,1.53,1.47,1.39,
     *            1.50,1.43,1.35,1.25,1.39,1.32,1.22,1.00/
      data valu/0.025,0.016667,0.008333,0.00/
c
      if (ndi.ge.40.and.ndf.ge.40) go to 10
c
c  number of degrees of freedom outside table range
      ratio=2.0
      return
c
c  determine points in table for interploating
   10 continue
      index=ndi/30.
      index1=ndf/30.
      if (index.gt.4) index=4
      if (index1.gt.4) index1=4
c
      go to (11,12,12,14), index
c
   11 continue
      m=1
      m1=2
      go to 20
c
   12 continue
      m=2
      m1=3
      go to 20
c
   14 continue
      m=3
      m1=4
c
   20 continue
c
      go to (21,22,22,24), index1
c
   21 continue
      n=1
      n1=2
      go to 30
c
   22 continue
      n=2
      n1=3
      go to 30
c
   24 continue
      n=3
      n1=4
c
   30 continue
c
c  compute interpolated value from table
      fm=1.0/ndi
      fn=1.0/ndf
c
      vm=valu(m)
      vm1=valu(m1)
c
c  interpolate along m
      vf=rattab(m,n)+(rattab(m1,n)-rattab(m,n))*
     *   (fm-vm)/(vm1-vm)
      vf1=rattab(m,n1)+(rattab(m1,n1)-rattab(m,n1))*
     *    (fm-vm)/(vm1-vm)
c
c
      vn=valu(n)
      vn1=valu(n1)
c  interpolate across n
      ratio=vf+(vf1-vf)*(fn-vn)/(vn1-vn)
c***** end of subroutine ftest *****
      return
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine h12(mode,lpivot,l1,m,u,iue,up,c,ice,icv,ncv,
     2               c1,ic1e,ic1v,nc1v)
c  c.l. lawson and r.j. hanson, jpl
c  from "solving least squares problems"
c  modified by c.h. thurber, spring 1980
c  construction and application of a single
c  householder transformation...  q =  i + u*(u**t)/b
      dimension u(m),c(*),c1(*)
      double precision sm,b
      one=1.
c
      if (0.ge.lpivot.or.lpivot.ge.l1.or.l1.gt.m) return
      cl=abs(u(lpivot))
      if (mode.eq.2) go to 60
c
c  construct the transformation
c
      do 10 j=l1,m
         cl=amax1(abs(u(j)),cl)
   10 continue
      if (cl) 130,130,20
   20 clinv=one/cl
      sm=(dble(u(lpivot))*clinv)**2
      do 30 j=l1,m
         sm=sm+(dble(u(j))*clinv)**2
   30 continue
c
c  convert dble prec sm to sngl prec sm1
c
      sm1=sm
      cl=cl*sqrt(sm1)
      if (u(lpivot)) 50,50,40
   40 cl=-cl
   50 up=u(lpivot)-cl
      u(lpivot)=cl
      go to 70
c
c  apply the transformation i+u*(u**t)/b to matrices c & c1
c
   60 if (cl) 130,130,70
   70 if (ncv.le.0) return
      b=dble(up)*u(lpivot)
c
c  b must be nonpositive here.  if b=0 return
c
      if (b) 80,130,130
   80 b=one/b
      i2=1-icv+ice*(lpivot-1)
      incr=ice*(l1-lpivot)
      do 120 j=1,ncv
         i2=i2+icv
         i3=i2+incr
         i4=i3
         sm=c(i2)*dble(up)
         do 90 i=l1,m
            sm=sm+c(i3)*dble(u(i))
            i3=i3+ice
   90    continue
         if (sm) 100,120,100
  100    sm=sm*b
         c(i2)=c(i2)+sm*dble(up)
         do 110 i=l1,m
            c(i4)=c(i4)+sm*dble(u(i))
            i4=i4+ice
  110    continue
  120 continue
      i2=1-ic1v+ic1e*(lpivot-1)
      incr=ic1e*(l1-lpivot)
      do 220 j=1,nc1v
         i2=i2+ic1v
         i3=i2+incr
         i4=i3
         sm=c1(i2)*dble(up)
         do 190 i=l1,m
            sm=sm+c1(i3)*dble(u(i))
            i3=i3+ic1e
  190    continue
         if (sm) 200,220,200
  200    sm=sm*b
         c1(i2)=c1(i2)+sm*dble(up)
         do 210 i=l1,m
            c1(i4)=c1(i4)+sm*dble(u(i))
            i4=i4+ic1e
  210    continue
  220 continue
  130 return
c***** end of subroutine h12 *****
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine hyrot  (nmax,x, y, cs, sn, n)
      integer n
      real    x(nmax), y(nmax), cs, sn
c
c
      real    xx
      integer j
c
c
      do 10 j = 1, n
         xx = x(j)
         x(j) = xx*cs + y(j)*sn
         y(j) = y(j)*cs - xx*sn
   10 continue
      return
c***** end of subroutine hyrot *****
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -

c
      subroutine input1
c  this routine reads in control parameters and number of eq's
c
c  declaration statements:
cfa   character*9 day
cfa   character*8 tm
      Character DateToday*11,TimeNow*8
cfh      character*32 ctime
c
c  common block variables:
      include 'simulps_common.inc'
c
cfa   call date(day)
cfa   call time(tm)
cfa   write(16,1605) day,tm
      Call DateTime(DateToday,TimeNow)
      write(16,1605) DateToday,TimeNow
 1605 format(' Computation began at ',a11,1x,a8)
cfh for date and time utilities to work on Solaris2.x, compile with -lV77
cfh       call system("date > temp1")
cfh       open(321,file='temp1')
cfh      read(321,'(a32)',end=3333) ctime
cfh 3333  write(16,1605) ctime
cfh 1605 format(' Computation began at ',a32)
cfh       close(321)
cek
      write(16,9937)maxpar,mxpari,maxev,maxsta,maxobs
 9937  format(' Program Simulps14 (20-Apr-99 FH)',/, 
     * 'Added paraxial raytracing.',/,'Simulps13q (15-Mar-98 DMEP).'
     * '  Solves for Vp and '
     2 'Vp/Vs; Input data is P travel-time and S-P time.',/,
     3 '       Can vary relative weighting of S-P times in '
     4 'hypocenter location.',/,
     * /,'       OR can solve for Qp using tstar as input',/,/,
     5 '       Allows fixed nodes (up to ',i5,' parameters,'
     *  ' up to ',i5,' solution parameters);'/,
     6 '       up to ',i4,' events, ',i5,' stations, ',i4,
     *  ' observations per event',
     7 /,'       Psuedo-bending; Allows less curvature "moho" for ',
     8 'initial arcuate paths.')
      read(1,*,err=999) neqs,nsht,nbls,wtsht,kout,kout2,kout3
 1033 format(2i3,f4.1)
        if(neqs+nsht+nbls.le.maxev)go to 20
        write(16,22)
 22     format('0too many events for program arrays.')
        stop
  20    continue
c     write(16,9938)
 9938 format(/,'  program simul3m (jan 1985)',/,'  * fast art *',
     * /,' * pseudo-bending *')
      read(1,*) nitloc,wtsp,eigtol,rmscut,zmin,dxmax,rderr,ercof
      read(1,*) hitct,dvpmx,dvsmx,idmp,(vdamp(j),j=1,3),stepl
      read(1,*) ires,i3d,nitmax,snrmct,ihomo,rmstop,ifixl
      read(1,*) delt1,delt2,res1,res2,res3
      read(1,*) ndip,iskip,scale1,scale2
      read(1,*) xfac,tlim,nitpb(1),nitpb(2)
      read(1,*) iusep,iuses,invdel
      read(1,*) iuseq, dqmax, qdamp
      if((iuseq.eq.1).and.((neqs.ne.0).or.(nbls.ne.0))) then
        write(16,1611) neqs,nsht,nbls
 1611   format(/,'******* INPUT ERROR: neqs=',i3,'nsht=',i3,'nbls=',
     &  i3,/,'For Q inversion, t* data must be read in as Shots',
     &  /,'******* STOP *******',/)
        stop
      endif
      if((iuseq.eq.1).and.(iuses.eq.1)) then
        write(16,1610)
 1610   format(/,'******* INPUT ERROR: Q inversion is only for P',
     2  '*******              Resetting iuses to 0',/)
      endif
 1051 format(2f7.2,2f5.2)
 1011 format(4i3,f5.3)
 1021 format(i3,3f5.3)
      write(16,1030)
      write(16,1040) kout,kout2,kout3
c      write(16,1031)
c      write(16,1041) neqs,nsht,nbls,wtsht,nitloc,wtsp,zmin,eigtol,
c     2 rmscut,hitct,dvpmx,dvsmx
      write(16,1620)
      write(16,1621) neqs,nsht,nbls,wtsht,nitloc,wtsp,zmin,eigtol,
     2 rmscut,hitct
      write(36,1620)
      write(36,1621) neqs,nsht,nbls,wtsht,nitloc,wtsp,zmin,eigtol,
     2 rmscut,hitct
 1620 format(/,' neqs nsht nbls wtsht',
     2 ' nitloc wtsp   zmin   eigtol  rmscut hitct')
 1621 format(1x,3i4,4x,f4.1,1x,i3,f8.2,f7.2,f8.3,f9.3,f6.0)
c
      if(iuseq.eq.0) then
       if(iuses.gt.0) then
         if(iusep.eq.1) then
           write(36,1676) iusep,iuses
           write(16,1676) iusep,iuses
 1676      format(/,' Both P-velocity and Vp/Vs are included in ',
     2    'inversion  (iusep=',i2,', iuses=',i2,')')
         else
           write(16,1677) iusep,iuses
           write(36,1677) iusep,iuses
 1677      format(/,' Only S velocities are included in ',
     2    'inversion  (iusep=',i2,', iuses=',i2,')')
         endif
       else
         write(16,1678) iusep,iuses
 1678    format(/,' Only P velocities are included in ',
     2    'inversion  (iusep=',i2,', iuses=',i2,')')
c  put station damping in 2nd position if no Vs used
cfh !! but makes people wonder, as then the output vpvsdamp is 
cfh    not what they have input.... change that somehow??? 
         vdamp(2)=vdamp(3)
       endif
       write(16,1623)
       write(16,1624) dvpmx,dvsmx,idmp,(vdamp(j),j=1,3)
       write(36,1623)
       write(36,1624) dvpmx,dvsmx,idmp,(vdamp(j),j=1,3)
      else
        dvpmx=dqmax
        vdamp(1)=qdamp
        iusep = 1
        iuses = 0
         write(16,1679) iuseq,iusep,iuses
         write(36,1679) iuseq,iusep,iuses
 1679   format(/,' Inversion only for Q ',
     2      ' (iuseq=',i2,' ,iusep=',i2,' ,iuses=',i2,')')
        write(16,1625)
        write(16,1626) dvpmx,idmp,vdamp(1)
        write(36,1625)
        write(36,1626) dvpmx,idmp,vdamp(1)
      endif
 1623 format(/,'    dvpmx  dvpvsmx  idmp     vpdamp    vpvsdmp',
     2 '    stadamp')
 1624 format(f9.2,f9.3,i4,f13.3,2f11.3)
 1625 format(/,'     dqmax    idmp        qdamp')
 1626 format(f9.2,i8,f14.6)
c
      if(invdel.ne.0) write(16,1080) invdel
 1080 format(/,' Station delays included in inversion',
     2 ' (invdel=',i2,')',/)
c
c      write(16,1032)
c      write(36,1032)
c      write(16,1042) idmp,(vdamp(j),j=1,3),ires,
c     2 nitmax,snrmct,dxmax,rderr,ercof,ihomo,rmstop,ifixl,delt1,delt2,
c     3 res1,res2,res3
c      write(36,1042) idmp,(vdamp(j),j=1,3),ires,
c     2 nitmax,snrmct,dxmax,rderr,ercof,ihomo,rmstop,ifixl,delt1,delt2,
c     3 res1,res2,res3
c
      write(16,1627)
      write(36,1627)
      write(16,1628) ires,nitmax,snrmct,dxmax,rderr,ercof,ihomo,rmstop,
     2 ifixl,delt1,delt2,res1,res2,res3
      write(36,1628) ires,nitmax,snrmct,dxmax,rderr,ercof,ihomo,rmstop,
     2 ifixl,delt1,delt2,res1,res2,res3
 1627 format('  ires nitmax snrmct dxmax rderr ercof')
 1628 format(i3,i7,f9.5,3f6.2,
     * /,' # its. for 1-d vel. model = ',i3,/,' rms for term. = ',f5.3
     * ,/,' fix locations for iterations = ',i3
     * ,/,' distance weighting:',2f7.2,'; residual weighting:',3f5.2)
      write(16,1061) stepl
      write(16,1155)
 1155 format(' parameters for approximate ray tracer',
     * /,6x,'ndip iskip scale1 scale2')
      write(16,1151) ndip,iskip,scale1,scale2
 1151 format(5x,i3,i6,2f7.2)
 1061 format(' step length for integration:',f6.3)
 1030 format(/, ' control parameters',/,' kout kout2 kout3')
 1040 format(1x,i3,i5,i6)
 1031 format(/,' neqs nsht nbls wtsht',
     2 ' nitloc wtsp  zmin eigtol  rmscut hitct',
     3 ' dvpmx dvpvsmx')
 1032 format(/,' idmp    vpdamp   vpvsdmp   stadamp ires ',
     2 'nitmax snrmct dxmax rderr ercof')
 1041 format(1x,3i4,4x,f4.1,1x,i3,f8.2,f6.2,f6.3,f9.3,f6.0,2f6.2)
 1042 format(i3,f12.2,2f10.2,i3,i7,f9.5,3f6.2,
     * /,' # its. for 1-d vel. model = ',i3,/,' rms for term. = ',f5.3
     * ,/,' fix locations for iterations = ',i3
     * ,/,' distance weighting:',2f7.2,'; residual weighting:',3f5.2)
      write(16,1166) i3d,xfac,tlim,nitpb(1),nitpb(2)
 1166 format('  parameters for pseudo-bending:',/,
     * '  i3d xfac   tlim  nitpb(1) nitpb(2)',/,i4,f6.2,f7.4,i6,i10)
cfhRT
      if (i3d.eq.4) write(16,*)'i3d = 4: use ray shooting'
cfhRT end
c  increment iuses so it can be an index to arrays
      iuses=iuses+1
       ddlt=1.0/(delt2-delt1)
      if(res3.gt.res2) then
        dres12=0.98/(res2-res1)
        dres23=0.02/(res3-res2)
      else
        dres12=1.0/(res2-res1)
        dres23=0.0
      endif
      nevt=neqs+nsht+nbls
      return
  999 write(16,1699)
 1699 format('error in input1, probably forgot kout3')
c***** end of subroutine input1 *****
      stop
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine input2
c  this routine reads in the station list, sets up the
c  coordinate system, and calculates the stations' cartesian coordinates.
c  (reads from file02 )
c  subroutines required: setorg; disto;
c  common block variables:
      common/shortd/ xltkm,xlnkm,rota,nzco,xlt,xln,snr,csr
      include 'simulps_common.inc'
c
      character*1 cns, cew
c
      write(16,2000)
 2000 format(/,'  origin :  latitude   longitude   rotation')
c  read in center of coordinates and rotation angle
c 22aug95  For New Zealand (lat S and long E) make nzco=1,
c          For US (lat N and long W) make nzco=0
c          Note that this is only used in AZTOA as a quick
c          fix for getting azimuths correct
cfh 8apr98 when using changes from ek, the coordinate stuff should
cfh        be correct anyway, so nzco is useless (HAS to be 0?)
c
      read(2,*) ltdo,oltm,lndo,olnm,rota,nzco
c  If inverting station corrections, setup output file22 to
c  be used as input file in future runs.
      if((invdel.eq.0).or.(kout.lt.2)) goto 5
cfa   open(unit=22,status='new',file='newstns.txt')
      open(unit=22,file='newstns.txt')
        rewind 22
      write(22,2001) ltdo,oltm,lndo,olnm,rota,nzco
 2001 format(i3,1x,f5.2,i4,1x,f5.2,f8.2,i3)
    5 write(16,2002) ltdo,oltm,lndo,olnm,rota
 2002 format(12x,i3,1x,f5.2,2x,i4,1x,f5.2,3x,f7.2)
c  set up short-distance conversion factors, given center of coords
      call setorg(ltdo,oltm,lndo,olnm)
      write(16,2004)
 2004 format('   station   latitude   longitude   elev',
     2 '     x      y      z   pdl s-pdl nfixst')
c  read in number of stations
      read(2,*) nsts
        if(nsts.le.maxsta)go to 40
        write(16,41)
 41     format('0Too many stations for input arrays.')
        stop
 40     continue
c  read in station list
c  Now read to end of file, skipping sta with lat and lon=0
cfhek  changes made to read east-west, north-south (read in cns, cew and use it)
      j=1
   15 continue
c      do 20 j=1,nsts
      read(2,2007,end=30) stn(j),ltds(j),cns,sltm(j),lnds(j),cew,
     *     slnm(j),ielev,pdl(j),sdl(j),nfixst(j)
cfh test elevation needs to be i7 when trying to shoot reverse (then
cfh   station is at source-depth... but now back to i5.
c  2007 format(2x,a4,i2,a1,f5.2,i4,a1,f5.2,i7,2f5.2,1x,i1)
 2007 format(2x,a4,i2,a1,f5.2,i4,a1,f5.2,i5,2f5.2,1x,i1)
      z=-ielev*1.0e-3
cfhek
cek200596 change made to recognize North South and East West
      if(cns.eq.'S'.or.cns.eq.'s') then
        ltds(j)=-1.*ltds(j)
        sltm(j)=-1.*sltm(j)
      endif
      if(cew.eq.'E'.or.cew.eq.'e') then
        lnds(j)=-1.*lnds(j)
        slnm(j)=-1.*slnm(j)
      endif
c  calculate cartesian coordinates of station
      call disto(1,j,x,y)
      if((kout2.lt.4).or.(j.eq.1)) then 
cek
         if(ltds(j).ge.0.and.sltm(j).ge.0.) then
            cns='N'
            latdeg=iabs(ltds(j))
            alatmin=abs(sltm(j))
         else
            cns='S'
            latdeg=iabs(ltds(j))
            alatmin=abs(sltm(j))
         endif
         if(lnds(j).ge.0.and.slnm(j).ge.0.) then
            cew='W'
            londeg=iabs(lnds(j))
            alonmin=abs(slnm(j))
         else
            cew='E'
            londeg=iabs(lnds(j))
            alonmin=abs(slnm(j))
         endif
         write(16,2009) j,stn(j),latdeg,cns,alatmin,londeg,cew,
     2        alonmin,ielev,x,y,z,pdl(j),sdl(j),nfixst(j)
cek           write(16,2009) j,stn(j),ltds(j),sltm(j),lnds(j),
cek     2      slnm(j),ielev,x,y,z,pdl(j),sdl(j),nfixst(j)
         endif
cek
 2009    format(1X,i4,3x,a4,1x,i3,a1,f5.2,2x,i4,a1,f5.2,2x,i5,
     *      1x,3f7.2,2f5.2,i3)
c        call latlon(x,y,latsta,xltsta,lonsta,xlnsta)
c        write(16,1666) latsta,xltsta,lonsta,xlnsta
c1666    format(2x,'checking latlon ',2(i4,f6.2))
c  store station coordinates
         stc(1,j)=x
         stc(2,j)=y
         stc(3,j)=z
         j=j+1
         goto 15
   20 continue
   30 continue
      nstsrd=j-1
      if(nsts.ne.nstsrd) then
        write(16,2098) nstsrd, nsts,nstsrd
 2098 format('*************************************',/,
     2  'Number station read in',i5,' NOT EQUAL Number of '
     3  'stations specified',i5,/,
     4  ' Continue with ',i5,/,
     4  '*************************************')
        nsts=nstsrd
      endif
      if(kout2.lt.4) return
c
c  kout2=4, condensed printout
   50 continue
      write(16,1605) nsts
 1605 format('  number of stations read in Input2 =',i5,
     2 ' (print 1st and last only)')
      j=nsts
cek
      if(ltds(j).ge.0.and.sltm(j).ge.0.) then
         cns='N'
         latdeg=iabs(ltds(j))
         alatmin=abs(sltm(j))
      else
         cns='S'
         latdeg=iabs(ltds(j))
         alatmin=abs(sltm(j))
      endif
      if(lnds(j).ge.0.and.slnm(j).ge.0.) then
         cew='W'
         londeg=iabs(lnds(j))
         alonmin=abs(slnm(j))
      else
         cew='E'
         londeg=iabs(lnds(j))
         alonmin=abs(slnm(j))
      endif
      write(16,2009) j,stn(j),latdeg,cns,alatmin,londeg,cew,
     2     alonmin,ielev,x,y,z,pdl(j),sdl(j),nfixst(j)
cek      write(16,2009) j,stn(j),ltds(j),sltm(j),lnds(j),slnm(j),
cek     2 ielev,x,y,z,pdl(j),sdl(j),nfixst(j)
      return
c
c  not enough stations in list - error
   99 nsts=j-1
      write(16,2099) nsts
 2099 format('/ *** too few stations in list *** continue with ',i5)
      if(kout2.eq.4) goto 50
      return
c***** end of subroutine input2 *****
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine input3
c  this routine reads in the initial velocity model in the
c  form of velocity specified on a uniform but not
c  necessarily evenly spaced grid of points
c  (reads from file03 )
c
c  common block variables:
      include 'simulps_common.inc'
c
c  declaration statements:
      integer ixf(maxpar),iyf(maxpar),izf(maxpar)
      character*1 vtype(2)
      parameter(zero=0.0,izero=0)
c
      vtype(1)='P'
      vtype(2)='S'
c
c  for this version the gridpoints can be unevenly spaced
c  the origin of the coordinate system is at (x,y,z)=(0,0,0)
c  which will not in general correspond to the point
c  xn(1),yn(1),zn(1).
c  xn,yn,zn should be factors of bld (ie: a.0 for bld=1.0 or a.b for bld=0.1)
c
c input the number of gridpoints in x, y and z directions
c  and bld factor (1.0 or 0.1 km) used to set up velocity interpolation grid
      read(3,3002) bld,nx,ny,nz
 3002 format(f4.1,3i3)
      if((bld.ne.1.0).and.(bld.ne.0.1)) then
        write(16,1625) bld
 1625   format(/, '******** STOP *********, bld must be 1.0 or 0.1,
     2   not ',f6.2)
      endif
        atemp=iuses*(nx-2)*(ny-2)*(nz-2)
        if(atemp.le.maxpar)goto 40
        write(16,42)
 42     format('0Too many nodes for program array sizes.')
        stop
 40     continue
c
c  input the x grid, y grid, and z grid
cfh (from veronique lankar) read in in free format
c        read(3,3004) (xn(i),i=1,nx)
c        read(3,3004) (yn(i),i=1,ny)
c        read(3,3004) (zn(i),i=1,nz)
         read(3,*) (xn(i),i=1,nx)
         read(3,*) (yn(i),i=1,ny)
         read(3,*) (zn(i),i=1,nz)
c
 3003 format(3i3)
 3004 format(25f6.1)
c
      write(16,3005) bld,nx,ny,nz
 3005 format(//,' velocity grid size:',/,
     * 'bld =',f4.1,5x,' nx =',i3,5x,'ny =',i3,5x,'nz =',i3)
c
cfh give all these numbers the same format
      write(16,3006) (xn(i),i=1,nx)
cfh 3006 format(/,' xgrid',/,3x,12f7.1,8f6.1)
 3006 format(/,' xgrid',/,3x,20f7.1)
      write(16,3007) (yn(i),i=1,ny)
cfh 3007 format(/,' ygrid',/,3x,12f7.1,8f6.1)
 3007 format(/,' ygrid',/,3x,20f7.1)
      write(16,3008) (zn(i),i=1,nz)
cfh 3008 format(/,' zgrid',/,3x,8f6.1,12f7.1/)
 3008 format(/,' zgrid',/,3x,20f7.1/)
c
c  read in which nodes to have fixed velocity
c  end with blank line
      i=1
   50 read(3,3003) ixf(i),iyf(i),izf(i)
      if(ixf(i).le.0) goto 60
      i=i+1
      goto 50
   60 continue
      inf=i-1
c
c  now read in the velocity values
   65 write(16,3101)
c     do 38 kv=1,iuses
         kv=1
         do 37 k=1,nz
            k2=k + (kv-1)*nz
            write(16,3015) k,vtype(kv),zn(k)
            do 36 j=1,ny
cfh               read(3,3011) (vel(i,j,k2),i=1,nx)
                  read(3,*)  (vel(i,j,k2),i=1,nx)
               write(16,3013) (vel(i,j,k2),i=1,nx)
   36       continue
   37    continue
c  38 continue
c CHANGE FOR VP/VS INVERSION
      if((iuses.eq.2).or.(iuseq.eq.1)) then
        do 100 k=1,nz
          if(iuseq .eq. 0) then
              write(16,3016) k,zn(k)
          else
              write(16,3017) k,zn(k)
          endif
           do 99 j=1,ny
             if(iuseq .eq. 0) then
cfh                 read(3,3011) (vpvs(i,j,k),i=1,nx)
                 read(3,*)  (vpvs(i,j,k),i=1,nx)
                 write(16,3013) (vpvs(i,j,k),i=1,nx)
             else
               read(3,3014) (qval(i,j,k),i=1,nx)
               write(16,3014) (qval(i,j,k),i=1,nx)
             endif
   99     continue
  100  continue
c  compute Vs from Vp and Vp/Vs or compute 1/tstar
        kv=2
       if(iuseq .eq. 0) then
           do 120 k=1,nz
              ks=k+nz
              write(16,3015) k,vtype(kv),zn(k)  
              do 115 j=1,ny
                 do 110 i=1,nx
                    vel(i,j,ks)=vel(i,j,k)/vpvs(i,j,k)
  110           continue
                 write(16,3014) (vel(i,j,ks),i=1,nx)
  115        continue
  120     continue
       else
         do 140 k=1,nz
            ks=k+nz
            write(16,3018) k,zn(k)
            do 135 j=1,ny
               do 130 i=1,nx
                  vel(i,j,ks)=vel(i,j,k)*qval(i,j,k)
  130           continue
                 write(16,3014) (vel(i,j,ks),i=1,nx)
  135        continue
  140     continue
       endif
      endif
c
 3013 format(20f6.2)
 3014 format(20f7.1)
 3015 format(/,' layer',i3,5x,a1,' velocity',10x,'z =',f7.1)
 3016 format(/,' layer',i3,5x,'Vp/Vs',10x,'z =',f7.1)
 3017 format(/,' layer',i3,5x,'Q',10x,'z =',f7.1)
 3018 format(/,' layer',i3,5x,'Q * Vp',10x,'z =',f7.1)
 3011 format(20f5.2)
 3101 format(//,' velocity values on three-dimensional grid')
c  compute total number of gridpoints (nodes)
      nodes=nx*ny*nz
      nxy=nx*ny
      nx2=nx-2             ! number non-edge nodes in row
      nxy2=nx2*(ny-2)      ! number non-edge nodes in layer
      nz2=nz-2
      nodes2=nz2*nxy2
c  peripheral nodes
      nx1=nx-1
      ny1=ny-1
      nz1=nz-1
c  Number of medium parameters to invert for
      npar=nodes2*iuses
      nparv=npar
      if(invdel.ne.0)npar=(npar+nsts*iuses)
c
c  Check to see whether medium parameters fit within array sizes
      if(nparv.gt.maxpar) goto 980
c
cfhdmep
c get number of Vp and Vp/Vs nodes that are free in the inversion
c    nodes reduced by fixednodes
      nparpi=nodes2
      nparsi=nodes2
c
c  fix specified nodes by setting nfix(k)=1, else=0
      if(inf.eq.0) goto 496
        do 70 i=1,inf
           iizf=izf(i)-2
c  if s velocity node
cfhdmep           if(izf(i).gt.nz) iizf=izf(i)-4
           if(izf(i).gt.nz) then
              iizf=izf(i)-4
              nparsi=nparsi-1
           else
              nparpi=nparpi-1
           endif
c
           k=iizf*nxy2 + (iyf(i)-2)*nx2 + (ixf(i)-1)
           nfix(k)=1
   70   continue
c
       write(16,1610)
 1610 format(/,' velocity FIXED at the following nodes(1):')
  311 do 495 kv=1,iuses
         nz1=nz-1
         ny1=ny-1
         do 320 k=2,nz1
          if(iuseq.eq.0) then
            if(kv.eq.1) write(16,1009) k,vtype(kv),zn(k)
 1009       format(/,' layer',i3,5x,a1,'-velocity nodes',
     2         10x,'z =',f7.1)
            if(kv.eq.2) write(16,3016) k,zn(k)
          else
            write(16,3017) k,zn(k)
          endif
            kk=k+(kv-1)*nz2
            do 310 j=2,ny1
               n1=(kk-2)*nxy2+(j-2)*nx2+1
               n2=n1+nx2-1
               write(16,1005) (nfix(i),i=n1,n2)
  310       continue
  320    continue
 1005 format('    ',18i6)
  495 continue
  496 continue
c
c  ndexfx: index from full nodes to nodes reduced by fixed (invert nodes)
c  mdexfx: index from inversion solution nodes to full velocity nodes
      in=0
      do 80 i=1,nparv
         if(nfix(i).eq.1) goto 80
         in=in+1
         ndexfx(i)=in
         mdexfx(in)=i
   80 continue
      inf2=nparv-in
      if(inf2.eq.inf) goto 85
        write(16,1615) inf,nparv,in,inf2
 1615   format(/,' **** number of fixed nodes input,',i4,
     2  ', does not equal velocity nodes,',i4,', minus invert',
     3  ' nodes,',i4,'.  Continue with inf=',i5,' ****',/)
        inf=inf2
   85 continue
      nparvi=nparv-inf
      npari=npar-inf
      if(invdel.eq.0) goto 95
c  also set indices if station delays are included in inversion
      i1=nparv+1
      do 90 i=i1,npar
         is=i-nparv
c s-delay
         if(is.gt.nsts) is=is-nsts
         if(nfixst(is).eq.1) goto 90
         in=in+1
         ndexfx(i)=in
         mdexfx(in)=i
   90 continue
      npari=in
   95 continue
      write(16,1620) npar,nparv,npari,nparvi
 1620 format(' INPUT3:npar,nparv,npari,nparvi',4i6)
c  Check to see whether medium parameters fit within array sizes
      if(npari.gt.mxpari) goto 990
      if(npar.gt.maxpar) goto 995
c
c  Set up an array which is used to point to node indices, for any x,y,z
      call bldmap
c
      return
c
  980 continue
      write(16,1698) nparv,maxpar
 1698 format(/,'  ****** STOP ******',/,i8,' velocity nodes, program',
     2 ' arrays only allow',i6)
      stop
  990 continue
      write(16,1699) npari,mxpari
 1699 format(/,'  ****** STOP ******',/,i8,' parameters to invert for',
     2 ', program arrays only allow',i6)
      stop
  995 continue
      write(16,1695) npar,maxpar
 1695 format(/,'  ****** STOP ******',/,i8,' parameters, arrays are ',
     2 'for ',i8)
      stop
c
c***** end of subroutine input3 *****
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine input4(n,nfile)
c  this routine reads in the p travel times for all stations observing
c  the current event.  first trial hypocentral parameters are read, then
c  station names and weights and travel times.  arrivals at stations not
c  in the station list are discarded; unnormalized weights are calculated.
c  (reads earthquakes from file04, reads shots from file07 )
c  subroutine required: disto;
c
c  declaration statements:
c  local variables:
      real dep,x,y,pwt
      dimension nwav(2),rmki(6),ip(6),tt(6),sta(6),is(6)
      character*110 line
cfa
      character*1 cns, cew, is
      character*4 blank,iblank,blank0,chk,rmki,sta
c
c  common block variables:
      include 'simulps_common.inc'
c
      data  blank,iblank,blank0 /4h    ,4h    ,4h0   /
cek
      cns='N'
      cew='W'
c
      if (n.gt.1) go to 1
c  print out heading for event cards
      write(16,4000)
 4000 format(//,' trial event locations:',/,
     2 5x,'n    origin time     latitude longitude  depth',
     3 ' mag',5x,'x      y      z',4x,'nob np ns')
      nobt=0
      nobtp=0
      nobts=0
    1 nobs=0
      nwav(1)=0
      nwav(2)=0
      wsum=0
c  check for extra travel time card
c  read in event card
cek ss - cfh ? what does that change?
CC le prossime due commentate da Solarino e riscritte
c  10 read(nfile,4111,end=99) nchar,line,chk
c4111 format(q,a85,t1,a4)
c     write(16,4012) nchar,line,chk
   10 read(nfile,4111,end=99) line,chk
 4111 format(a85,t1,a4)
c
 4012 format(2x,i5,' characters in line',/,2x,a85,/,2x,'first 4 ',
     2 'char = ',a4)
    5 if((chk.eq.blank).or.(chk.eq.blank0)) goto 10
c
cek200596 change made to recognize North South and East West
c
   15 read(line,4001,err=5098) iyrmo(n),iday(n),ihr(n),
     2 mino(n),seco(n),ltde(n),cns,eltm(n),lnde(n),cew,elnm(n),
     2 dep,rmag(n)
 4001 format(a4,a2,1x,a2,i2,1x,f5.2,i3,a1,f5.2,1x,i3,a1,f5.2,2f7.2)
      if(cns.eq.'S'.or.cns.eq.'s') then
        ltde(n)=-1.*ltde(n)
        eltm(n)=-1.*eltm(n)
      endif
      if(cew.eq.'E'.or.cew.eq.'e') then
        lnde(n)=-1.*lnde(n)
        elnm(n)=-1.*elnm(n)
      endif
cek200596 end change made to recognize North South and East West
c
c
      if (iyrmo(n).eq.iblank) go to 10
c  calculate event location in cartesian coordinate system
      call disto(2,n,x,y)
c  store event coordinates
      evc(1,n)=x
      evc(2,n)=y
      evc(3,n)=dep
c  read in travel times to stations in groups of six
   20 continue
c  format including phase-remarks
      read(nfile,4007) line
 4007 format(a110)
      if(iuseq.eq.0) then
        read(line,4006,err=22) (sta(j),rmki(j),tt(j),j=1,6),
     2  (is(j),ip(j),j=1,6)
      else
        read(line,4029,err=22) (sta(j),rmki(j),tt(j),j=1,6),
     2    (is(j),ip(j),j=1,6)
      endif
 4006 format(6(2a4,f6.2),t1,6(5x,a1,1x,i1,6x))
 4029 format(6(2a4,f10.4),t1,6(5x,a1,1x,i1,10x))
      goto 25
c Alberto Michelini (LBL) format
   22 read(line,4008,err=23) (sta(j),rmki(j),tt(j),j=1,5),
     2  (is(j),ip(j),j=1,5)
 4008 format(5(2a4,f7.4),t1,5(5x,a1,1x,i1,7x))
      goto 25
c  old format
   23 read(line,4005,err=5099) (sta(j),is(j),ip(j),tt(j),j=1,6)
 4005 format(6(a4,a1,i1,f6.2))
c
c  loop over the six readings
c  terminate if station is a blank (end of observations for event)
c  skip if weight is incorrect in some way
c  search list for current station, and index it if in list
c  also calculate p-arrival time and unnormalized weight
c  and increment number of observations for event
c
   25 continue
c  blank line separates the events
      if ((sta(1).eq.blank).or.(sta(1).eq.blank0)) go to 50
      do 30 j=1,6
         if(ip(j).ge.4) goto 30
         pwt=1.0/(2.**(ip(j)))
         if (pwt.le.0.0) go to 30
c search station list
         do 33 k=1,nsts
            if (sta(j).eq.stn(k)) go to 35
   33    continue
         if(kout2.gt.1) goto 30
         if(sta(j).eq.blank .or. sta(j).eq.blank0) then
            write(16,1609)
 1609       format(' Blank-station observation ignored')
         else
            write(16,1610) sta(j),iyrmo(n),iday(n),ihr(n),
     *      mino(n),seco(n)
 1610       format(' ** WARNING:  Observed station "',a4,
     *      '" (event ',a4,a2,1x,a2,i2,1x,f5.2,
     *      ') is not in station list--observation ignored **')
         endif
         go to 30
c  add to set of observing stations if on list
   35    continue
c  don't use arrivals further away than delt2
         dx=evc(1,n)-stc(1,k)
         dy=evc(2,n)-stc(2,k)
         dz=evc(3,n)-stc(3,k)
c  Use epicentral distance instead of slant distance
c        delta=sqrt((dx*dx)+(dy*dy)+(dz*dz))
cfh - rely on delta being epicentral distance in trvhet!
         delta=sqrt((dx*dx)+(dy*dy))
         if(delta.lt.delt2) goto 36
         if(kout2.eq.0) write(16,1602) delta,delt2,stn(k),n
 1602    format(' **** delta,delt2=',f15.2,f8.2,2x,a4,
     &     ' not used for event',i4)
         goto 30
c        set iuses to 0 (changed to iuses+1 in input1) to disable
c           S-wave processing.
   36    if(iuses.eq.1.and.is(j).eq.'S') goto 30
         if((iusep.eq.0).and.((is(j).eq.'P').or.(is(j).eq.'p')))
     *      goto 30
         nobs=nobs+1
c check for too many observations 
         if(nobs.gt.maxobs) then
            write(61,1603) iyrmo(n),iday(n),ihr(n),mino(n),maxobs
 1603       format('  INPUT ERROR, Too many readings for event:',
     2      a4,a2,1x,a2,i2,/,'    CONTINUE with first',i5,
     3      'observations')
            write(16,1603) iyrmo(n),iday(n),ihr(n),mino(n),maxobs
            nobs=maxobs
            goto 50
         endif
         dlta(nobs,n)=delta
         iw(nobs,n)=ip(j)
         isto(nobs,n)=k
         secp(nobs,n)=tt(j)+seco(n)
         rmk(nobs,n)=rmki(j)
c  here, except for the addition of pdl(k) in the above line
c  are additions by wap to read in s times. If the time is
c  an s time, is(j) will equal 'S'. If this is true, the
c  proper adjustment will be made for the station delay, and
c  a 1 will be put in intsp(nobs,n).
         intsp(nobs,n)=0
         if(rmk(nobs,n).eq.blank) rmk(nobs,n)=' P  '
         if((is(j).ne.'S').and.(is(j).ne.'s')) goto 37
c  intsp(nobs,n) tells whether the nobs'th observation of the
c  arrival time is a P (0), or an S (1).
         intsp(nobs,n)=1
c** S-P CHANGE
c   For S-P we do not want to add origin time
         secp(nobs,n)=tt(j)
         if(rmk(nobs,n).eq.blank) rmk(nobs,n)=' S  '
  37     continue
         kv=intsp(nobs,n)+1
         nrd(k,kv)=nrd(k,kv)+1
         nwav(kv)=nwav(kv)+1
         wt(nobs,n)=pwt
c  19:july-1983 Change in normalizing factor
         wsum=wsum+pwt
c        wsum=wsum+pwt*pwt
   30 continue
c  continue reading stations and travel times
      go to 20
   50 continue
c  end of travel time readings for current event
c  check that there are at least four readings for this event
c  (does not matter for shots)
      if(nobs.eq.0) goto 90
      if((nobs.lt.4).and.(n.le.neb)) go to 90
cek
      if(ltde(n).ge.0.and.eltm(n).ge.0.) then
        cns='N'
        latdeg=iabs(ltde(n))
        alatmin=abs(eltm(n))
      else
        cns='S'  
        latdeg=iabs(ltde(n))
        alatmin=abs(eltm(n))
      endif
      if(lnde(n).ge.0.and.elnm(n).ge.0.) then
        cew='W'
        londeg=iabs(lnde(n))
        alonmin=abs(elnm(n))
      else
        cew='E'  
        londeg=iabs(lnde(n))
        alonmin=abs(elnm(n))
      endif
cfhek - output again correct - no sign but qualifier
cek
      write(16,4004) n,iyrmo(n),iday(n),ihr(n),mino(n),seco(n),
     2 latdeg,cns,alatmin,londeg,cew,alonmin,dep,rmag(n),x,y,dep,
     3 nobs,nwav(1),nwav(2)
cek      write(16,4004) n,iyrmo(n),iday(n),ihr(n),mino(n),seco(n),
cek     2 ltde(n),cns,eltm(n),lnde(n),cew,elnm(n),dep,rmag(n),x,y,dep,
cek     3 nobs,nwav(1),nwav(2)
 4004 format(3h **,1x,i3,1x,a4,a2,1x,a2,i2,f6.2,i3,a1,f5.2,i4,a1,f5.2,
     2 2f7.2,3f6.2,3x,3i3)
c  normalize reading weights
c   change in normalizing factor 19-jul-1983
      wfac=nobs/wsum
c     wfac=sqrt(nobs/wsum)
c  check for too many readings
      if(nobs.le.maxobs) goto 55
        write(16,1698) n,nobs,maxobs,maxobs
 1698   format(' **** ERROR **** in event ',i4,', TOO MANY ',
     2  'OBSERVATIONS, nobs=',i6,' array size allows',i6,/,
     2  '   continuing with',i6,' observations',/)
        nobs=maxobs
   55 kobs(n)=nobs
      nobt=nobt+nobs
      nobtp=nobtp+nwav(1)
      nobts=nobts+nwav(2)
      if(n.gt.netemp) then
        nobtex=nobtex+nobs
        nobt=nobt+(wtsht-1)*nobs
      else
        nobteq=nobteq+nobs
      endif
      do 60 j=1,nobs
      wt(j,n)=wt(j,n)*wfac
   60 continue
      return
c  not enough readings for event n
   90 continue
      write(16,4009)  iyrmo(n),iday(n),ihr(n),mino(n)
 4009 format(' *** event ',a4,a2,1x,a2,i2,' should be discarded - ',
     2 'too few observations ***',/,' *** SKIPPING TO NEXT EVENT ***')
      if(nfile.eq.7) then       ! shot datafile
        nsht=nsht-1
      else
        if(nfile.eq.4) then     !earthquake datafile
          if(ifixl.le.0) then
            neqs=neqs-1
          else
            nbls=nbls-1
          endif
          netemp=netemp-1
        else                    ! blast datafile
          nbls=nbls-1
          nbtemp=nbtemp-1
        endif
        neb=neb-1
      endif
      nevt=nevt-1
      goto 1
c  ran out of event cards - error]
   99 write(16,4099) nfile
 4099 format('**** end of data file',i2,' - neqs too large ****')
      if(nfile.ne.4) goto 199
c  assume input error if neqs was less than 10% off, and continue with smaller neqs
c  assume other error,possibly wrong file, if more than 10% off, then stop
        if((abs(netemp-n)).gt.(0.10*netemp)) goto 199
        netemp=n-1
        neb=nbtemp+netemp
        nevt=neb+nsht
          if(ifixl.le.0) then
            neqs=n-1
          else
            nbls=neb
          endif
      write(16,1699) netemp
 1699 format(' continue with neqs= ',i4)
      return
  199 continue
      write(16,1690)
 1690 format(' stop since neqs more than 10% off, or problem '
     2 ,'was nbls or nsht')
      stop
 5098 write(16,5050) n
      write(16,4012) nchar,line,chk
cek
      if(ltde(n).ge.0.and.eltm(n).ge.0.) then
        cns='N'
        latdeg=iabs(ltde(n))
        alatmin=abs(eltm(n))
      else
        cns='S'
        latdeg=iabs(ltde(n))
        alatmin=abs(eltm(n))
      endif
      if(lnde(n).ge.0.and.elnm(n).ge.0.) then
        cew='W'
        londeg=iabs(lnde(n))
        alonmin=abs(elnm(n))
      else
        cew='E'
        londeg=iabs(lnde(n))
        alonmin=abs(elnm(n))
      endif
cek
      write(16,4011) iyrmo(n),iday(n),ihr(n),mino(n),seco(n),
     2 latdeg,cns,alatmin,londeg,cew,alonmin,dep,rmag(n)
cek      write(16,4011) iyrmo(n),iday(n),ihr(n),mino(n),seco(n),
cek     2 ltde(n),eltm(n),lnde(n),elnm(n),dep,rmag(n)
 4011 format(1x,a4,a2,a3,i2,1x,f5.2,1x,i3,a1,f5.2,1x,i3,a1,f5.2,2f7.2)
      goto 10
c     stop
 5099   write(16,5050) n
      write(16,4015) (sta(j),is(j),ip(j),tt(j),j=1,6)
 4015 format(1x,6(a4,a1,i1,f6.2))
 5050   format('0error in data in input4. event= ',i8)
        stop
c****** end of subroutine input4 ******
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -

c
      subroutine intmap(x,y,z,ip,jp,kp)
c  Modified by W. Prothero so a single call can get the indices
c  common block variables:
      include 'simulps_common.inc'
c
      ip=int((x+xl)/bld)
      ip=ixloc(ip)
      jp=int((yl+y)/bld)
      jp=iyloc(jp)
      kp=int((z+zl)/bld)
      kp=izloc(kp)
c  If an array element=0, the position is off the map.
      return
c***** end of subroutine intmap *****
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
       subroutine latlon(x,y,lat,xlat,lon,xlon)
c  Subroutine to convert from Cartesian coordinates back to
c  latitude and longitude.
c
        common/shortd/xltkm,xlnkm,rota,nzco,xlt,xln,snr,csr
c
      rad=1.7453292e-2
      rlt=9.9330647e-1
c
      fy=csr*y-snr*x
      fx=snr*y+csr*x
c
      fy=fy/xltkm
      plt=xlt+fy
c
      xlt1=atan(rlt*tan(rad*(plt+xlt)/120.))
      fx=fx/(xlnkm*cos(xlt1))
      pln=xln+fx
c
      lat=plt/60.
      xlat=plt-lat*60.
      lon=pln/60.
      xlon=pln-lon*60.
c
      return
c
c***** end of subroutine latlon ******
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine loceqk(ne,niti,nwr)
c  this routine locates the given event in the initial velocity
c  structure using approximate ray tracing
cfhRT or using paraxial raytracing if i3d=4
cfhRT paraxial raytracing is done on the squared slowness grid
cfhRT but the hypocenter parameter derivatives are computed
cfhRT again with ttmder on the inversion grid
c
c  common block variables:
      common/shortd/xltkm,xlnkm,rota,nzco,xlt,xln,snr,csr
      include 'simulps_common.inc'
c
c  declaration statements:
      real dr5(3),dr5i(3),s(4),v(4,4),adj(4),ster(4),cov(4)
      real w(maxobs)
      double precision ah(maxobs,maxobs),ac(3,3)
c  variables for error ellipse calculations
      integer iaz(3),idip(3)
      real covar(4,4),v3(4,4),serr(3)
      parameter (drad=1.7453292d-02)
c
c  Define small change in hypocenter, depending on bld factor
      small=0.5
      if(bld.eq.0.1) small=0.1
c
      xmov=900.0
c  nitl=counter for location iterations
      nitl=0
c  niti=counter for inversion iterations, Thurber starts with niti=1,
c       Eberhart-Phillips starts with niti=0
      if(niti.eq.0) then
        jfl=0
      else
        jfl=2
        if(ihomo.eq.niti) jfl=1
      end if
      jflag=jfl
cfh  when does the following happen ???
      if (nitloc.eq.0) go to 9999
      rmswt=0.0
      rmslst=10.0
      rmsbfl=20.0
      nobs=kobs(ne)
cfh only on first inversion iteration do we have no angles !
      if(niti.eq.0) then 
        iprep=0         !to mark that there are not yet any angles (.or.iterrai.eq.1)
      else 
       iprep=1
      endif
c
c  location iteration loop
    1 continue
c  event coordinates
      xe=evc(1,ne)
      ye=evc(2,ne)
      ze=evc(3,ne)
c
cfhRT prepare for paraxial raytracing
cfh -- this creates a lot of disk I/O
cfh -- why not store tet_sav, phi_sav, ji_sav
cfh -- in fixed array?
      if(i3d.eq.4) then
         if (ides.gt.1) write(61,*)'In routine loceqk'
         call preparax(ne,nobs,iprep)
         write(80,'(/)')
         write(80,*)' RKP-raytracing for event ',ne,
     .   ' in loceqk with iprep = ',iprep
      endif
cfhRT
c
c  loop over all observations of this event
      do 10 no=1,nobs
       ns=isto(no,ne)
       xr=stc(1,ns)
       yr=stc(2,ns)
       zr=stc(3,ns)
cfhRT the next chunk of code is for thurber-raytracing
      if(i3d.ne.4) then
c  check for P or S reading 
       isp=intsp(no,ne)
cfh this is also needed when parax RT (for aztoa)
c       ns=isto(no,ne)
c       xr=stc(1,ns)
c       yr=stc(2,ns)
c       zr=stc(3,ns)
c  If small change in hypocenter location, start with
c  old pb 3D path for additional hypocenter only iterations
       if(i3d.lt.2) goto 120
       if(pl(no).eq.0.0) pl(no)=dlta(no,ne)
c     if(xmov.gt.(0.05*pl(no))) goto 120
c  small change is defined at beginning of loceqk
       if(xmov.gt.small) goto 120
c  Move end of raypath
       dr5(1)=xe-rp(1,1,no)
       dr5(2)=ye-rp(2,1,no)
       dr5(3)=ze-rp(3,1,no)
       do 105 k=1,3
         dr5i(k)=dr5(k)/5.0
  105  continue
       do 115 j=1,5
         dj=float(5-(j-1))
         do 110 k=1,3
           rp(k,j,no)=rp(k,j,no)+dr5i(k)*dj
  110    continue
  115  continue
       ttime=ttc(no)
       goto 125
c  determine 3-d path using circular raypaths.
c  determine approximate path
  120  ncrold=nco(no,ne)
       ndpold=ndo(no,ne)
       call rayweb(no,isp,xe,ye,ze,xr,yr,zr,
     *  fstime,jflag,ncrold,ndpold)
       nco(no,ne)=ncrold
       ndo(no,ne)=ndpold
c
c  calculate residual and hypocentral partial derivatives
       ttime=fstime
c
       if (i3d.lt.2) go to 12
  125  continue
c  number of pb iter depends on distance
       nitpbu=nitpb(1)
       if(dlta(no,ne).gt.delt1) nitpbu=nitpb(2)
       call minima(no,isp,ttime,nitpbu,jpb)
       if(jpb.lt.nitpbu) goto 130
         write(26,2601) no,stn(ns),jpb,nitpbu
 2601    format(' Minima: no=',i4,2x,a4,', used maximum ',
     2   'number PB iter.: j=',i3,', nitpb=',i3)
  130  continue
   12  continue
c
cfhRT -end of thurber raytracing
cfhRT now for paraxial raytracing
cfh trvhet had to be modified to take one observation at a time
cfh  - also change neit to ne
      else if (i3d.eq.4)  then
         call trvhet(ne,no,ttime,iprep)
      end if 
c 
cfhRT derivatives are again computed with the old simulps code for both raytracers
cfhRT make sure that the raypath sampling in the paraxial raytracer is ok for that
c
cfhRT unsuccessful RKP shooting leads to using ART-PB instead...
      call ttmder(ne,no,0,ttime)
c        write(16,305)ns,ttime
c305    format(' for station ',i8,' the ttime= ',f7.3)
      ttc(no)=ttime
      if((niti.lt.nitmax).or.(kout.lt.3)) goto 10
cfhRT  az and toa are defined in common. paraxial raytracer also gives
cfhRT  azimuth and take-off angle, so we could skip this calc.?
cfh -- but RKP defines az = phi differently: phi = 90 + azim, if phi > 360, phi = phi -360
        x1=rp(1,1,no)
        x2=rp(1,2,no)
        y1=rp(2,1,no)
        y2=rp(2,2,no)
        z1=rp(3,1,no)
        z2=rp(3,2,no)
        call aztoa(x1,x2,y1,y2,z1,z2,xr,yr,azim,tkofan)
        az(no)=azim
        toa(no)=tkofan
 10   continue                  !end of loop over observations
cfhRT for paraxial raytracer now write out final angles
      if (i3d.eq.4) then
         call finparax(ne,nobs)
         iprep=1
      endif
c
      jfl=2
      jflag=2
c  calculate weights and apply to hypocenter matrix;
c  calculate rms residual
c  place derivs + resids into a-matrix for inversion
      call wthyp(ne,nobs,ah,rmswt,nwr,w)
      if((kout2.eq.0).or.(kout2.eq.2)) write(16,1001) nitl,xe,ye,ze,
     2 seco(ne),rmswt
 1001 format('  location, o.t., rmswt on iteration',i3,' are -',3f7.2,
     2 ',',f7.2,',',f6.3)
      if(nwr.lt.4) return
c  terminate or continue iterating?
c  check for worsened location, over 2 iterations
      if(rmswt.gt.rmslst) then
        if(rmswt.gt.rmsbfl) then
          evc(1,ne)=xbflst
          evc(2,ne)=ybflst
          evc(3,ne)=zbflst
          seco(ne)=otbflt
          rmswt=rmsbfl
          goto 999
        endif
      endif
      if (nitl.ge.nitloc) go to 999
c ************* wap *****************************
c  if rapid convergence, continue despite rmscut
c     write(16,1666) rmswt,rmslst,rmscut
 1666 format(' rmswt,rmslst,rmscut ',3f6.3)
      if((rmslst-rmswt).ge.0.02) goto 15
c  quit if 2 iterations better than rmscut
      if (rmswt.le.rmscut) then
        if(rmslst.le.rmscut) then
          if(rmsbfl.le.rmscut) goto 999
        endif
      endif
c  WP added this to end iteration when convergence in reached.
c  GE terminate when no improvement of rms in 2 iterations
      if((rmslst-rmswt).lt.0.005) then
        if((rmsbfl-rmslst).lt.0.005) goto 999
      endif
      if(ne.gt.neqs) goto 15
      if(xmov.lt.0.05) goto 999
c  find new hypocenter
   15 continue
      xbflst=xlst
      ybflst=ylst
      zbflst=zlst
      otbflt=otlst
      rmsbfl=rmslst
      xlst=evc(1,ne)
      ylst=evc(2,ne)
      zlst=evc(3,ne)
      otlst=seco(ne)
      rmslst=rmswt
      nitl=nitl+1
c  determine singular value decomposition of hypocenter matrix
      call fksvd(ah,s,v,maxobs,4,nobs,4,1,.false.,.true.)
c  calculate adjustments and apply to hypocentral parameters
c  determine number of non-zero singular values
      nfre=0
      do 20 i=1,4
      if (s(i).gt.eigtol) nfre=nfre+1
   20 continue
c  calculate adjustments
c   only adjust origin time for blast
      if(ne.gt.neqs) then
c    calculate adjustment
        adj(1)=0.0
        do 25 j=1,nfre
        adj(1)=adj(1)+v(1,j)*ah(j,5)/s(j)
   25   continue
c    apply adjustment
        seco(ne)=seco(ne)+adj(1)
c  calculate adjustments for earthquakes
      else
        do 30 i=1,4
           adj(i)=0.0
           do 29 j=1,nfre
              adj(i)=adj(i)+v(i,j)*ah(j,5)/s(j)
   29      continue
   30   continue
c  check for poor depth constraint
        adj(4)=adj(4)*0.75    !Cliff uses 0.75, Bill uses 0.50
        if (s(4).le.eigtol) adj(4)=0.0
        if(s(4).le.eigtol) write(16,9876)
 9876 format(' *** poorly constrained depth ***')
c  apply adjustments to hypocentral parameters
        seco(ne)=seco(ne)+adj(1)
        do 40 i=1,3
           i1=i+1
c  check to see that hyp. adjustment is less than dxmax
           if(adj(i1)) 32,38,34
   32      if(adj(i1).lt.(-dxmax)) adj(i1)= -dxmax
           goto 38
   34      if(adj(i1).gt.dxmax) adj(i1)=dxmax
   38      evc(i,ne)=evc(i,ne)+adj(i1)
   40   continue
c  check for depth less than zmin
      if(evc(3,ne).gt.zmin) goto 45
        write(16,1610) zmin,evc(3,ne)
 1610 format(5x,'***** Depth Fixed at ZMIN *****',' zmin, zadj= ',2f7.2)
        evc(3,ne)=zmin
   45 end if
      dx=xlst-evc(1,ne)
      dy=ylst-evc(2,ne)
      dz=zlst-evc(3,ne)
      dot=otlst-seco(ne)
      xmov=sqrt(dx*dx + dy*dy + dz*dz + dot*dot)
      go to 1
  999  continue
c  residual reduction achieved - proceed to parameter separation
c  if location only, calculate ssqr here instead of parsep
      if(nitmax.gt.0) goto 48
c  compute contribution to ssqr
      sqwtsh=sqrt(wtsht)
      totrms(ne)=0.0
      do 18 i=1,nobs
         resi=res(i)
         resi2=resi*resi
         totrms(ne)=totrms(ne)+resi2
         rrw=resi2*w(i)
         ssqrw=ssqrw+rrw
         wnobt=wnobt+w(i)
         if(intsp(i,ne).eq.0) then
c  P arrival
            ssqrwp=ssqrwp+rrw
            wnobtp=wnobtp+w(i)
         else
c  S arrival
            ssqrws=ssqrws+rrw
            wnobts=wnobts+w(i)
         endif
         if (ne.gt.netemp) resi=resi*sqwtsh
         ssqr=ssqr+resi*resi
   18 continue
      totrms(ne)=sqrt(totrms(ne)/nobs)
c
c  Covariance calculations for hypocenter parameters
c  Compute diagonal elements of covariance matrix
c  set reading error (sec).
   48 sig=rderr
      sig2=sig*sig
c  initially set st.er. to zero (so correct for blasts)
      do 47 i=1,4
         ster(i)=0.0
   47 continue
      ni=4
      if (ne.gt.neqs) ni=1
      do 50 i=1,ni
         sum=0.0
         do 60 j=1,4
            sss=s(j)
c fix s so won't get arithmetic fault
            if(sss.eq.0.0) sss=0.00000001
            sum=sum+v(i,j)*v(i,j)/(sss*sss)
   60    continue
         cov(i)=sig2*sum
         ster(i)=sqrt(cov(i))
   50 continue
      wrmsr(ne)=rmswt
      write(16,1099)ne,nitl,rmswt,seco(ne),(evc(j,ne),j=1,3),ster
 1099 format(1x,'event ',i3,';nit=',i3,';wtd rms res=',f6.3,
     2 ';ot,x,y,z =',f6.2,2f8.2,f7.2,
     3 '; St.Er.=',f8.4,'(ot)',f7.3,'(x)',f7.3,'(y)',
     4 f9.3,'(z)')
c  this is from Fred Klein's program HYPOINVERSE (6mar1989)
c  set Fred's "n" to 4
      n=4
C--SKIP THE REMAINING CALCS IF THEY ARE NOT NEEDED FOR FINAL OUTPUT
      if((kout.lt.5).or.(ne.gt.neqs)) goto 9999
C--CALCULATE COVARIANCE MATRIX AS SIGMA**2 * V * EIGVAL**-2 * VT
C--ESTIMATED ARRIVAL TIME ERROR
        SIGSQ=RDERR*rderr+ERCOF*rmswt*rmswt
        TEMP=EIGTOL**2
        DO 252 I=1,4
        DO 250 J=1,I
        COVAR(I,J)=0.
        IF (I.GT.N .OR. J.GT.N) THEN
          IF (I.EQ.J) COVAR(I,J)=999.
          GO TO 250
        END IF
        DO 245 L=1,N
c 245 COVAR(I,J)=COVAR(I,J)+V(I,L)*V(J,L)/(s(L)**2+TEMP)
245     COVAR(I,J)=COVAR(I,J)+V(I,L)*V(J,L)/(s(L)*s(L))
        COVAR(I,J)=SIGSQ*COVAR(I,J)
250     COVAR(J,I)=COVAR(I,J)
252     CONTINUE

C--EVALUATE THE HYPOCENTER ERROR ELLIPSE BY DIAGONALIZING
C--THE SPATIAL PART OF THE COVARIANCE MATRIX
C--USE ac AS TEMPORARY STORAGE
        DO 257 I=1,3
        DO 255 J=1,3
255     ac(I,J)=COVAR(I+1,J+1)
257     CONTINUE
        CALL fkSVD (ac,SERR,V3,MMAX,3,3,3,0,.FALSE.,.TRUE.)
        DO 260 I=1,3
        SERR(I)=SQRT(SERR(I))
        IF (SERR(I).GT.99.) SERR(I)=99.
260     CONTINUE

C--COMPUTE ERH AND ERZ AS THE LARGEST OF THE HORIZ AND VERTICAL
C--PROJECTIONS OF THE PRINCIPAL STANDARD ERRORS
        ERH=0.
        ERZ=0.
        DO 265 I=1,3
        TEMP=SERR(I)*SQRT(V3(1,I)**2+V3(2,I)**2)
        IF (TEMP.GT.ERH) ERH=TEMP
        TEMP=SERR(I)*ABS(V3(3,I))
        IF (TEMP.GT.ERZ) ERZ=TEMP
265     CONTINUE
        IF (ERZ.GT.99.) ERZ=99.
        IF (ERH.GT.99.) ERH=99.
c     write(16,1098)ne,nitl,rmswt,seco(ne),(evc(j,ne),j=1,3),
c    2 erh,erz
 1098 format(2x,'event ',i3,'; nit=',i3,'; wtd rms res=',f6.3,
     2 '; ot,x,y,z =',4f6.2,
     3 '; Hypinv.code St.Er.=',f10.3,' ERH',6x,f9.3,'ERZ')

C--NOW CALC THE ORIENTATIONS OF THE PRINCIPAL STD ERRORS
        DO 290 J=1,3
        IAZ(J)=0
        IDIP(J)=90
        TEMP=SQRT(V3(1,J)**2+V3(2,J)**2)
        IF (TEMP.EQ.0.) GO TO 290
c  Change indices because different coordinates. Hypoinverse array has 1=lat, 2=long(west=positive)
c     IAZ(J)=ATAN2(-V3(2,J),V3(1,J))/drad
c   Simul has evc(1)=west, evc(2)=north
c   and allows rotation
      iaz(j)=(atan2(-v3(1,j),v3(2,j))-rota)/drad
        IDIP(J)=ATAN2(V3(3,J),TEMP)/drad
        IF (IDIP(J).LT.0) THEN
          IDIP(J)=-IDIP(J)
          IAZ(J)=IAZ(J)+180
        END IF
        IF (IAZ(J).LT.0) IAZ(J)=IAZ(J)+360
        if(iaz(j).gt.360) iaz(j)=iaz(j)-360
290     CONTINUE
      call out(ne,niti,nwr,serr,iaz,idip,erh,erz)
 9999 continue
cdmep update dlta
      do 300 no=1,nobs
         ns=isto(no,ne)
         dx=evc(1,ne)-stc(1,ns)
         dy=evc(2,ne)-stc(2,ns)
         dlta(no,ne)=sqrt(dx*dx+dy*dy)
  300 continue
      return
c***** end of subroutine loceqk *****
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine ludecp(a,ul,n,ier)
c  routine to perform cholesky decomposition
c
c  common block variables:
      include 'simulps_common.inc'
c
      dimension a(mgsol),ul(mgsol)
      data zero,one,four,sixtn,sixth/0.0,1.,4.,16.,.0625/
      d1=one
      d2=zero
      rn=one/(n*sixtn)
      ip=1
      ier=0
      do 45 i=1,n
         iq=ip
         ir=1
            do 40 j=1,i
            x=a(ip)
            if(j.eq.1) go to 10
               do 5 k=iq,ip1
               x=x-ul(k)*ul(ir)
               ir=ir+1
    5       continue
   10       if (i.ne.j) go to 30
            d1=d1*x
            if (a(ip)+x*rn.le.a(ip)) go to 50
   15       if (abs(d1).le.one) go to 20
            d1=d1*sixth
            d2=d2+four
            go to 15
   20       if (abs(d1).ge.sixth) go to 25
            d1=d1*sixtn
            d2=d2-four
            go to 20
   25       ul(ip)=one/sqrt(x)
            go to 35
   30       ul(ip)=x*ul(ir)
   35       ip1=ip
            ip=ip+1
            ir=ir+1
   40    continue
   45 continue
      go to 9005
   50 ier=129
 9000 continue
 9005 return
c* * * end of subroutine ludecp * * *
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine luelmp(a,b,n,x)
c  routine to perform elimination part of solution of ax=b
c
c  common block variables:
      include 'simulps_common.inc'
c
      dimension a(mgsol),b(mxpri1),x(mxpri1)
      data zero/0./
c  solution of ly=b
      ip=1
      kw=0
      do 15 i=1,n
         t=b(i)
         im1=i-1
         if (kw.eq.0) go to 9
         ip=ip+kw-1
         do 5 k=kw,im1
            t=t-a(ip)*x(k)
            ip=ip+1
    5    continue
         go to 10
    9    if (t.ne.zero) kw=i
         ip=ip+im1
   10    x(i)=t*a(ip)
         ip=ip+1
   15 continue
c  solution of ux=y
      n1=n+1
      do 30 i=1,n
         ii=n1-i
         ip=ip-1
         is=ip
         iq=ii+1
         t=x(ii)
         if (n.lt.iq) go to 25
         kk=n
         do 20 k=iq,n
            t=t-a(is)*x(kk)
            kk=kk-1
            is=is-kk
   20    continue
   25    x(ii)=t*a(is)
   30 continue
      return
c***** end of subroutine luelmp *****
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine medder(ne)
c routine to add medium derivatives from shot data
c  to the medium matrix dtmp for inversion
c
c  common block variables:
      include 'simulps_common.inc'
      common/wtpars/w(maxobs)
c
      nobs=kobs(ne)
      rms=0.0
c  compute weighting
c     as in parsep
c
      nwr=0
      wnorm=0.0
      do 13 j=1,nobs
c  reading weight
         w(j)=wt(j,ne)
c  residual weighting
c  downweighting(linear) 0 to 98% res1 to res2, 98 to 100% res2 to res3
         ares=abs(res(j))
         if(ares.le.res2) then
            wr=1.0-(ares-res1)*dres12
            if (wr.gt.1.0) wr=1.0
         else
            if(res3.gt.res2) then
               wr=0.02-(ares-res2)*dres23
               if (wr.lt.0.0) wr=0.0
            else
               wr=0.0
            endif
         endif
c  distance weighting
         wd=1.0-(dlta(j,ne)-delt1)*ddlt
         if (wd.gt.1.0) wd=1.0
         if (wd.lt.0.0) wd=0.0
c  unnormalized weight
         w(j)=w(j)*wr*wd
c  19-jul-1983 Change normalizing factor
         wnorm=wnorm+w(j)
c        wnorm=wnorm+w(j)*w(j)
         if (w(j).gt.0.0) nwr=nwr+1
  13  continue
c  check to be sure 4 or more readings with nonzero weight
c    dmep 02dec83
      if(nwr.ge.4) goto 12
      write(16,1612)
 1612 format(' ****** less than 4 readings with nonzero weights')
c  normalize weights
c    19-jul-1983 Change in normalizing factor
c     wfac=sqrt(nwr/wnorm)
   12 wfac=nwr/wnorm
      if(ne.gt.netemp) wfac=wfac*wtsht
      nswrt=nswrt+nwr
      do 14 j=1,nobs
         w(j)=w(j)*wfac
   14 continue
c  store residuals and compute rms residual
c  compute weighted rms residual as in wthyp
      wnorm=0.0
      rmswt=0.0
      do 10 ii=1,nobs
         resp(ii)= res(ii)*w(ii)
         w2=w(ii)*w(ii)
         rmswt= rmswt + w2*res(ii)* res(ii)
         wnorm=wnorm+w2
   10 continue
      rmswt=sqrt(rmswt/wnorm)
      wrmsr(ne)=rmswt
      write(16,1001) ne,rmswt
 1001 format(' explosion ',i3,'; weighted rms residual =',f7.3)
c  store partial derivatives
      nono=npari*nobs
      do 20 j=1,nono
         i=(j-1)/npari+1
         dtmp(j)=dtm(j)*w(i)
c check station corrections
c        im=j-(i-1)*npari
c        is=mdexfx(im)-nparv
c        if((is.le.0).or.(dtm(j).ne.1.0000)) goto 20
c        write(16,1620) is,j,dtm(j),dtmp(j)
c1620    format(' MEDDER:is,j,dtm,dtmp',i5,i10,2f14.6)
   20 continue
c  zero out dtm matrix
      do 45 m=1,nono
         dtm(m)=0.0
   45 continue
      return
c***** end of subroutine medder *****
      end
c
c-------------------------------------------------------
c
      subroutine minima(no,isp,fstime,npbmax,jpb)
c
c*****this routine finds the minimum path using pseudo-bending
c
c  common block variables:
      common/pathm/x(130),y(130),z(130),v(130),vq(130),tra,qtra,n,nn
      common/temp/xtemp(130),ytemp(130),ztemp(130),rtemp(130),ttemp(130)
      common/pat/xt(130),yt(130),zt(130),rt(130),rm,rs,rtm,rts,nt,j
      common/upb/ upbtot,nupb
      include 'simulps_common.inc'
c
      tra=fstime
      n=nrp(no)
c
      do 20 i=1,n
         x(i)=rp(1,i,no)
         y(i)=rp(2,i,no)
         z(i)=rp(3,i,no)
   20 continue
c
      do 21 i=1,n
         xp=x(i)
         yp=y(i)
         zp=z(i)
         xtemp(i)=xp
         ytemp(i)=yp
         ztemp(i)=zp
         call vel3(isp,xp,yp,zp,vp)
         v(i)=vp
         call vel3(1,xp,yp,zp,vpp)
         vq(i)=vpp
   21 continue
      call travel
c
      nn=n-1
      nupb=nupb+1
c     write(61,6000) nupb
 6000 format(' nubp=',i8)
      do 100 j=1,npbmax
         ta=tra
         call bend(isp,xfac)
         call travel
         deltat=ta-tra
c
         if (deltat.lt.0.0) go to 102
         do 22 i=1,n
            x(i)=xtemp(i)
            y(i)=ytemp(i)
            z(i)=ztemp(i)
   22    continue
         if(deltat.le.tlim) go to 102
100   continue
c
c
  102 continue
      if(iuseq .eq. 1) then
        call qtravel
      endif


      if(j.gt.npbmax) j=npbmax
      upbtot=upbtot + float(j)
      jpb=j
  105 do 300 i=1,n
         rp(1,i,no)=x(i)
         rp(2,i,no)=y(i)
         rp(3,i,no)=z(i)
300   continue
      if(iuseq .eq. 0) then
        fstime=tra
      else
      fstime=qtra
      endif
c
      return
c ***** end of subroutine minima *****
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine out(ne,nit,nwr,serr,iaz,idip,erh,erz)
c
c  declaration statements:
      integer iaz(3),idip(3)
      real serr(3)
        DIMENSION KSIG(3)
        CHARACTER KSFL*1
      character*1 is,ie
cfa   character*9 day
cfa   character*8 tm
      Character DateToday*11,TimeNow*8
c
c  common block variables:
      include 'simulps_common.inc'
c
cfa   call date(day)
cfa   call time(tm)
c
c  Write new hypocenters to file12 in hypoinverse format
cfa
      Call DateTime(DateToday,TimeNow)
      if(ne.eq.1) write(12,1201) nit,DateToday,TimeNow
 1201 format(' iteration step=',i3,'  computed',1x,a11,1x,a8,
     1 ' hyp error ellipse',
     2 /,' Date HrMn Sec   Lat    Long DepthMgNwr',7x,'Rms',
     3 'Az1D1Ser1Az2D2Ser2Mg',3x,'Ser3',
     4 5x,'Erh Erz')
      i=ne
      call latlon(evc(1,i),evc(2,i),lat,xlat,lon,xlon)
c       convert origin time into gmt
        sec= seco(i)
        nin=mino(i)
 210    if(sec.lt.0) goto 230
 220    if(sec.lt.60) goto 240
        sec=sec-60
        nin=nin+1
        goto 220
  230   sec=sec+60
        nin=nin-1
        goto 210
  240 continue
c       write(12,260) iyrmo(i),iday(i),ihr(i),nin,sec,lat,xlat,lon,
c    * xlon,evc(3,i),rmag(i)
c260  format(a4,a2,1x,a2,i2,1x,f5.2,i3,1x,f5.2,1x,i3,1x,f5.2,2f7.2)
c
c  use Fred Klein's HYPOINVERSE format from his routine HYSUM
C--CALLED BY HYPOINVERSE TO OUTPUT SUMMARY DATA
c  set latitude to be north and longitude to be west
      is='n'
      ie='w'
c  use hypoinverse format
c  NOTE: I've left all the variables in the HYPOINVERSE output
c  for possible later use.
c  However those that are not calculated will be zero.
      ih71s=0
      iunit=12
c  Find minimum source-receiver distance
      do 5001 j=1,kobs(i)
      if(dlta(j,i).lt.dmin) dmin=dlta(j,i)
 5001 continue
C--CONVERT SOME DATA TO INTEGER FOR OUTPUT
        KLTM=xlat*100.+.5
        KLNM=xlon*100.+.5
        KQ=sec*100.+.5
        KZ=evc(3,i)*100.+.5
        LFMAG=rmag(i)*10.+.5
        LXMAG=rmag(i)*10.+.5
        KDMIN=DMIN+.5
        IF (KDMIN.GT.999) KDMIN=999
        KRMS=wrmsr(i)*100.+.5
        KERH=ERH*100.+.5
        KERZ=ERZ*100.+.5
c  nfm not used
c       NFM=NFRM
c       IF (NFM.GT.99) NFM=99
c use rmag(i) as input from for004 for both mxmag and mfmag
        mxmag=nint(10.0*rmag(i))
        mfmag=nint(10.0*rmag(i))
        IXMAG=.1*MXMAG+.5
        IF (IXMAG.GT.999) IXMAG=999
        IFMAG=.1*MFMAG+.5
        IF (IFMAG.GT.999) IFMAG=999
        DO 10 k=1,3
10      KSIG(k)=SERR(k)*100.
C--WRITE A SUMMARY RECORD
C--HYPO71 FORMAT
c  maxgap is not used although it could be calculated from az 
c  array (in AZTOA and LOCEQK)
        IF (IH71S.EQ.2 .AND. IUNIT.EQ.12) THEN
          KSFL=' '
          IF (NWS.GT.0) KSFL='S'
          WRITE (12,1002) iyrmo(i),iday(i),ihr(i),nin,sec,LAT,IS,
c       2 xlat,LON,IE,xlon,evc(3,i),rmag(i),nwr,MAXGAP,DMIN,
     2 xlat,LON,IE,xlon,evc(3,i),rmag(i),nwr,DMIN,
     3 wrmsr(i),erh,erz,ksfl
1002    FORMAT (a4,a2,1x,a2,I2,F6.2,I3,A1,F5.2,I4,A1,F5.2,2F7.2,
c     2 I3,I4,F5.1,F5.2,2F5.1,1X,A1)
     2 I3,4x,F5.1,F5.2,2F5.1,1X,A1)
        ELSE

C--HYPOINVERSE FORMAT
          WRITE (IUNIT,1001) iyrmo(i),iday(i),ihr(i),nin, KQ,LAT,IS,
c    2 KLTM,LON,IE,KLNM,KZ, LXMAG,nwr,MAXGAP, KDMIN,KRMS,
     2 KLTM,LON,IE,KLNM,KZ, LXMAG,nwr,KDMIN,KRMS,
c    3 (IAZ(k),IDIP(k),KSIG(k),k=1,2), LFMAG,REMK,KSIG(3),
     3 (IAZ(k),IDIP(k),KSIG(k),k=1,2), LFMAG,KSIG(3),
c    4 RMK1,RMK2,NWS, KERH,KERZ,NFM, IXMAG,IFMAG,KXMMAD,KFMMAD,
     4 KERH,KERZ, IXMAG,IFMAG
1001      FORMAT (a4,a2,a2,i2, I4,I2,A1,
c    2 I4,I3,A1,I4,I5, I2,3I3,I4,
     2 I4,I3,A1,I4,I5, I2,i3,3x,i3,I4,
c    3 2(I3,I2,I4), I2,A3,I4,
     3 2(I3,I2,I4), I2,3x,I4,
c    4 2A1,I2, 2I4,I2, 4I3,
     4 2x,2x,2I4,2x, 2i3)
c    5 A3,A1, 3A1, I1,I3)
        END IF
c***** end of subroutine out *****
      return
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine outadj(nit,istop,istop1)
c
c  declaration statements:
      real fillin(20)
      real x(5000)
      parameter(zero=0.0,izero=0)
      character*5 vtype(3)
c
c  common block variables:
      include 'simulps_common.inc'
c
      vtype(1)='P-Vel'
      vtype(2)='Vp/Vs'
      vtype(3)='Q    '
c
      write(16,1000) ssqr,nobt,nobtp,nobts,nobteq,nobtex,wtsht,
     2 nwrt,nswrt
       write(36,1000) ssqr,nobt,nobtp,nobts,nobteq,nobtex,wtsht,
     2 nwrt,nswrt
 1000 format(/,' sum of squared residuals =',f10.3,
     2 '; total number of observations =',i6,
     3 ', P obs =',i6,', S obs =',i6,
     4 /,53x,'earthquake obs.=',i6,', explosion obs.=',i6,
     5 ' (wtsht=',f5.2,')',
     6 /,20x,'with non-zero wt:',t65,'nwrt=',i6,',',10x,
     7 'nswrt=',i6)
      rmsres=sqrt(ssqr/float(nobt))
      rmssep=0.0
      do 88 ne=1,nevt
         rmssep=rmssep+seprms(ne)
   88 continue
      rmssep=sqrt(rmssep/nobt)
      write(16,1999) rmsres,rmssep
 1999 format(' rms residual =',f8.5,'   model rms =',f8.5)
      rmsw=sqrt(ssqrw/wnobt)
      write(16,1600) ssqrw,wnobt,rmsw
 1600 format(/,' weighted sum of squared residuals =',f10.3,
     * '; weighted total number of obs =',f10.3,
     * '; weighted rms res = ',f10.5)
c  store ssqr and number of velocity parameters for f-test
c  unless last step was backup
      if(istop1.ge.2) goto  5
      ssqrw1=ssqrw
      ssqr1=ssqr
      mbl1=mbl
      var1=var
      varw1=varw
      ndof1=ndof
      wndof1=wndof
    5 if(invdel.eq.0)go to 123
c=================================break================================
c  Output station corrections
      write(16,111)
  111 format(/,6x,'STATION CORRECTIONS AND ADJUSTMENTS',
     2 /,' station  Pdelay Adjusted S-Pdelay Adjusted')
      do 40 n=1,nsts
         nv=n+nparv
         nv1=nv+nsts
         if(hit(nv).lt.hitct.and.hit(nv1).lt.hitct) go to 40
         if(hit(nv).lt.hitct) vadj(nv)=0.
         if(hit(nv1).lt.hitct) vadj(nv1)=0.
c        write(16,120)stn(n),vadj(nv),vadj(nv1)
         write(16,130) stn(n),pdl(n),vadj(nv),sdl(n),vadj(nv1)
  130    format(1x,a4,2(f10.7,f10.7))
  120    format(3(1x,a4,1x,2f10.3))
   40 continue
c     write(16,110)
  110 format(/,' station corrections')
c     write(16,120)(stn(i),pdl(i),sdl(i),i=1,nsts)
 123  continue
c
      if(nparvi.eq.0) goto 999
c
      do 919 i=1,nx
         fillin(i)=0.0
  919 continue
      nx1=nx-1
      ny1=ny-1
      nz1=nz-1
c
      do 25 kv=1,iuses
         ivtype=kv
         if(iuseq.eq.1) ivtype=3
         if((kv.eq.1).and.(iusep.eq.0)) goto 25
c
c        write(16,1688) nit
c1688    format(' outadj, nit=',i5)
         if(nit.gt.1) goto 488
         write(16,1633)
 1633    format(/,' DERIVATIVE WEIGHT SUM ')
         do 486 k=2,nz1
            k2=k + (kv-1)*nz
c          if(iuseq.eq.0) then
c              write(16,1009) k,vtype(kv),zn(k)
c          else
c              write(16,1009) k,vtype(3),zn(k)
c          endif
            write(16,1009) k,vtype(ivtype),zn(k)
            kk=k+(kv-1)*nz2
            do 485 j=2,ny1
               n1=(kk-2)*nxy2+(j-2)*nx2+1
               n2=n1+nx2-1
               write(16,1635)(hit(i),i=n1,n2),zero
 1635          format(' 0.',18f7.0)
               do 484 i=n1,n2
                  sumhit(k2)=sumhit(k2)+hit(i)
  484          continue
  485       continue
  486    continue
  488    continue
c
         write(16,1001)
 1001    format(/,'  velocity model changes')
         do 12 k=2,nz1
            k2=k + (kv-1)*nz
            if(sumhit(k2).eq.0.0) goto 12
c          if(iuseq.eq.0) then
c              write(16,1009) k,vtype(kv),zn(k)
c          else
c              write(16,1009) k,vtype(3),zn(k)
c          endif
            write(16,1009) k,vtype(ivtype),zn(k)
            write(16,6002) (fillin(i),i=1,nx)
            kk=k + (kv-1)*nz2
            do 10 j=2,ny1
               j1=(kk-2)*nxy2+(j-2)*nx2+1
               j2=j1+nx2-1
               if(iuseq.eq.0) then
                 write(16,1003) (vadj(i),i=j1,j2),fillin(1)
               else
                 write(16,1003) (qadj(i),i=j1,j2),fillin(1)
               endif
   10       continue
            write(16,6002) (fillin(i),i=1,nx)
   12    continue
 6002    format(20f7.2)
 6003    format(20f7.1)
 1003    format('   0.00',20f7.2)
         if(iuseq.eq.0) then
           write(16,2001)
         else
           write(16,2002)
         endif
 2001    format(/,' corrected velocity model',/)
 2002    format(/,' corrected Q model',/)
         do 23 k=2,nz1
            k2=k + (kv-1)*nz
            if(sumhit(k2).eq.0.0) goto 23
c          if(iuseq.eq.0) then
c              write(16,1009) k,vtype(kv),zn(k)
c          else
c              write(16,1009) k,vtype(3),zn(k)
c          endif
            write(16,1009) k,vtype(ivtype),zn(k)
 1009       format(/,' layer',i3,5x,a5,' nodes',10x,'z =',f7.1)
            do 22 j=1,ny
               if(kv.eq.1) then
               if(iuseq.eq.0) then
                 write(16,6002) (vel(i,j,k2),i=1,nx)
               else
                 write(16,6003) (qval(i,j,k),i=1,nx)
               endif
             endif
               if(kv.eq.2) write(16,6002) (vpvs(i,j,k),i=1,nx)
  22        continue
  23     continue
c  Find average and variance (section taken from COMPVEL.FOR)
         write(16,3030)
 3030    format(/,'  STATISTICS, by Depth, excluding edge nodes')
         iv=0
         varv=0.0
         do 200 k=2,nz1
c          if(iuseq.eq.0) then
c              write(16,3015) zn(k),vtype(kv)
c          else
c              write(16,3015) zn(k),vtype(3)
c          endif
            write(16,3015) zn(k),vtype(ivtype)
            k2=k+(kv-1)*nz
            ix=0
            do 190 j=2,ny1
               do 180 i=2,nx1
                  ix=ix+1
                  if(kv.eq.1) then
                   if(iuseq.eq.0) then
                       x(ix)=vel(i,j,k2)
                   else
                       x(ix)=qval(i,j,k)
                   endif
                  else
                     x(ix)=vpvs(i,j,k)
                  endif
  180          continue
  190       continue
            call avsd(0.00,x,ix,sd,av,devtot)
            iv=iv+ix 
            varv=varv+devtot
            write(16,3017) ix,av,sd
  200    continue
         varv=varv/iv
         sdv=sqrt(varv)
       if(iuseq.eq.0) then
           write(16,3031)
       else
           write(16,3032)
       endif
 3031    format(/,' VARIANCE OF THIS VELOCITY MODEL')
 3032    format(/,' VARIANCE OF THIS Q MODEL')
         write(36,3018) iv,vtype(ivtype),varv,sdv
         write(16,3018) iv,vtype(ivtype),varv,sdv
 3015    format(/,' grid z=',f8.2,5x,a5)
 3017    format('  For ', i5,' gridpts, av=',f7.2,', sd=',f7.3)
 3018    format('  For all',i5,1x,a5,' gridpts, variance=',f12.5,
     2    ', sd=',f10.3)
c
   25 continue
      goto 999     !skip to make output shorter
c     if (nitloc.eq.0.or.neqs.eq.0) go to 999
c     write(16,1004)
c1004 format(/,' new hypocenters',/,4x,'number',5x,'ot',7x,'x',8x,
c    2 'y',8x,'z')
c     do 20 i=1,neqs
c        write(16,1008) I,SECo(i),(evc(j,i), j=1,3)
c1008    format(I9,4F9.2)
c  20 continue
  999 continue
c***** end of subroutine outadj *****
      return
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine outend
c  Routine for final output.  See description of output files
c  and "kout" output control parameter at the beginning of
c  the program.
c
c  common block variables:
      include 'simulps_common.inc'
      common/machin/ eta,tol
        common/shortd/xltkm,xlnkm,rota,nzco,xlt,xln,snr,csr
      common/upb/ upbtot,nupb
c
c  declaration statements:
      integer lat,lon
      real xlat,xlon
      real zeroi(maxnx), velav(200)
      real type(3),ttime(maxobs),sekms(maxnx)
      data type(1),type(2),type(3) /1hP,1hS,1hQ/
      parameter(zero=0.0,izero=0)
      character*50 formvl,formrs,line,frmrs1,frmrs2,frmvl1,frmvl2
      character*2 ch2nx2
      character*1 ch2nx3,ch2nx4,cns,cew
      character*5 vtype(3)

cfa   character*9 day
cfh      character*32 ctime
cfa   character*8 tm

      Character DateToday*11,TimeNow*8

cfhek      cns='n'
cfhek      if(nzco.eq.1) cns='s'
cfhek      cew='w'
cfhek      if(nzco.eq.1) cew='e'
      vtype(1)='P-Vel'
      vtype(2)='Vp/Vs'
      vtype(3)='Q'
cfa   call date(day)
cfa   call time(tm)

      Call DateTime(DateToday,TimeNow)

cfh      call system("date > temp3")
cfh      open(321,file='temp3')
cfh      read(321,'(a32)',end=3333) ctime
cfh 3333 continue
cfh      close(321)
c
      formrs='(1x,3hres,5x,10(1h(,f5.2,1h),5x),3hres)'
      frmrs1='(1x,3hres,5x,10(1h(,f5.2,1h),5x))'
cfh format problems
c      frmrs2='(9x,2(12x),8(1h(,f5.2,1h),5x),3hres)'
      frmrs2='(9x,02(12x),08(1h(,f5.2,1h),5x),3hres)'
      if(iuseq.eq.0) then
        formvl='(1h0,f4.2,10(f6.2,1h+,f5.4),f6.2)'
        frmvl1='(1h0,f4.2,10(f6.2,1h+,f5.4))'
        frmvl2='(1h0,4x,02(12x),08(f6.2,1h+,f5.4),f6.2)'
      else
        formvl='(    f5.0,10(f6.0,1h+,f5.1),f6.0)'
        frmvl1='(    f5.0,10(f6.0,1h+,f5.0))'
        frmvl2='(1h0,4x,02(12x),08(f6.0,1h+,f5.0),f6.0)'
      endif
      do 2 i=1,20
         zeroi(i)=zero
    2 continue
c
c
      write(16,100) ssqr,nobt,nobtp,nobts,nobteq,nobtex,wtsht,
     2 nwrt,nswrt
  100 format(/,' sum of squared residuals =',f10.3,
     2 '; total number of observations =',i6,
     3 ', P obs =',i6,', S obs =',i6,
     4 /,53x,'earthquake obs.=',i6,', explosion obs.=',i6,
     5 ' (wtsht=',f5.2,')',
     6 /,20x,'with non-zero wt:',t65,'nwrt=',i6,',',10x,
     7 'nswrt=',i6)
c** change for creating synthetic data, nitmax= -1
      if(nitmax.eq.-1) goto 76
      rmsres=sqrt(ssqr/float(nobt))
      write(16,1999) rmsres
 1999 format(' rms residual =',f8.5)
      rmsw=sqrt(ssqrw/wnobt)
      write(16,1600) ssqrw,wnobt,rmsw
 1600 format(/,' weighted sum of squared residuals =',f10.3,
     * '; weighted total number of obs =',f10.3,
     * '; weighted rms res = ',f10.5)
c
c  Write new hypocenters to output(file16)
   76 write(16,1650)
 1650 format(//,15x,'FINAL LOCATIONS',//,5x,'YRMODY HRMN  SEC',
     2 2x,'LATITUDE LONGITUDE  DEPTH    MAG  NO RMSRES',5x,'x',6x,
     3 'y',6x,'z')
c  Map 0,1,2,3,4,5 to -1,0,1,2,2,3
      kkout=nint(kout*0.85)-1
      if(kkout) 301,295,290
c  Write new travel time file24 to be used as input in future runs
cfa  290 open(unit=24,status='new',file='f24.txt')
  290 open(unit=24,file='f24.txt')
cfa   if(nbls.gt.0) open(unit=28,status='new',file='f28.txt')
      if(nbls.gt.0) open(unit=28,file='f28.txt')
c  Write new hypocenters to file13 in hypo71 format
cfa  295 open(unit=13,status='new',file='finalsmpout.txt')
  295 open(unit=13,file='finalsmpout.txt')
        rewind(13)

cfa    write(13,1301) neqs,day,tm
cfh      write(13,9301) neqs,ctime
cfa 1301 format(i5,75x,'computed',2x,a9,1x,a8)
        Call DateTime(DateToday,TimeNow)
        write(13,1301) neqs,DateToday,TimeNow
 1301 format(i5,75x,'computed',2x,a11,1x,a8)

cfh 9301 format(i5,75x,'computed at ',a32)
  301 neqbl=neqs+nbls
      iunit=24
       rmsall = 0.0
      do 5 i=1,nevt
         if(i.gt.neqs) iunit=28
c  convert origin time into gmt
         sec= seco(i)
         nin=mino(i)
 210     if(sec.lt.0) goto 230
 220     if(sec.lt.60) goto 240
         sec=sec-60
         nin=nin+1
         goto 220
 230     sec=sec+60
         nin=nin-1
         goto 210
  240    call latlon(evc(1,i),evc(2,i),lat,xlat,lon,xlon)
cek
              if(lat.ge.0.and.xlat.ge.0.) then
                   cns='N'
                   latdeg=iabs(lat)
                   alatmin=abs(xlat)
              else
                   cns='S'
                   latdeg=iabs(lat)
                   alatmin=abs(xlat)
              endif
              if(lon.ge.0.and.xlon.ge.0.) then
                   cew='W'
                   londeg=iabs(lon)
                   alonmin=abs(xlon)
              else
                   cew='E'
                   londeg=iabs(lon)
                   alonmin=abs(xlon)
              endif
cek
         if(kkout) 304,303,302
c** Change for synthetic data, nitmax= -1
  302    if((i.gt.neqbl).and.(nitmax.ne.-1)) goto 303
cfh      for synth. data (nshts >0, neqs, nbls = 0) unit 28 is not opened
cfh       -  open it now
cfa      open(unit=28,status='new',file='f28.txt')
         open(unit=28,file='f28.txt')
         if(i.eq.1) then
           write(iunit,2404) iyrmo(i),iday(i),ihr(i),
     2      nin,sec,latdeg,cns,alatmin,londeg,cew,alonmin,evc(3,i),
     3      rmag(i)
cek           write(iunit,2404) iyrmo(i),iday(i),ihr(i),
cek     2      nin,sec,lat,xlat,lon,xlon,evc(3,i),rmag(i),
cek     3      day,tm
         else
           write(iunit,2401) iyrmo(i),iday(i),ihr(i),
     2      nin,sec,latdeg,cns,alatmin,londeg,cew,alonmin,evc(3,i),
     3      rmag(i)
cek           write(iunit,2401) iyrmo(i),iday(i),ihr(i),
cek     2      nin,sec,lat,xlat,lon,xlon,evc(3,i),rmag(i)
 2401    format(a4,a2,1x,a2,i2,1x,f5.2,i3,a1,f5.2,1x,i3,a1,f5.2,2f7.2)
 2404    format(a4,a2,1x,a2,i2,1x,f5.2,i3,a1,f5.2,1x,i3,a1,f5.2,2f7.2,
     2      10x,'computed',2x,a9,1x,a8)
         end if
c  write travel time observations for each event
         do 30 jobs=1,kobs(i)
            ttime(jobs)=secp(jobs,i)
            if(intsp(jobs,i).eq.0) ttime(jobs)=secp(jobs,i)-seco(i)
   30    continue
       if(iuseq.eq.0) then
         write(iunit,2402) (stn(isto(j,i)),rmk(j,i),
     2      ttime(j),j=1,kobs(i))
         write(iunit,2403)
       else
         write(iunit,2405) (stn(isto(j,i)),rmk(j,i),
     2        ttime(j),j=1,kobs(i))
         write(iunit,2406)
       endif
cek
 303    write(13,1302) iyrmo(i),iday(i),ihr(i),nin,sec,
     2    latdeg,cns,alatmin,londeg,cew,alonmin,evc(3,i),
     3    rmag(i),kobs(i),wrmsr(i)
cek 303    write(13,1302) iyrmo(i),iday(i),ihr(i),nin,sec,
cek     2      lat,xlat,lon,xlon,evc(3,i),rmag(i),kobs(i),wrmsr(i)
c
cek Now write it on file 16
  304    write(16,1602) i,iyrmo(i),iday(i),ihr(i),nin,sec,
     2      latdeg,cns,alatmin,londeg,cew,alonmin,evc(3,i),rmag(i),
     3      kobs(i),wrmsr(i),(evc(j,i),j=1,3)
cek  304    write(16,1602) i,iyrmo(i),iday(i),ihr(i),nin,sec,
cek     2      lat,xlat,lon,xlon,evc(3,i),rmag(i),kobs(i),wrmsr(i),
cek     3      (evc(j,i),j=1,3)
      rmsall = rmsall+wrmsr(i)
    5 continue
      rmsall = rmsall/nevt
      write(16,2407) rmsall
 2407 format('RMSALL:  ',f8.6)
 2402 format(6(a4,a4,f6.2))
 2403 format(4h0   )
 2405 format(6(a4,a4,f10.4))
 2406 format(6h     0)
 1302 format(a4,a2,1x,a2,i2,f6.2,1x,i2,a1,f5.2,1x,i3,a1,f5.2,f7.2,
     2 2x,f5.2,i3,9x,f5.2)
cek 1602 format(1x,i3,1x,a4,a2,1x,a2,i2,f6.2,1x,i2,'n',f5.2,1x,i3,'w',
cek     2 f5.2,f7.2,2x,f5.2,i4,f6.2,3f7.2)
 1602 format(1x,i3,1x,a4,a2,1x,a2,i2,f6.2,i3,a1,f6.2,i4,a1,
     2 f6.2,f7.2,2x,f5.2,i4,f6.2,3f7.2)
    8 if(kkout) 310,306,305
  305 close(unit=24)
  306 close(unit=13)
c
  310 continue
c  output number of observations by station
      write(16,1640)
 1640 format(//,15x,'TALLY OF OBSERVATIONS',//,1x,
     2 10('station obs  '),/,10(7x,'P  S-P'),/,
     3 10(' ==== === ==='))
c     write(16,1645) (stn(i),(nrd(i,kv),kv=1,2),i=1,nsts)
c the above statement gave nonsense in the sun compiled version
      write(16,1645) (stn(i),nrd(i,1),nrd(i,2),i=1,nsts)
 1645 format(10(1x,a4,2i4))
c
c** Change for synthetic data, nitmax= -1
      if(nitmax.le.0) goto 900
      if(invdel.eq.0)go to 353
c       output final station corrections
        write(16,110)
 110  format(//,' FINAL STATION CORRECTIONS',/,' stn  typ Delay  StErr',
     2 ' Resol   nobs  stn  stn grdpt  inv soln',/,29x,
     3         'nrd hit name num   num arry arry',/,51x,
     4                           '  num  num',/,
     5 '   _____________ _____ _____ ___ ___',5(' ____'))
        rat=ssqrw/ssqrw1
c  also write new station corrections to file22 to be used as input
c  in future runs.
cfa   if(kout.ge.2) write(22,2201) nsts,day,tm
cfa2201 format(i4,24x,'computed',2x,a9,1x,a8)
      Call DateTime(DateToday,TimeNow)
      if(kout.ge.2) write(22,2201) nsts,DateToday,TimeNow
 2201 format(i4,24x,'computed',2x,a11,1x,a8)
      do 120 i=1,nsts
         ielev= stc(3,i)*(-1.0e+03)
cek
              if(ltds(i).ge.0.and.sltm(i).ge.0.) then
                   cns='N'
                   latdeg=iabs(ltds(i))
                   alatmin=abs(sltm(i))
              else
                   cns='S'
                   latdeg=iabs(ltds(i))
                   alatmin=abs(sltm(i))
              endif
              if(lnds(i).ge.0.and.slnm(i).ge.0.) then
                   cew='W'
                   londeg=iabs(lnds(i))
                   alonmin=abs(slnm(i))
              else
                   cew='E'
                   londeg=iabs(lnds(i))
                   alonmin=abs(slnm(i))
              endif
cek fh added stc to changes from ek
         write(22,2207) stn(i),latdeg,cns,alatmin,londeg,cew,
     2      alonmin,ielev,pdl(i),sdl(i),nfixst(i),stc(1,i),stc(2,i)
cek         write(22,2207) stn(i),ltds(i),sltm(i),lnds(i),
cek     2      slnm(i),ielev,pdl(i),sdl(i),nfixst(i)
 2207    format(2x,a4,i3,a1,f6.2,i4,a1,f6.2,i5,2f5.2,i3,2f8.3)
c
cfh         write(22,2207) stn(i),ltds(i),sltm(i),lnds(i),
cfh     2      slnm(i),ielev,pdl(i),sdl(i),nfixst(i),stc(1,i),stc(2,i)
c2207    format(2x,a4,i2,1x,f5.2,i4,1x,f5.2,i5,2f7.4,i3)
cfhek 2207    format(2x,a4,1x,i2,1x,f5.2,i4,1x,f5.2,i5,2f7.4,i3,2f8.3)
         if(nfixst(i).eq.1) goto 118
         nvp=nparv+i
         if(hit(nvp).eq.zero) goto 115
         stderr(nvp)=rat*stderr(nvp)
         nvpi=ndexfx(nvp)
         nvpa=index(nvpi)
         ihit=nint(hit(nvp))
         write(16,125) stn(i),pdl(i),stderr(nvp),drm(nvp),
     2      nrd(i,1),ihit,stn(i),i,nvp,nvpi,nvpa
  125    format(1x,a4,' P  ',f7.3,2f6.3,2i4,1x,a4,2i5,2i5)
  115    continue
         if(iuses.eq.2) then
            nvs=nvp+nsts
            if(khit(nvs).eq.izero) goto 120
            stderr(nvs)=rat*stderr(nvs)
            nvsi=ndexfx(nvs)
            nvsa=index(nvsi)
            ihit=nint(hit(nvs))
            write(16,195) sdl(i),stderr(nvs),drm(nvs),
     2         nrd(i,2),ihit,stn(i),i,nvs,nvsi,nvsa
  195       format(5x,' S-P',f7.3,2f6.3,2i4,1x,a4,2i5,2i5)
         endif
  118    if(kout.lt.2) goto 120
  120 continue
      if(kout.ge.2) close(22)
c
 353  continue
c  skip velocity output if all fixed
      if(nparvi.eq.0) goto 900
c  compute vp/vs
c     if(iuses.eq.1) goto 365
c     do 360 k=1,nz
c        ks=k+nz
c        do 358 j=1,ny
c           do 356 i=1,nx
c              vpvs(i,j,k)=vel(i,j,k)/vel(i,j,ks)
c 356       continue
c 358    continue
  360 continue
c
  365 if(kout.lt.2) goto 311
c  Write the final velocity model to file23 in a format that
c  can be used as input for another run.                     
c  and to file25 in station format for plotting.
cfa   open(unit=23,status='new',file='velomod.out')
      open(unit=23,file='velomod.out')
      if(kout.gt.3)
cfa  2 open(unit=25,status='new',file='results.for_plot')
     2 open(unit=25,file='results.for_plot')
cfa      write(23,2302) bld,nx,ny,nz,iuses,day,tm
cfa 2302 format(f4.1,4i3,7x,'computed',2x,a9,1x,a8)
      Call DateTime(DateToday,TimeNow)
      write(23,2302) bld,nx,ny,nz,iuses,DateToday,TimeNow
 2302 format(f4.1,4i3,7x,'computed',2x,a11,1x,a8)
      write(23,2304) (xn(i),i=1,nx)
      write(23,2304) (yn(i),i=1,ny)
      write(23,2304) (zn(i),i=1,nz)
 2304 format(20f6.1)
      write(23,2305) izero,izero,izero
 2305 format(3i3)
      do 38 kv=1,iuses
         do 36 k=1,nz
           k2=k+(kv-1)*nz
           if((kout.gt.3).and.(k.gt.1).and.(k.lt.nz)) then
cek average layer velocity:  velav
              velav(k)=0.0
              do ii1=2,nx-1
                 do kk1=2,ny-1
                    velav(k)=velav(k)+vel(ii1,kk1,k2)
                 enddo
              enddo
              velav(k)=velav(k)/((nx-2)*(ny-2))
              write(25,2501) k,zn(k),velav(k)
cfhek 2501       format(1x,'LAYR',1x,i2,8x,f7.2,'k')
 2501         format(1x,'LAYER',1x,i2,8x,f7.2,'km',3x,f7.2,1x,'kmsec-1')
              write(25,4455)
 4455      format(1x,' long., lat., percent velo change, abs. velocity')
cek average layer velocity
           endif
          if(iuseq.eq.0) then
              write(23,2311) (vel(i,1,k2),i=1,nx)
          else
              write(23,2511) (qval(i,1,k),i=1,nx)
          endif
          do 34 j=2,ny-1
             if(iuseq.eq.0) then
                 write(23,2311) (vel(i,j,k2),i=1,nx)
             else
                 write(23,2511) (qval(i,j,k),i=1,nx)
             endif
               if((k.eq.1).or.(k.eq.nz)) goto 34
               if(kout.le.3) goto 34
               do 33 i=2,nx-1
                  call latlon(xn(i),yn(j),lat,xlat,lon,xlon)
cek
                if(lat.ge.0.and.xlat.ge.0.) then
                   cns='N'
                   lat=iabs(lat)
                   xlat=abs(xlat)
                else
                   cns='S'
                   lat=iabs(lat)
                   xlat=abs(xlat)
                endif
                if(lon.ge.0.and.xlon.ge.0.) then
                   cew='W'
                   lon=iabs(lon)
                   xlon=abs(xlon)
                else
                   cew='E'
                   lon=iabs(lon)
                   xlon=abs(xlon)
                endif
cek
                if(iuseq.eq.0) then
cek                  write(25,2502) vel(i,j,k2),lat,cns,xlat,lon,cew,xlon,
cek     2            xn(i),yn(j),zn(k)
cek 2502             format(2x,f4.2,i2,a1,f5.2,1x,i3,a1,f5.2,10x,3f7.2)
cek  velodiff= total percent velocity change relative to average layer velo
                  velodiff=100.*(vel(i,j,k2)-velav(k2))/velav(k2)
                  alonb=abs(lon+xlon/60.)
                  if(cew.eq.'E') then
                    alona=alonb
                  else
                    alona=-1.*alonb
                  endif
                  alatb=abs(lat+xlat/60.)
                  if(cns.eq.'N') then
                    alata=alatb
                  else
                    alata=-1.*alatb
                  endif
                  write(25,2503) alona,alata,velodiff,vel(i,j,k2)
 2503             format(1x,f9.4,1x,f9.4,1x,f6.2,2x,f6.2)
                else
cfh q-output not changed here for coordinates...
                    write(25,2502) qval(i,j,k),lat,xlat,lon,xlon,
     2              xn(i),yn(j),zn(k)
                endif
 2502             format(2x,f4.2,i2,f5.2,1x,i3,f5.2,10x,3f7.2)
 33            continue
 34         continue
          if(iuseq.eq.0) then
              write(23,2311) (vel(i,ny,k2),i=1,nx)
          else
              write(23,2511) (qval(i,ny,k),i=1,nx)
          endif
   36    continue
   38 continue
c  write Vp/Vs at end of new velocity file
      if(iuses.eq.1) goto 50
      do 42 k=1,nz
         write(23,2311) (vpvs(i,1,k),i=1,nx)
         do 134 j=2,ny-1
            write(23,2311) (vpvs(i,j,k),i=1,nx)
            if((k.eq.1).or.(k.eq.nz)) goto 134
            if(kout.le.3) goto 134
            do 133 i=2,nx-1
               call latlon(xn(i),yn(j),lat,xlat,lon,xlon)
cek
              if(lat.ge.0.and.xlat.ge.0.) then
                   cns='N'
                   lat=iabs(lat)
                   xlat=abs(xlat)
              else
                   cns='S'
                   lat=iabs(lat)
                   xlat=abs(xlat)
              endif
              if(lon.ge.0.and.xlon.ge.0.) then
                   cew='W'
                   lon=iabs(lon)
                   xlon=abs(xlon)
              else
                   cew='E'
                   lon=iabs(lon)
                   xlon=abs(xlon)
              endif
              write(25,2502) vpvs(i,j,k),lat,cns,xlat,lon,cew,xlon,
     2             xn(i),yn(j),zn(k)
cek
  133       continue
  134    continue
         write(23,2311) (vpvs(i,ny,k),i=1,nx)
   42 continue
c
 2311 format(20f5.2)
 2511 format(20f7.1)
   50 close(unit=23)
      if(kout.gt.3) close(unit=25)
c
  311 do 495 kv=1,iuses
         if((kv.eq.1).and.(iusep.eq.0)) goto 495
  312    if(iuseq.eq.0) then
           write(16,1000) type(kv)
       else
           write(16,1501) type(3)
       endif
 1000    format(//,' FINAL ',a1,'-VELOCITY MODEL')
 1501    format(//,' FINAL ',a1,' MODEL')
         do 10 k=1,nz
            k2=k+(kv-1)*nz
            if(sumhit(k2).eq.0.0) goto 10
c write out vp/vs
            if(kv.eq.2) then
               write(16,1009) k,vtype(kv),zn(k)
               do 9 j=1,ny
                  write(16,1001) (vpvs(i,j,k),i=1,nx)
    9          continue
            endif
          if(iuseq.eq.0) then
              write(16,1010) k,type(kv),zn(k)
          else
              write(16,1511) k,type(3),zn(k)
          endif
 1009       format(/,' layer',i3,5x,a5,' nodes',10x,'z =',f7.1)
 1010       format(/,' layer',i3,5x,a1,'-velocity nodes',10x,
     2         'z =',f7.1)
 1511       format(/,' layer',i3,5x,a1,' nodes',10x,
     2         'z =',f7.1)
            do 11 j=1,ny
             if(iuseq.eq.0) then
                 write(16,1001) (vel(i,j,k2),i=1,nx)
             else
                 write(16,1521) (qval(i,j,k),i=1,nx)
             endif
   11       continue
   10    continue
c
         write(16,1003) 
         nz1=nz-1
         ny1=ny-1
         do 22 k=2,nz1
            k2=k+(kv-1)*nz
            if(sumhit(k2).eq.0.0) goto 22
          if(iuseq.eq.0) then
              write(16,1009) k,vtype(kv),zn(k)
          else
              write(16,1009) k,vtype(3),zn(k)
          endif
            kk=k+(kv-1)*nz2
            do 20 j=2,ny1
               n1=(kk-2)*nxy2+(j-2)*nx2+1
               n2=n1+nx2-1
               write(16,1005) (khit(i),i=n1,n2),izero
   20       continue
   22    continue
c
 1003    format(/,' OBSERVATION MATRIX - KHIT - (will be 0 for',
     2      ' fixed nodes)')
 1005    format('     0',20i6)
 1001    format(20f6.2)
 1521    format(20f7.1)
c
         write(16,1633)
 1633    format(/,' DERIVATIVE WEIGHT SUM ')
         do 487 k=2,nz1
            k2=k+(kv-1)*nz
            if(sumhit(k2).eq.0.0) goto 487
          if(iuseq.eq.0) then
              write(16,1009) k,vtype(kv),zn(k)
          else
              write(16,1009) k,vtype(3),zn(k)
          endif
            kk=k+(kv-1)*nz2
            do 486 j=2,ny1
               n1=(kk-2)*nxy2+(j-2)*nx2+1
               n2=n1+nx2-1
               write(16,1635)  (hit(i),i=n1,n2),zero
 1635          format(' 0.',18f7.0)
  486       continue
  487    continue
c
cfh additional resolution output to file 25 implemented in simulps12 (0697) by ek
cfh not included here
         if(ires.eq.0) goto 495
c
c  assign -1.0 to diagonal resolution for fixed gridpoint
         do 490 n=1,nparv
            if(nfix(n).eq.1) drm(n)=-1.0
  490    continue
c
       if(iuseq.eq.0) then
           write(16,1634)
       else
           write(16,1534)
       endif
 1634    format(/,' RESOLUTION : GRIDOINT NUMBER, DIAGONAL RESOLUTION',
     2      ' ELEMENT (-1.0 indicates fixed velocity gridpoint)')
 1534    format(/,' RESOLUTION : GRIDOINT NUMBER, DIAGONAL RESOLUTION',
     2      ' ELEMENT (-1.0 indicates fixed Q gridpoint)')
       if(ires.eq.1) then
cfa         write(47,1637) nx-2,ny-2,nz-2,day,tm
cfa1637       format('1 ',3i3,6x,'Resolution Matrix, Computed',2x,a9,
            Call DateTime(DateToday,TimeNow)
            write(47,1637) nx-2,ny-2,nz-2,DateToday,TimeNow
 1637       format('1 ',3i3,6x,'Resolution Matrix, Computed',2x,a11,
     2      1x,a8)
            write(47,2304) (xn(i),i=2,nx-1)
            write(47,2304) (yn(i),i=2,ny-1)
            write(47,2304) (zn(i),i=2,nz-1)
            write(47,2305) izero,izero,izero
       endif

         do 489 k=2,nz1
            k2=k+(kv-1)*nz
c
c          to make the Q output quite similar to the velmodel
c
c
          if(iuseq.eq.0) then
              if(sumhit(k2).eq.0.0) goto 489
              write(16,1009) k,vtype(kv),zn(k)
          else
              write(16,1009) k,vtype(3),zn(k)
          endif
            kk=k+(kv-1)*nz2
            do 488 j=2,ny1
               n1=(kk-2)*nxy2+(j-2)*nx2+1
               n2=n1+nx2-1
               write(16,1636) (i,drm(i),i=n1,n2)
             if(ires.eq.1) then
                 write(47,1638) (drm(i),i=n1,n2)
             endif
 1636          format(1x,20(i5,':',f7.4))
 1638          format(20f8.4)
  488       continue
  489    continue
  495 continue
c
      rat=ssqrw/ssqrw1
      if(ires.eq.0) goto 900
c
c  write out pointer from full nodes to inversion nodes
      if(inf.le.0) goto 545
cfhdmep - unit 18 now opened in main; npar... written in rescov
cfh      open(unit=18,carriagecontrol='list',status='new')
cfh      write(18,1800) npar,nparv,npari,nparvi
cfh 1800 format(4i7,'   0.0001')
      do 540 l=1,npar
         write(18,1801) l,ndexfx(l)
 1801    format(2i7)
  540 continue
      close(18)
c
  545 do 550 l=1,npar
         if(khit(l).eq.0) goto 550
         if ((hit(l).lt.hitct).or.(nfix(l).eq.1)) go to 550
         stderr(l)=rat*stderr(l)
c        write(16,2004) l,drm(l),stderr(l)
 2004    format('  parameter number ',i3,': resolution:',f8.4,
     *      '  error (%):',f8.4)
c
  550 continue
cfh  the following output is not really readable and has very ugly
cfh  formats -skip it for now!
      goto 77
cfh
c  Also write resolution in more readable format
      if(iuseq.eq.0) then
        write(16,1026)
      else
        write(16,1526)
      endif
 1026 format(/,10x,'VELOCITY WITH STANDARD ERROR(km/s) AND RESOLUTION'
     2 /,'  where error and res are both 0.00, the node was not',
     3 ' inverted for',
     4 ', where resol.=-1.00, gridpoint velocity was fixed')
 1526 format(/,10x,'Q WITH STANDARD ERROR(km/s) AND RESOLUTION'
     2 /,'  where error and res are both 0.00, the node was not',
     3 ' inverted for',
     4 ', where resol.=-1.00, gridpoint Q was fixed')
      nx2=nx-2
      nxy2=nx2*(ny-2)
c  create proper sized format for nx nodes
cfh problem if nx2.ge.20 
      if(nx2.le.10) then
        write(line,27) nx2
        read(line,28) ch2nx2
   27   format(1x,i2)
   28   format(1x,a2)
        formvl(11:12)=ch2nx2
        formrs(14:15)=ch2nx2
      else
        nx3=nx2-10
        write(line,25) nx3
        read(line,26) ch2nx3
cfh next 2 lines
c   25   format(1x,i1)
c   26   format(1x,a1)
   25   format(1x,i2.2)
   26   format(1x,a2)
        nx4=10-nx3
        write(line,25) nx4
        read(line,26) ch2nx4
        frmvl2(9:9)=ch2nx4
        frmvl2(16:16)=ch2nx3
        frmrs2(5:5)=ch2nx4
        frmrs2(12:12)=ch2nx3
      endif
      do 75 kv=1,iuses
         do 70 k=2,nz-1
            k2=k+(kv-1)*nz 
            if(sumhit(k2).eq.0.0) goto 70
            kk=k+(kv-1)*nz2
            if(iuseq.eq.0) then
              write(16,1009) k,vtype(kv),zn(k)
            else
              write(16,1009) k,vtype(3),zn(k)
            endif
            if(kv.eq.1) then
             if(iuseq.eq.0) then
                 write(16,1021) (vel(i,1,k2),i=1,nx)
             else
                 write(16,1022) (qval(i,1,k),i=1,nx)
             endif
            else
               write(16,1021) (vpvs(i,1,k),i=1,nx)
            endif
            kp=(kk-2)*nxy2
            do 60 j=2,ny-1
               jp=(j-2)*nx2
               kjp=kp+jp
c  convert standard error from % to km/s
               do 55 i=1,nx2
                  v=vel(i+1,j,k2)
                  if(kv.eq.2) v=vpvs(i+1,j,k)
                  sekms(i)=stderr(kjp+i)*v
                  if(stderr(kjp+i).gt.tol) then
                     covdi(kjp+i)=1.0/(sekms(i)*sekms(i))
                  endif
   55          continue
               if(nx2.le.10) then
                  if(kv.eq.1) then
                   if(iuseq.eq.0) then
                       write(16,formvl) vel(1,j,k2),(vel(i+1,j,k2),
     2                    sekms(i),i=1,nx2),vel(nx,j,k2)
                   else
                       write(16,formvl) qval(1,j,k),(qval(i+1,j,k),
     2                    sekms(i),i=1,nx2),qval(nx,j,k)
                   endif
                  else
                     write(16,formvl) vpvs(1,j,k),(vpvs(i+1,j,k),
     2                  sekms(i),i=1,nx2),vpvs(nx,j,k)
                  endif
                  write(16,formrs) (drm(kjp+i),i=1,nx2)
               else
                  if(kv.eq.1) then
                   if(iuseq.eq.0) then
                       write(16,frmvl1) vel(1,j,k2),(vel(i+1,j,k2),
     2                    sekms(i),i=1,nx2),vel(nx,j,k2)
                   else
                       write(16,frmvl1) qval(1,j,k),(qval(i+1,j,k),
     2                    sekms(i),i=1,nx2),qval(nx,j,k)
                   endif
                  else
                     write(16,frmvl1) vpvs(1,j,k),(vpvs(i+1,j,k),
     2                  sekms(i),i=1,nx2),vpvs(nx,j,k)
                  endif
                  write(16,frmrs1) (drm(kjp+i),i=1,10)
                  if(kv.eq.1) then
                   if(iuseq.eq.0) then
                       write(16,frmvl2) (vel(i+1,j,k2),sekms(i),
     2                    i=11,nx2),vel(nx,j,k2)
                   else
                       write(16,frmvl2) (qval(i+1,j,k),sekms(i),
     2                    i=11,nx2),qval(nx,j,k)
                   endif
                  else
                     write(16,frmvl2) (vpvs(i+1,j,k),sekms(i),
     2                  i=11,nx2),vpvs(nx,j,k)
                  endif
                  write(16,frmrs2) (drm(kjp+i),i=11,nx2)
               endif
   60       continue
            if(kv.eq.1) then
             if(iuseq.eq.0) then
                 write(16,1024) (vel(i,ny,k2),i=1,nx)
             else
                 write(16,1025) (qval(i,ny,k),i=1,nx)
             endif
            else
               write(16,1024) (vpvs(i,ny,k),i=1,nx)
            endif
   70    continue
   75 continue
   16 format('0',f4.2,12(f7.2,'+',f4.2))
 1021 format(f5.2,f7.2,10f12.2)
 1022 format(2f6.0,10f12.2)
 1023 format(1x,'res',6x,10('(',f4.2,')',6x))
 1024 format('0',f4.2,f7.2,10f12.2)
 1025 format(2f6.0,10f12.2)
c
cfh 
   77 continue
c
c  write out 1/covariance(diag) file in format like velocity file
c  so can be plotted
      if((kout2.eq.5).and.(ires.gt.0)) then
cfa     open(unit=45,status='new',file='f45.txt')
        open(unit=45,file='f45.txt')
cfa        write(45,4502) nx,ny,nz,day,tm
cfa 4502   format(3i3,6x,'1/Diag. Covariance, Computed',2x,a9,
        Call DateTime(DateToday,TimeNow)
        write(45,4502) nx,ny,nz,DateToday,TimeNow
 4502   format(3i3,6x,'1/Diag. Covariance, Computed',2x,a11,
     2   1x,a8)
        write(45,2304) (xn(i),i=1,nx)
        write(45,2304) (yn(i),i=1,ny)
        write(45,2304) (zn(i),i=1,nz)
        write(45,2305) izero,izero,izero
      write(16,1739)
 1739 format(/,' 1/(Covariance Diag. )')
      do 746 kv=1,iuses
c  write peripheral grids to file45 also
         do 725 j=1,ny
            write(45,1737) (zeroi(i),i=1,nx)
  725    continue
         do 736 k=2,nz1
            k2=k+(kv-1)*nz
            if(sumhit(k2).gt.0.0) then
            if(iuseq.eq.0) then
                 write(16,1009) k,vtype(kv),zn(k)
            else
                 write(16,1009) k,vtype(3),zn(k)
            endif
          endif
            kk=k+(kv-1)*nz2
            write(45,1737) (zeroi(i),i=1,nx)
            do 735 j=2,ny1
               n1=(kk-2)*nxy2+(j-2)*nx2+1
               n2=n1+nx2-1
               if(sumhit(k2).gt.0.0)
     2            write(16,1737) zero,(covdi(i),i=n1,n2),zero
 1737          format(10e10.3)
               write(45,1737) zero,(covdi(i),i=n1,n2),zero
  735       continue
  736    continue
         write(45,1737) (zeroi(i),i=1,nx)
         do 745 j=1,ny
            write(45,1737) (zeroi(i),i=1,nx)
  745    continue
  746 continue
      close(45)
      endif
c
  900 continue
      if(i3d.eq.0.or.i3d.eq.4) goto 950
      upbavg=upbtot/(float(nupb))
      write(16,1665) nupb, upbavg
 1665 format(/,' For',i10,' calls to MINIMA, average number',
     2 ' psuedo-bending iter. =',f7.2)
cfh  950 continue
cfh      call system("date > temp2")
cfh      open(321,file='temp2')
cfh      read(321,'(a32)',end=3334) ctime
cfh 3334 write(16,1605) ctime
cfh 1605 format(/,' Computation finished at ',a32)
cfh      close(321)
cfa 950 call time(tm)
  950 Call DateTime(DateToday,TimeNow)
      write(16,1660) DateToday,TimeNow
 1660 format(/,' Computation finished at ',a11,1x,a8)
c***** end of subroutine outend *****
      return
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine outlis(ne)
c  write out each event in a format similar to HYPO71 listing format
c  so that it can be used as input to FPFIT fault-plane solution
c  program
c
c  declaration statements:
      parameter (one=1.00)
      character*1 phs(2),cew,cns
cfa   character*9 day
cfa   character*8 tm

      Character DateToday*11,TimeNow*8
c
c  common block variables:
      include 'simulps_common.inc'
      common/wtpars/w(maxobs)
c
      data phs/'P','S'/
c
cek distiguish North-South and East-West by N,S,E,W
      cew='W'
      cns='N'
c
      if (ne.gt.1) goto 5
c  print header line
cfa   call date(day)
cfa   call time(tm)
cfa   write(34,3407) day,tm
      Call DateTime(DateToday,TimeNow)
      write(34,3407) DateToday,TimeNow
cfa 3407 format(/,' AZIM and TOA calculated with 3D velocity model ',
cfa     2 '(SIMULPS15) ',a11,1x,a8,/)
 3407 format(/,' AZIM and TOA calculated with 3D velocity model ',
     & '(SIMULPS15) ',a11,1x,a8,/)
    5 write(34,3401)
 3401 format(2x,'DATE',4x,'ORIGIN',3X,'LATITUDE LONGITUDE  DEPTH',4X,
     2 'MAG NO',11X,'RMS')
      call latlon(evc(1,ne),evc(2,ne),lat,xlat,lon,xlon)
cek
              if(lat.ge.0.and.xlat.ge.0.) then
                   cns='N'
                   lat=iabs(lat)
                   xlat=abs(xlat)
              else
                   cns='S'
                   lat=iabs(lat)
                   xlat=abs(xlat)
              endif
              if(lon.ge.0.and.xlon.ge.0.) then
                   cew='W'
                   lon=iabs(lon)
                   xlon=abs(xlon)
              else
                   cew='E'
                   lon=iabs(lon)
                   xlon=abs(xlon)
              endif
cek
cek  0596
  303 write(34,3402) iyrmo(ne),iday(ne),ihr(ne),mino(ne),seco(ne),
     2 lat,cns,xlat,lon,cew,xlon,evc(3,ne),rmag(ne),kobs(ne),wrmsr(ne)
 3402 format(1x,a4,a2,1x,a2,i2,f6.2,1x,i2,a1,f5.2,1x,i3,a1,f5.2,f7.2,
     2 2x,f5.2,i3,9x,f5.2)
cek  303 write(34,3402) iyrmo(ne),iday(ne),ihr(ne),mino(ne),seco(ne),
cek     2 lat,xlat,lon,xlon,evc(3,ne),rmag(ne),kobs(ne),wrmsr(ne)
cek 3402 format(1x,a4,a2,1x,a2,i2,f6.2,1x,i2,'n',f5.2,1x,i3,'w',f5.2,f7.2,
cek     2 2x,f5.2,i3,9x,f5.2)
c
      write(34,3403)
 3403 format(/,2x,'STN  DIST  AZ TOA PRMK HRMN  PSEC TPOBS',
     2 14X,'PRES  PWT')
      nobs=kobs(ne)
      do 100 i=1,nobs
c  don't use S arrivals for fault-plane solution
         if(intsp(i,ne).eq.1)  goto 100
         iaz=nint(az(i))
         itoa=nint(toa(i))
         write(34,3405) stn(isto(i,ne)),dlta(i,ne),iaz,
     2      itoa,rmk(i,ne),ihr(ne),mino(ne),secp(i,ne),
     3      (secp(i,ne)-seco(ne)),res(i),w(i)
  100 continue
 3405 format(1x,a4,1x,f5.1,1x,i3,1x,i3,1x,a4,
     2 1x,a2,i2,1x,f5.2,1x,f5.2,
     3 13x,f5.2,1x,f5.2)
      write(34,3406)
 3406 format(/)
      return
c ****** end of subroutine outlis ******
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -

c
      subroutine outres(ne)
c  write out station residuals and weights ( from parsep) for each event
c
c  common block variables:
      include 'simulps_common.inc'
      common/wtpars/w(maxobs)
c
c  declaration statements:
      real ttobs(maxobs)
      character*1 phs(2)
      parameter (one=1.00)
c
      data phs/'P','S'/
c
      do 10 j=1,kobs(ne)
         ttobs(j)=secp(j,ne)
         if(intsp(j,ne).eq.0) ttobs(j)=secp(j,ne)-seco(ne)
   10 continue
      if(ne.gt.(neqs+nbls)) goto 200
      write(16,3) ne,iyrmo(ne),iday(ne),ihr(ne),mino(ne),seco(ne),
     2 rmag(ne)
      write(20,3) ne,iyrmo(ne),iday(ne),ihr(ne),mino(ne),seco(ne),
     2 rmag(ne)
 3    format(1h0,' residuals and weights(parsep) for event=',i4, 
     2 3x,a4,a2,1x,a2,i2,1x,f5.2,'  mag ',f4.2)
      write(16,51)
51    format(1x,4('sta  ph  wt  res:O-C ttobs delta',1x))
      write(20,55)
   55 format(1x,'sta  ph  wt  res:O-C ttobs ttcal  delta',
     2 4x,'x ev',4x,'y ev',4x,'z ev',4x,'x st',4x,'y st',
     3 4x,'z st')
      write(16,53) (stn(isto(j,ne)),rmk(j,ne),w(j),res(j),
     2 ttobs(j),dlta(j,ne),j=1,kobs(ne))
53    format(4(1x,a4,a4,f5.2,f7.3,2f6.2))
      do 20 j=1,kobs(ne)
         ttcal=ttobs(j)-res(j)
         write(20,54) stn(isto(j,ne)),rmk(j,ne),w(j),res(j),
     2      ttobs(j),ttcal,dlta(j,ne),
     3      (evc(ie,ne),ie=1,3),(stc(is,isto(j,ne)),is=1,3)
   20 continue
   54 format(1x,a4,a4,f5.2,f7.3,2f6.2,f7.2,6f8.2)
      write(16,1601)
 1601 format(/)
      return
  200 continue
      write(16,33) ne,iyrmo(ne),iday(ne),ihr(ne),mino(ne),seco(ne),
     2 rmag(ne)
      write(20,33) ne,iyrmo(ne),iday(ne),ihr(ne),mino(ne),seco(ne),
     2 rmag(ne)
 33   format(1h0,' residuals and weights(medder) for event=',i4, 
     2 3x,a4,a2,1x,a2,i2,1x,f5.2,'  mag ',f4.2)
      write(16,51)
      write(20,55)
      write(16,53) (stn(isto(j,ne)),rmk(j,ne),w(j),res(j),
     2 ttobs(j),dlta(j,ne),j=1,kobs(ne))
      do 30 j=1,kobs(ne)
         ttcal=ttobs(j)-res(j)
         write(20,54) stn(isto(j,ne)),rmk(j,ne),w(j),res(j),
     2      ttobs(j),ttcal,dlta(j,ne),
     3      (evc(ie,ne),ie=1,3),(stc(is,isto(j,ne)),is=1,3)
   30 continue
      write(16,1601)
      return
c ****** end of subroutine outres ******
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine parsep(ne,nwr)
c  routine to perform parameter separation using qr
c
c  common block variables:
      include 'simulps_common.inc'
      common/wtpars/w(maxobs)
c
c  declaration statements:
      real c1(maxobs,maxobs),u(maxobs),up
      parameter(zero=0.0,one=1.0)
c
      nobs=kobs(ne)
c
c  for shots, don't compute dtmp
      if (ne.gt.(neqs+nbls)) go to 100
c
c  compute weighting
c
      nwr=0
      wnorm=0.0
      do 13 j=1,nobs
c  reading weight
         w(j)=wt(j,ne)
c  residual weighting
c  downweighting(linear) 0 to 98% res1 to res2, 98 to 100% res2 to res3
         ares=abs(res(j))
         if(ares.le.res2) then
            wr=1.0-(ares-res1)*dres12
            if (wr.gt.1.0) wr=1.0
         else
            if(res3.gt.res2) then
               wr=0.02-(ares-res2)*dres23
               if (wr.lt.0.0) wr=0.0
            else
               wr=0.0
            endif
         endif
c  distance weighting
         wd=1.0-(dlta(j,ne)-delt1)*ddlt
         if (wd.gt.1.0) wd=1.0
         if (wd.lt.0.0) wd=0.0
c  unnormalized weight
         w(j)=w(j)*wr*wd
c  19-jul-1983 Change normalizing factor
         wnorm=wnorm+w(j)
c        wnorm=wnorm+w(j)*w(j)
         if (w(j).gt.0.0) nwr=nwr+1
   13 continue
c  check to be sure 4 or more readings with nonzero weight
c    dmep 02dec83
      if(nwr.ge.4) goto 12
      write(16,1612)
 1612 format(' ****** less than 4 readings with nonzero weights',
     2 /,20x,'SKIP TO NEXT EVENT *********')
      call outres(ne)
      return
c  normalize weights
c    19-jul-1983 Change in normalizing factor
c     wfac=sqrt(nwr/wnorm)
   12 wfac=nwr/wnorm
      if(ne.gt.netemp) wfac=wfac*wtsht
      resmax=zero
      do 14 j=1,nobs
         w(j)=w(j)*wfac
         ares=abs(res(j))
         if(ares.gt.resmax) resmax=ares
   14 continue
      if(resmax.gt.(3.0*res2)) call outres(ne)
      nwrt=nwrt+nwr
      if(nitmax.le.0) return
c
      do 10 i=1,nobs
         do 15 j=1,nobs
            c1(j,i)=zero
   15    continue
         c1(i,i)=w(i)
   10 continue
c  apply weighting to hypocenter matrix (1=o.t., 2,3,4=hypocenter)
      nsep=4
c  for blast, only do origin time
      if(ne.gt.neqs) nsep=1
      do 17 i=1,nsep
         do 16 j=1,nobs
            dth(j,i)=dth(j,i)*w(j)
   16    continue
   17 continue
c
c  perform qr decomposition
c  prepare parameters for input to h12 routine
c  loop over the four columns of dth for decomposition
      do 20 i=1,nsep
         irow=i
         irow1=i+1
c  put column of dth into u-vector
         do 22 j=1,nobs
            u(j)=dth(j,i)
   22    continue
c  compute single transformation
         call h12(1,irow,irow1,nobs,u,1,up,dth,1,maxobs,4,
     2      c1,1,maxobs,nobs)
   20 continue
c
c
c  qr decomposition complete - multiply residuals and medium matrix
c  by u0 matrix (q matrix minus its first four columns)
      nobs4=nobs-nsep
      do 35 j=1,nobs
         resj=res(j)
         do 34 i=1,nobs4
c  28 jan84 Changed from 4 to nsep to match CT. dmep
            i4=i+nsep
            resp(i)=resp(i)+c1(i4,j)*resj
   34    continue
   35 continue
c  compute separated medium matrix and store
      do 30 n=1,nobs
         nn=(n-1)*npari
         do 37 i=1,nobs4
            ii=(i-1)*npari
            i4=i+nsep
            c4n=c1(i4,n)
            do 36 m=1,npari
               im=ii+m
               nm=nn+m
               dtmp(im)=dtmp(im)+c4n*dtm(nm)
   36       continue
   37    continue
   30 continue
c
c  add to elements of g matrix
c
  100 continue
c
c  compute contribution to ssqr
      sqwtsh=sqrt(wtsht)
      totrms(ne)=0.0
      do 18 i=1,nobs
         resi=res(i)
         resi2=resi*resi
         totrms(ne)=totrms(ne)+resi2
         rrw=resi2*w(i)
         ssqrw=ssqrw+rrw
         wnobt=wnobt+w(i)
         if(intsp(i,ne).eq.0) then
c  P arrival
            ssqrwp=ssqrwp+rrw
            wnobtp=wnobtp+w(i)
         else
c  S arrival
            ssqrws=ssqrws+rrw
            wnobts=wnobts+w(i)
         endif
         if (ne.gt.netemp) resi=resi*sqwtsh
         ssqr=ssqr+resi*resi
   18 continue
      totrms(ne)=sqrt(totrms(ne)/nobs)
c
c  loop over all observations (in separated form)
      n1=1
      n2=nobs4
      if (ne.gt.(neqs+nbls)) n2=nobs
c     write(16,359)n1,n2,ne,neqs
c359  format(' parsep: n1,n2,ne,neqs= ',4i8)
c  loop over observations
      do 140 i=n1,n2
c        write(16,364)i,n1,n2
c364     format(' parsep: i,n1,n2= ',3i8)
         ii=(i-1)*npari
c  for a given observation, loop over all nodes
         do 130 j=1,npari
            jj=j+ii
            if (dtmp(jj).eq.zero) go to 130
c  add unknown for first observation only
            if (khit(mdexfx(j)).gt.0) go to 125
            mbl=mbl+1
            index(j)=mbl
c  zero next row of g matrix
            mbtot=nbtot+1
            nbtot=nbtot+mbl
            do 120 l=mbtot,nbtot
               g(l)=0.0
  120       continue
  125       khit(mdexfx(j))=khit(mdexfx(j))+1
c CHECK (only for shots since parameter separation messes up indices)
c           if (ne.le.(neqs+nbls)) go to 126
c           if((intsp(i,ne).eq.1).and.(j.le.nodes2))
c    2         write(36,3601) dtmp(jj),jj,j,i,khit(j),index(j)
c3601       format(' Parsep: dtmp(jj)=',f7.2,'jj=',i6,',j=',i4,
c    2         ',i=',i4,',khit(j)=',i5,',index(j)=',i5)
  126       k=index(j)
            kk2=((k-1)*k)/2
c  build rhs
            rhs(k)=rhs(k)+resp(i)*dtmp(jj)
c  build g matrix
            do 128 l1=1,j
               lj=l1+ii
               if (dtmp(lj).eq.zero) go to 128
               l=index(l1)
               m=l+kk2
               if (k.lt.l) m=k+((l-1)*l)/2
               g(m)=g(m)+dtmp(jj)*dtmp(lj)
c              if(g(m).lt.0.0) write(16,1620) g(m),m,dtmp(jj),
c    2            jj,dtmp(lj),lj,k,l,l1,i,ii,j
c1620          format(' g(m)=',f6.3,'m=',i6,',dtmp(jj)=',f6.3,
c    2            'jj=',i6,',dtmp(lj)=',f6.3,'lj=',i6,',k=',i4,
c    3            ',l=',i4,',l1=',i4,',i=',i4,',ii=',i6,',j=',i4)
  128       continue
  130    continue
  140 continue
c  zero out dtm and dtmp matrices for next event
      nono=nobs*npari
      seprms(ne)=zero
      do 45 m=1,nono
         dtm(m)=zero
         dtmp(m)=zero
   45 continue
      do 945 i=1,nobs
         seprms(ne)=seprms(ne)+resp(i)*resp(i)
         resp(i)=zero
  945 continue
      rmssep=sqrt(seprms(ne)/nobs)
      if(ne.gt.(neqs+nbls)) then
         write(16,1698) totrms(ne),rmssep
 1698    format('  ** total rms =',f8.5,
     2      '  **  model rms =',f8.5,' **')
      else
         write(16,1699) nwr,totrms(ne),rmssep
 1699    format(5x,'nwr=',i3,', ','** total rms =',f8.5,
     2      '  **  model rms =',f8.5,' **')
      endif
      if(totrms(ne).ge.(2.0*res1)) call outres(ne)
      return
c***** end of subroutine parsep *****
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine path(ne,no,xe,ye,ze,ttime)
c  this routine determines the minimum-time ray path
c  in two steps:  first, an approximate path is
c  determined using approximate ray tracing
c  then the approximate path is used as a starting point in
c  shooting ray tracing routine to determine the path in 3-d
c  ***  note - current version does not contain full 3-d ray
c  ***  tracing - routines are under development
c
c  declaration statements:
c
c  common block variables:
      include 'simulps_common.inc'
c
c  set up parameters to call ray tracing routine
c
c  S or P reading
      isp=intsp(no,ne)
c  receiver coordinates
      ns=isto(no,ne)
      xr=stc(1,ns)
      yr=stc(2,ns)
      zr=stc(3,ns)
c  determine 3-d path using circular raypaths.
c  determine approximate path
  120 jflag=jfl
      ncrold=nco(no,ne)
      ndpold=ndo(no,ne)
c
      call rayweb(no,isp,xe,ye,ze,xr,yr,zr,
     * fstime,jflag,ncrold,ndpold)
      ttime=fstime
      ttc(no)=ttime
      nco(no,ne)=ncrold
      ndo(no,ne)=ndpold
c
c  do psuedo-bending if i3d>0
      if (i3d.eq.0) return
c
  125 continue
c  number of pb iter depends on distance
      nitpbu=nitpb(1)
      if(dlta(no,ne).gt.delt1) nitpbu=nitpb(2)
      call minima(no,isp,ttime,nitpbu,jpb)
      if(jpb.lt.nitpbu) goto 130
        write(26,2601) no,stn(ns),jpb,nitpbu
 2601   format(' Minima: no=',i4,2x,a4,', used maximum ',
     2  'number PB iter.: j=',i3,', nitpb=',i3)
  130 continue
      ttc(no)=ttime
      if(kout3.eq.0) return
c  write out travel-time differences to file 19
      tdif=fstime-ttime
      write(19,1900) ne,stn(isto(no,ne)),dlta(no,ne),fstime,ttime,tdif
 1900 format(i4,1x,a4,f7.2,3f8.4)
c
      return
c***** end of subroutine path *****
      end
c
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine qtravel
c
      common/temp/xtemp(130),ytemp(130),ztemp(130),rtemp(130),ttemp(130)
c
      common/pathm/x(130),y(130),z(130),v(130),vq(130),tra,qtra,n,nn
c
      qtra=0
      do 60 i=2,n
         i1=i-1
         xd=xtemp(i)-xtemp(i1)
         yd=ytemp(i)-ytemp(i1)
         zd=ztemp(i)-ztemp(i1)
         ds=sqrt(xd*xd+yd*yd+zd*zd)
         tv=ds*(1.0/vq(i)+1.0/vq(i1))
         qtra=qtra+tv
  60  continue
      qtra=0.5*qtra
c
      return
c ***** end of subroutine qtravel *****
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine rayweb(no,isp,xe,ye,ze,xr,yr,zr,
     *            fstime,jflag,ncrold,ndpold)
c  approximate ray tracing package art2
c    with fast ray tracing code
c     by Cliff Thurber (from his simul3l version)
c
c  common block variables:
      include 'simulps_common.inc'
c
c  declaration statements:
c  parameters
      real xe,ye,ze,xr,yr,zr,fstime
c  local variables
      real delx,dely,delz,sep,delsep,pthsep(130),strpth(390),
     * fstpth(390),dipvec(3,9),disvec(390,9),trpath(390,9),
     * trtime(9),tmin,tt,trpth1(390)
      integer nd,ns,npt,ncr,i,ic,ndp,nc,n1,n2,n3,nn,np,ndpfst
c
c  compute source-receiver separation
      delx=xr-xe
      dely=yr-ye
      delz=zr-ze
      sep=sqrt(delx*delx+dely*dely+delz*delz)
c  determine integer parameters for set of curves to be constructed
      call setup(sep,scale1,scale2,nd,ns,npt,ncr)
c
c  set up pthsep array for straight-line travel time calculation
      sn1=1.0/ns
      delsep=sep*sn1
      do 20 i=1,ns
         pthsep(i)=delsep
   20 continue
c
c  determine points along straight-line path
      xstep=delx*sn1
      ystep=dely*sn1
      zstep=delz*sn1
c
      ic=0
      ns1=ns+1
      do 25 ii=1,ns1
c
         i=ii-1
c
         ic=ic+1
         strpth(ic)=xe+xstep*i
         fstpth(ic)=strpth(ic)
         ic=ic+1
         strpth(ic)=ye+ystep*i
         fstpth(ic)=strpth(ic)
         ic=ic+1
         strpth(ic)=ze+zstep*i
         fstpth(ic)=strpth(ic)
   25 continue
c
c  compute travel time along straight-line path
      call ttime(isp,ns,npt,strpth,pthsep,fstime)
c
cfh
      if (ncr.eq.1) go to 65
c
c  compute the dip vectors of length scale2
      call cmpdpv(xe,ye,ze,xr,yr,zr,scale2,ndip,dipvec)
c
c  compute the basic set of displacement vectors
      call cmpdsv(ndip,iskip,ns,dipvec,disvec)
c
c  set first and last points of all trial paths to source and receiver
      n1=3*npt-2
      n2=n1+1
      n3=n1+2
      ndip1=1+iskip
      ndip2=ndip-iskip
c
c  fast ray tracing code
c
      nz1=nz-1
      ncr0=1
      ncr1=ncr-1
      if (jflag.eq.0) go to 28
      ncr0=ncrold-1
      if (ncr0.lt.1) ncr0=1
      ncr1=ncrold+1
      if (ncr1.gt.ncr) ncr1=ncr
c  ndip was changed (in main) for ihomo iterations., make ndpold=vertical plane.
c     if(jfl.eq.1) ndpold=(ndip+1)/2     ! 21-feb-86, this is done in strt, so unnecessary here
      ndip1=ndpold-1
      if (ndip1.lt.1+iskip) ndip1=1+iskip
      ndip2=ndpold+1
      if (ndip2.gt.ndip-iskip) ndip2=ndip-iskip
   28 continue
c  set "old" values to straight line
      ncrold=0
      ndpold=(ndip+1)/2
c
c
      do 30 ndp=ndip1,ndip2
         trpath(1,ndp)=xe
         trpath(2,ndp)=ye
         trpath(3,ndp)=ze
         trpath(n1,ndp)=xr
         trpath(n2,ndp)=yr
         trpath(n3,ndp)=zr
   30 continue
      trpth1(1)=xe
      trpth1(2)=ye
      trpth1(3)=ze
      trpth1(n1)=xr
      trpth1(n2)=yr
      trpth1(n3)=zr
c
c  loop over the curve sets
      do 40 nc=ncr0,ncr1
         iz0=0
c
c  loop over different dips for one set of curves
         do 42 ndp=ndip1,ndip2
c
            npt2=npt-2
c  loop to determine points along one path
            do 44 np=1,npt2
               n1=3*np+1
               n3=n1+2
               do 43 nn=n1,n3
                  trpath(nn,ndp)=nc*disvec(nn,ndp)+strpth(nn)
                  trpth1(nn)=trpath(nn,ndp)
   43          continue
               if((i3d.lt.3).or.(ze.gt.zn(nz1))) goto 44
c  Use less curvature if below "moho"
               if(trpth1(n3).gt.zn(nz1)) then
                  if(iz0.eq.0) then
                     z0=trpth1(n3)-disvec(n3,ndp)
                     iz0=1
                  endif
                  trpath(n3,ndp)=disvec(n3,ndp)+z0
c                 write(61,600) trpth1(n3),zn(nz1),trpath(n3,ndp)
  600             format('trpth1(n3)',f7.2,',zn(nz1)',f7.2,
     2               'new trpth1(n3)',f7.2)
                  trpth1(n3)=trpath(n3,ndp)
               endif
   44       continue
c
c  set up pthsep array for travel time calculations
            if (ndp.eq.ndip1) call cmpsep(trpth1,pthsep,ns)
c  compute travel time along one path
            call ttime(isp,ns,npt,trpth1,pthsep,tt)
            trtime(ndp)=tt
   42    continue
c
c  sort through trtime to find fastest path from current set
         tmin=1.0e15
         do 50 ndp=ndip1,ndip2
            if (trtime(ndp).gt.tmin) go to 50
            tmin=trtime(ndp)
            ndpfst=ndp
   50    continue
c
c  compare fastest trtime to current value of fstime
c  replace fstime and fstpth if needed
         if (tmin.ge.fstime) go to 40
         fstime=tmin
c  reset "old" values
         ncrold=nc
         ndpold=ndpfst
c
         npt3=3*npt
         do 52 np=1,npt3
            fstpth(np)=trpath(np,ndpfst)
   52    continue
c
   40 continue
c
c  put fstpth into rp array
   65 continue
      do 60 np=1,npt
         n3=3*np
         n1=n3-2
         n2=n1+1
         rp(1,np,no)=fstpth(n1)
         rp(2,np,no)=fstpth(n2)
         rp(3,np,no)=fstpth(n3)
   60 continue
      nrp(no)=npt
c
c***** end of subroutine rayweb *****
      return
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine rescov
c  compute resolution and covariance
c  common block variables:
      include 'simulps_common.inc'
c
      varnce=ssqrw/float(nobt-mbl-4*neqs)
c     write(16,1001) varnce
 1001 format(/,' RESCOV: data variance is ',f11.6)
c  resolution and error calculations
      if(ires.eq.2) rewind (17)
c  avoid printing zeroth element of rhs array
c  index point from nodes(input) to solution arrays(rhs)
      do 5 kc=1,npari
         if(khit(mdexfx(kc)).eq.0) index(kc)=npari+1
         if(hit(mdexfx(kc)).lt.hitct) index(kc)=npari+1
         if(index(kc).eq.0) index(kc)=npari+1
    5 continue
c  zero out drm and stderr elements
      do 300 l=1,npar
         drm(l)=0.0
         stderr(l)=0.0
  300 continue
c
   10 do 500 l=1,npari
         l1=l
         l1n=mdexfx(l1)
         if(khit(l1n).eq.0) goto 500
         if(hit(l1n).lt.hitct) goto 500
         k=index(l1)
         jj=0
         ii=0
c put appropriate elements of g1 into rhs
         do 435 i=1,mbl
            do 430 j=1,i
               jj=jj+1
               if (i.eq.k.or.j.eq.k) go to 420
               go to 430
  420          ii=ii+1
               rhs(ii)=g1(jj)
c CHECK
c              if(g1(jj).lt.-5.0) write(16,1620)
c    2            g1(jj),jj,ii,i,j,mbl,l,k,npar
c1620          format(' g1(jj)=',f9.3,'jj=',i6,',ii=',i4,',i=',i4,
c    2            ',j=',i4,',mbl=',i4,',l=',i4,',k=',i4,',npar=',i4)
  430       continue
  435    continue
c  compute vector of resolution matrix for this parameter
         call luelmp(g,rhs,mbl,rhs)
c print out full resolution matrix if ires=2 or 3
         if(ires.lt.2) goto 440
cfhdmep write out number of free nodes and inverted nodes to unit 18 for
c later use with resolution matrix
         rewind (18)
         write(18,1800) npar,nparv,npari,nparvi,nparpi,nparsi,nrowp,
     2     nrows,nrowst
 1800    format(9i7,'   0.0001')
c
         write(17,1720) l1,mdexfx(l1)
 1720    format(/,'row=',i5,', gridpoint no.=',i5,/)
         write(17,1721) (rhs(index(kc)),kc=1,npari)
 1721    format(18f7.4)
c  store diagonal element
  440    drm(l1n)=rhs(k)
c  standard error and covariance calculation
         do 450 i=1,mbl
            rhs(i)=rhs(i)*varnce
  450    continue
c  compute vector of covariance matrix for this parameter
         call luelmp(g,rhs,mbl,rhs)
c  store standard error of slowness perturbation
         rhs(k)=abs(rhs(k))
         stderr(l1n)=sqrt(rhs(k))
  500 continue
  900 return
c***** end of subroutine rescov *****
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine setorg(ltdo,oltm,lndo,olnm)
c  this routine establishes the short distance conversion factors
c  given the origin of coordinates
c  the rotation angle is converted to radians also
c  common block variables:
      common/shortd/ xltkm,xlnkm,rota,nzco,xlt,xln,snr,csr
c  local variables:
      double precision dlt1,dlt2,dxlt,drad,drlt
      parameter ( re=6378.163, ell=298.26)
      parameter (drad=1.7453292d-2)
      parameter (drlt=9.9330647d-1)
      rota=rota*drad       !convert from degrees to radians
c
cek
cek correction for S and E: minutes must also be taken as negative!
cek
cek   EK  12.6.96
cek
cek        dxlt=dble(60.*ltdo+oltm)
cek        xln=60.*lndo+olnm
      dxlt=dble(60.*abs(ltdo)+oltm)
      xln=60.*abs(lndo)+olnm
      if(ltdo.lt.0) dxlt=-1.*dxlt
      if(lndo.lt.0) xln=-1.*xln
cek
c
      xlt=sngl(dxlt)
c  conversion factor for latitude
      dlt1=datan(drlt*dtan(dxlt*drad/60.d0))
      dlt2=datan(drlt*dtan((dxlt+1.d0)*drad/60.d0))
      del=sngl(dlt2-dlt1)
      r=re*(1.0-sngl(dsin(dlt1)**2)/ell)
      xltkm=r*del
c  conversion factor for longitude
      del=sngl(dacos(1.0d0-(1.0d0-dcos(drad/60.d0))*dcos(dlt1)**2))
      bc=r*del
      xlnkm=bc/sngl(dcos(dlt1))
      write(16,3001) xltkm,bc
 3001 format(5x,'short distance conversion factors',/,
     2       10x,'one min lat  ',f7.4,' km',/,
     3       10x,'one min lon  ',f7.4,' km',/)
c
c  convert coordinates with rotation cosines
      snr=sin(rota)
      csr=cos(rota)
      return
c***** end of subroutine setorg *****
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine setup(sep,scale1,scale2,nd,ns,npt,ncr)
c
c  parameters
      real sep,scale1,scale2
c
      integer nd,ns,npt,ncr
      nd=1+nint(3.32193*log10(sep/scale1))
      if (nd.gt.7) nd=7
c  number of segments along path
      ns=2**nd
c  number of points on path
      npt=ns+1
c
c  determine the number of curves - use scale2
      ncr=1+0.5*sep/scale2
c
      if (sep.gt.scale1) return
      nd=0
      ns=1
      npt=2
      ncr=1
c
c***** end of subroutine setup *****
      return
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine strt(nit)
c
c  declaration statements:
      parameter(zero=0.0)
c
c  common block variables:
      common/machin/ eta,tol
      include 'simulps_common.inc'
c
      iarr=maxpar
      iarri=mxpari
c
      nwrt=zero
      nswrt=zero
      ssqr=zero
      ssqrw=zero
      ssqrwp=zero
      ssqrws=zero
      wnobt=zero
      wnobtp=zero
      wnobts=zero
      do 77 n=1,iarri
         index(n)=0
         jndex(n)=0
         rhs(n)=zero
   77 continue
c
c
c  reset fast art arrays?
      if (nit.gt.ihomo) go to 80
      do 790 m=1,maxev
         do 79 n=1,maxobs
            ndo(n,m)=(ndip+1)/2
   79    continue
  790 continue
c
      if (nit.ge.1) go to 80
      do 780 m=1,maxev
         do 78 n=1,maxobs
            nco(n,m)=1
   78    continue
  780 continue
c
c
   80 mbl=0
      nbtot=0
c  zero hit counter arrays
      do 25 j=1,iarr
         hit(j)=zero
         khit(j)=0
   25 continue
c
c  zero out partial derivative arrays
      mxobs1=maxobs-1
      do 30 j=1,iarri
         jj2=maxobs*j
         jj1=jj2-mxobs1
         do 35 i=jj1,jj2
            dtm(i)=zero
            dtmp(i)=zero
   35    continue
   30 continue
      do 39 i=1,maxobs
         resp(i)=zero
   39 continue
c
      if (nit.gt.0) return
c
c  machine constants
      halfu=0.5
   50 temp1=1.0+halfu
      if (temp1.le.1.0) go to 100
      halfu=0.5*halfu
      go to 50
  100 eta=2.0*halfu
c
      temp2=0.5
  150 continue
      if (temp2.le.0.0) go to 200
      tol=temp2
      temp2=0.5*temp2
      go to 150
  200 continue
c
c     tol=tol/eta
c for unix make tol bigger
      tol=tol/(eta*eta)
c
      write(16,1055) eta,tol
 1055 format(/,'  computed machine constants eta and tol:',2e16.6)
c
      return
c***** end of subroutine strt *****
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine travel
c
      common/temp/xtemp(130),ytemp(130),ztemp(130),rtemp(130),ttemp(130)
c
      common/pathm/x(130),y(130),z(130),v(130),vq(130),tra,qtra,n,nn
c
      tra=0
      do 60 i=2,n
         i1=i-1
         xd=xtemp(i)-xtemp(i1)
         yd=ytemp(i)-ytemp(i1)
         zd=ztemp(i)-ztemp(i1)
         ds=sqrt(xd*xd+yd*yd+zd*zd)
         tv=ds*(1.0/v(i)+1.0/v(i1))
         tra=tra+tv
  60  continue
      tra=0.5*tra
c
      return
c ***** end of subroutine travel *****
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine ttime(isp,ns,npt,pathr,pthsep,tt)
c
c  travel time along path via trapezoidal rule integration
c
c  parameters
      real pathr(390),pthsep(130),tt
c
      integer ns,npt
c  local variables
      real vpt(130),x,y,z,v
c
      integer ip,np
c
c  loop over points along path to determine velocities
      ip=0
      do 10 np=1,npt
         ip=ip+1
         x=pathr(ip)
         ip=ip+1
         y=pathr(ip)
         ip=ip+1
         z=pathr(ip)
         call vel3(isp,x,y,z,v)
   10 vpt(np)=v
c
      tt=0.0
c  sum up travel time - use trapezoidal rule
      do 20 np=1,ns
         np1=np+1
c  Check for value outside defined area
         if((vpt(np).le.0.0).or.(vpt(np1).le.0.0)) goto 99
         tt=tt+pthsep(np)/(vpt(np)+vpt(np1))
   20 continue
      tt=2.0*tt
c
      return
   99 write(16,100) np,vpt(np),np1,vpt(np1)
  100 format(' **** ERROR IN TTIME ****, velocity le 0.0',
     2 /,'    np   vpt(np)   np1   vpt(np1)',/,1x,2(i5,f10.2))
      stop
c***** end of subroutine ttime *****
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine ttmder(ne,no,kopt,ttime)
c
c  declaration statements:
      dimension in(8)
c
c  common block variables:
      common/weight/ wv(8),ip,jp,kp,kpg
      include 'simulps_common.inc'
c
c
c   This subroutine has been modified for Q inversion 13-March-1998.
c   Following Andreas Rietbrock's version.
c   Then (when iuseq=1), the second part of the model array
c   is used to store Vp*Qp (not Vp/Vs).
c   The solution is only for Q, not for Vp though.  
c   Thus the solution arrays that would otherwise be related
c   to the first part of the model array (Vp) are used.
c   Therefore in ttmder, vel3 is called with "1" to retrieve
c   the V*Q value; and vel3 is called with "0" to get the 
c   indices for the solution arrays.
c
c
c  resegment ray path
c  calculate travel time derivatives with respect to hypocentral
c  parameters - use coords of first two points on raypath
c  determine slowness at source
      xe=rp(1,1,no)
      ye=rp(2,1,no)
      ze=rp(3,1,no)
c  check for P or S reading
      isp=intsp(no,ne)
      if(iuseq.eq.0) then
              call vel3(isp,xe,ye,ze,v)
      else
              call vel3(1,xe,ye,ze,v)
      endif
       
      us=1.0/v
c**  The program has been modified to include S-P times for the inversion.
c**  The corresponding travel-time derivatives are calculated.(06/29/92)
      if (isp.eq.1) then
              isp=0
              call vel3(isp,xe,ye,ze,vp)
              up=1.0/vp
              us=us-up
        isp=1
      endif
c**
c  determine cartesian derivatives from direction cosines
      dx=rp(1,2,no)-rp(1,1,no)
      dy=rp(2,2,no)-rp(2,1,no)
      dz=rp(3,2,no)-rp(3,1,no)
      ds=sqrt(dx*dx+dy*dy+dz*dz)
      uds=-us/ds
c  hypocentral derivatives
      dth(no,2)=uds*dx
      dth(no,3)=uds*dy
      dth(no,4)=uds*dz
c  origin time derivative
      dth(no,1)=1.0
c** S-P CHANGE
c   zero the origin time derivative for S-P
      if (isp.eq.1) dth(no,1)=0.0
c**
c  zero cartesian derivatives for quarry blasts
      if((ne.le.neqs).or.(ne.gt.(neqs+nbls))) goto 20
      dth(no,2)=0.0
      dth(no,3)=0.0
      dth(no,4)=0.0
   20 continue
c
c  skip next section if all velocity nodes are fixed (nparvi=0)
      if(nparvi.eq.0) goto 88
c  skip next section if only doing earthquake location
        if(kopt.eq.0) go to 88
c
c  travel time and velocity partial derivatives
      tt=0.0
      half=0.5
c  loop over segments comprising the ray path
      nrp1=nrp(no)-1
      pl(no)=0.0
      do 50 i=1,nrp1
         i1=i+1
         rx=rp(1,i,no)
         ry=rp(2,i,no)
         rz=rp(3,i,no)
         dx=rp(1,i1,no)-rx
         dy=rp(2,i1,no)-ry
         dz=rp(3,i1,no)-rz
c  compute segment length
         sl=sqrt(dx*dx+dy*dy+dz*dz)
         pl(no)=pl(no)+sl
c  decide on number of subsegments and compute length
         nseg=nint(sl/stepl)+1
         fnsegi=1.0/float(nseg)
         ssl=sl*fnsegi
         dxs=dx*fnsegi
         dys=dy*fnsegi
         dzs=dz*fnsegi
c
         xp=rx-half*dxs
         yp=ry-half*dys
         zp=rz-half*dzs
c  loop over subsegments
         do 55 is=1,nseg
            xp=xp+dxs
            yp=yp+dys
            zp=zp+dzs
c
          if(iuseq.eq.0) then
               call vel3(isp,xp,yp,zp,v)
          else
               call vel3(1,xp,yp,zp,v)
               call vel3(0,xp,yp,zp,vv)
          endif
            dt=ssl/v
c
c
c  The next section is a change from 'block' to 'linear'
c   partial derivatives, by C.Thurber,may10,1983.
c  Nodes with non-zero weight
            in(1)=ip-1+nx2*(jp-2)+nxy2*(kp-2)-nxy2*(2*isp)
            in(2)=in(1)+1
            in(3)=in(1)+nx2
            in(4)=in(3)+1
            in(5)=in(1)+nxy2
            in(6)=in(5)+1
            in(7)=in(5)+nx2
            in(8)=in(7)+1
c
c  Assign zero weight to boundary nodes (these nodes are not 
c  included in the inversion, but are in the velocity array,
c  thus we want to avoid writing to negative or incorrect 
c  elements of the partial derivative matrix)
c
            if(ip.eq.1) then
c              write(16,1610) no,xp,yp,zp,v,ip,jp,kpg
               wv(1)=0.0
               wv(3)=0.0
               wv(5)=0.0
               wv(7)=0.0
            else
               if(ip.eq.nx1) then
c                 write(16,1610) no,xp,yp,zp,v,ip,jp,kpg
                  wv(2)=0.0
                  wv(4)=0.0
                  wv(6)=0.0
                  wv(8)=0.0
               end if
            endif
c
            if(jp.eq.1) then
c              write(16,1610) no,xp,yp,zp,v,ip,jp,kpg
               wv(1)=0.0
               wv(2)=0.0
               wv(5)=0.0
               wv(6)=0.0
            else
               if(jp.eq.ny1) then
c                 write(16,1610) no,xp,yp,zp,v,ip,jp,kpg
                  wv(3)=0.0
                  wv(4)=0.0
                  wv(7)=0.0
                  wv(8)=0.0
               endif
            endif
c
            if((kpg.eq.1).or.(kpg.eq.(nz1+1))) then
c              write(16,1610) no,xp,yp,zp,v,ip,jp,kpg
               do 30 izg=1,4
                  wv(izg)=0.0
   30          continue
            else
               if((kpg.eq.nz1).or.(kpg.eq.(2*nz1))) then
c                 write(16,1610) no,xp,yp,zp,v,ip,jp,kpg
                  do 35 izg=5,8
                     wv(izg)=0.0
   35             continue
               endif
            endif
 1610       format(' ASSIGNING ZERO WEIGHTS IN TTMDER',
     2         ' no=',i3,',xp=',f7.2,',yp=',f7.2,',zp=',
     3         f7.2,',v=',f5.3,',ip=',i2,',jp=',i2,',kpg=',i2)
c
c  Accumulate model partial derivatives
            do 48 kk=1,2
               kk1=kk-1
               do 47 jj=1,2
                  jj1=jj-1
                  do 46 ii=1,2
                     ii1=ii-1
                     ijk=ii+2*jj1+4*kk1
c skip boundary nodes
                     if(wv(ijk).lt.0.05) goto 46
c  skip fixed nodes
                     if(nfix(in(ijk)).eq.1) goto 46
                     ini=ndexfx(in(ijk))
c check for writing to an array element that is outside of the inversion
c  solution array
                     if((ini.lt.1).or.(ini.gt.nparvi)) then
                        write(16,1606) ini,ijk
 1606                   format(' *** Error in TTMDER, accessing',
     2                     ' gridpoint outside of velocity inversion',
     3                     ' gridpoints, ini=',i5,', ijk=',i5,/,
     4                     22x,'Probably boundary gridpoints are',
     5                     ' too close (TTMDER tries to write DTM',
     6                     ' elements with wv >= 0.05)')
                        write(16,1603) ne,no,xp,yp,zp,v,ip,jp,kp,kpg
 1603                   format(' ne=',i5,', no=',i5,', xp=',f8.2,
     2                     ', yp=',f8.2,', zp=',f8.2,', v=',f8.3,/,
     3                     21x,'ip=',i6,',   jp=',i6,',   kp=',i6,
     4                     ',   kpg=',i6)
                        write(16,1607) (j,in(j),j,wv(j),j=1,8)
 1607                   format(' in(',i1,')=',i6,' wv(',i1,')=',e15.5)
                        write(16,1608)
 1608                   format(' * * * * STOP * * * * (to avoid',
     2                     ' writing outside of defined DTM array)')
                        stop
                     end if
                     inp=ini+(no-1)*npari
                     hit(in(ijk))=hit(in(ijk))+wv(ijk)
c** S-P CHANGE
c   set up equation to solve for delta-Vp/Vs
                     if (isp.eq.0) then
                     if(iuseq.eq.0) then
                       dtm(inp)=dtm(inp)+dt*wv(ijk)*
     &                     vel(ip+ii1,jp+jj1,kp+kk1)/v
                     else
                       call vel3(1,xp,yp,zp,vv)
                       dtm(inp)=dtm(inp)+dt*wv(ijk)*
     &                     vel(ip+ii1,jp+jj1,kp+kk1)/v
                       call vel3(0,xp,yp,zp,vv)
                     endif
                   endif
                     if (isp.eq.1) then
                        isp=0
                        call vel3(isp,xp,yp,zp,vp)
                        dtm(inp)=dtm(inp)+ssl*wv(ijk)/vp
                        isp=1
                     endif
c***
   46             continue
   47          continue
   48       continue
c
   55    continue
   50 continue
c  arrival time residual
  88    continue
      is=isto(no,ne)
      res(no)=secp(no,ne)-seco(ne)-ttime-pdl(is)
      if(i3d.eq.4.and.ides.gt.1) write(61,*) 
     . 'res from ttmder for this obs: ',res(no)
c** S-P CHANGE
c**     Estimate the calculated P and S times and then the S-P time
c       Then estimate the residuals between the observed and calculated.
      if (isp.eq.1) then
      intsp(no,ne)=0
cfhRT here again use trvhet if i3d = 4
cfh  (do we really have to do the whole raytracing again???)
      if(i3d.eq.4) then 
cfh here we have a problem: shall we start from scratch for these calls to trvhet?
cfh - regarding the initial angles, the ones stored for this ray are for S-velocities.
cfh - there should be a P-observation ray but where is it?
cfh - are the P and S raypaths so similar that we can use the same angles??
cfh - so for now use previously computed angles to start (iprep=1)      
        call trvhet(ne,no,ptime,1)
      else
        call path(ne,no,xe,ye,ze,ptime)
      endif
      intsp(no,ne)=1
      smptime=ttime-ptime
      res(no)=secp(no,ne)-smptime - sdl(is)
      endif
c**
c
  93  continue
      if(invdel.eq.0)return
c  Calculate station delay partial derivatives (unless doing location only)
      if(kopt.eq.0) return
      if(nfixst(is).eq.1) return
      ncor=nparv+is
c  If the observation is an S-wave, put the derivative nsts further up.
      if(intsp(no,ne).ne.0)ncor=ncor+nsts
      ini=ndexfx(ncor)
c   The derivative array, dtm(), is a long, linear array with all
c   of the observations and their derivatives end to end.
c   hit is an array which is used for this observation only. Khit
c   is set in subroutine parsep.
c   Each observation occupies npar locations, which = 1+(no-1)*npar to
c   no*npar
c   npar=nodes2, or nodes2+2*nsts when stn delays are being inverted, too.
      inp=(no-1)*npari+ini
      dtm(inp)=1.0
c checking station corrections
c     write(16,1620) is,inp,dtm(inp)
c1620 format(' TTMDER:is,inp,dtm',i5,i10,f14.6)
      hit(ncor)=hit(ncor)+1.0
      return
c***** end of subroutine ttmder *****
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine vel3(isp,x,y,z,v)
c  This routine is Cliff Thurber's
c  common block variables:
      common/weight/ wv(8),ip,jp,kp,kpg
      include 'simulps_common.inc'
c
c  use Prothero's intmap here
      call intmap(x,y,z,ip,jp,kp)
      if(ip.eq.0.or.jp.eq.0.or.kp.eq.0) then
        write(6,*)'Error in intmap - position off the map: ',x,y,z
        stop
      endif
c
cfhtest 
c      write(61,*)'x, y, z = ',x,y,z
c      write(61,*)'ip, jp, kp = ',ip,jp,kp
c
      ip1=ip+1
      jp1=jp+1
      kp1=kp+1
cfh check for outer boundary (to larger values) hit (needed for bldrtgrd)
cfh (anyway should only be the case if value is on boundary)
      if (ip1.gt.nx) then
          ip1 = ip -1
          xf=(x-xn(ip))/(xn(ip)-xn(ip1))
      else 
          xf=(x-xn(ip))/(xn(ip1)-xn(ip))
      endif
      if (jp1.gt.ny) then
         jp1 = jp -1
         yf=(y-yn(jp))/(yn(jp)-yn(jp1))
      else 
         yf=(y-yn(jp))/(yn(jp1)-yn(jp))
      endif
      if (kp1.gt.nz) then
         kp1 = kp -1
         zf=(z-zn(kp))/(zn(kp)-zn(kp1))
      else 
         zf=(z-zn(kp))/(zn(kp1)-zn(kp))
      endif
c
c       write(16,100)x,xl,ip,y,yl,jp,z,zl,kp
c100    format(3(2f7.3,i3))
cfh      xf=(x-xn(ip))/(xn(ip1)-xn(ip))
cfh      yf=(y-yn(jp))/(yn(jp1)-yn(jp))
cfh      zf=(z-zn(kp))/(zn(kp1)-zn(kp))
      xf1=1.0-xf
      yf1=1.0-yf
      zf1=1.0-zf
c
      wv(1)=xf1*yf1*zf1
      wv(2)=xf*yf1*zf1
      wv(3)=xf1*yf*zf1
      wv(4)=xf*yf*zf1
      wv(5)=xf1*yf1*zf
      wv(6)=xf*yf1*zf
      wv(7)=xf1*yf*zf
      wv(8)=xf*yf*zf
c  calculate velocity
c  S-velocity is stored after P-velocity
c  (or V*Q if iuseq=1)
      kpg=kp
cfh put the following in an if - then construct.
      if(isp.eq.1) then 
        kp=kp+nz
        kp1=kp+1
cfh again
        if (kp1.gt.(2*nz)) kp1 = kp -1
      endif
c
      v=wv(1)*vel(ip,jp,kp)+wv(2)*vel(ip1,jp,kp)
     2 +wv(3)*vel(ip,jp1,kp)+wv(4)*vel(ip1,jp1,kp)
     * +wv(5)*vel(ip,jp,kp1)+wv(6)*vel(ip1,jp,kp1)
     * +wv(7)*vel(ip,jp1,kp1)+wv(8)*vel(ip1,jp1,kp1)
      return
c***** end of subroutine vel3 *****
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine veladj(nit)
c  routine to set up matrices for velocity inversion
c
c  declaration statements:
      integer nm(4),mbla(3)
      real r2(4),rhm(4),rhsvar(4),sqnorm(4),snrm(4),c1(2),dampc1(2)
      character*11 solnam(4)
      parameter(zero=0.0)
c
c  common block variables:
      include 'simulps_common.inc'
c
      data solnam/'Velocity   ','P-Velocity ','Vp/Vs ratio',
     2 'Sta. Corr. '/
      if(iuseq.eq.1) solnam(2)='  Qp * Vp  '
c
c  compute adjustments to velocity model
c
        write(16,385)
 385    format(/,' Compute velocity adjustments to model-Veladj')
c  remove unknowns with fewer than nmin observations
c  mbl was defined in PARSEP as no. rows (ie: no. velocity gridpts.
c    observed), in VELADJ, it's adjusted by the <hitct gridpts.
      kbl=mbl
      if (hitct.eq.0.0) go to 230  !skip next section if hitcutoff not used
c
      do 10 i=1,3
         mbla(i)=0
   10 continue
      mbl=0
      ii=0
      jj=0
c  construct inverse mapping for index
c  index points from nodes(input) to solution arrays (rhs,g), while 
c  jndex points in the other direction.
c       loop over all sampleable nodes.
c  twice as many nodes if both P and S velocity
      do 170 i=1,npari
c  g has no equations for Unsampled nodes
         if (khit(mdexfx(i)).eq.0) go to 170
         jndex(index(i))=i
  170 continue
c       now remove equations from g for poorly sampled nodes
c  jj=counter for old g array, ii=counter for new g array
      do 220 i=1,kbl
         if(khit(mdexfx(jndex(i))).eq.0) goto 180
         if(hit(mdexfx(jndex(i))).ge.hitct) goto 190
  180    jj=jj+i
         go to 220
  190    mbl=mbl+1
c  reorder rhs and index
         rhs(mbl)=rhs(i)
         index(jndex(i))=mbl
         do 210 j=1,i
            jj=jj+1
            if (khit(mdexfx(jndex(j))).eq.0) goto 210
            if(hit(mdexfx(jndex(j))).ge.hitct) goto 200
            go to 210
  200       ii=ii+1
            g(ii)=g(jj)
  210    continue
  220 continue
c  revise jndex array for reduced g array
      do 225 i=1,npari
         if(khit(mdexfx(i)).eq.0) goto 225
         if(hit(mdexfx(i)).lt.hitct) goto 225
         jndex(index(i))=i
  225 continue
c  add damping (different for Vp and Vs) to diagonal elements of g
  230 j=0
      do 240 i=1,mbl
         j=j+i
         kv=(mdexfx(jndex(i))-1)/nodes2+1
c  Correction for vp & sta corr only (w/out vp/vs)
         if((kv.eq.2).and.(iuses.eq.1)) kv=3
         if(kv.gt.3) kv=3
         g(j)=g(j)+vdamp(kv)
  240 continue
c  if desired store g and rhs for resolution and error calculation
      if (ires.eq.0) go to 290
c  store .rhs
      do 260 n=1,mbl
         rhs1(n)=rhs(n)
  260 continue
      ng=mbl*(mbl+1)/2
      do 270 n=1,ng
         g1(n)=g(n)
  270 continue
c  subtract off vdamp from diagonal of g1
      j=0
      do 280 i=1,mbl
         j=j+i
         kv=(mdexfx(jndex(i))-1)/nodes2+1
         if((kv.eq.2).and.(iuses.eq.1)) kv=3
         if(kv.gt.3) kv=3
         mbla(kv)=mbla(kv)+1
         g1(j)=g1(j)-vdamp(kv)
  280 continue
c
  290 ier=0
      write(16,20) kbl,mbl,(solnam(i+1),mbla(i),i=1,3)
  20  format(' total nodes observed=',i4,', nodes inverted for=',i4,
     2 3(',',1x,a11,':',i4))
cfhdmep
c  Save number of Vp, Vp/Vs and station correction nodes for later
c  use with resolution matrix output
      nrowp=mbla(1)
      nrows=mbla(2)
      nrowst=mbla(3)
c
c  perform lu decomposition
      call ludecp(g,g,mbl,ier)
      if (ier.ne.0) write(16,1001)
 1001 format('  *** error in ludecp ***')
      if(ier.eq.129) write(16,1610)
 1610 format('  ier=129, matrix A is algorithmically not positive ',
     2 'definite')
c  perform lu elimination
      call luelmp(g,rhs,mbl,rhs)
c  inversion complete
c
c  calculate solution vector statistics
c  mean of rhs
      ivadj=0
      if(nparvi.eq.0) goto 36
      do 22 i=1,4
         rhm(i)=0.0
         nm(i)=0
   22 continue
      do 23 n=1,npari
         if(khit(mdexfx(n)).eq.0) goto 23
         if(hit(mdexfx(n)).lt.hitct) goto 23
         k=(mdexfx(n)-1)/nxy2 + 2
         ia=k/nz + 2
         if((ia.eq.3).and.(iuses.eq.1)) ia=4
         if(ia.gt.4) ia=4
         rhm(ia)=rhm(ia)+rhs(index(n))
         nm(ia)=nm(ia)+1
         if(ia.gt.3) goto 23
         rhm(1)=rhm(1)+rhs(index(n))
         nm(1)=nm(1)+1
   23 continue
      do 24 i=1,4
         if(nm(i).gt.0.0) rhm(i)=rhm(i)/float(nm(i))
         rhsvar(i)=0.0
         sqnorm(i)=0.0
   24 continue
      do 25 n=1,npari
         if(khit(mdexfx(n)).eq.0) goto 25
         if(hit(mdexfx(n)).lt.hitct) goto 25
         k=(mdexfx(n)-1)/nxy2 + 2
         ia=k/nz + 2
         if((ia.eq.3).and.(iuses.eq.1)) ia=4
         if(ia.gt.4) ia=4
         sqnorm(ia)=sqnorm(ia)+rhs(index(n))*rhs(index(n))
         r2(ia)=rhs(index(n))-rhm(ia)
         rhsvar(ia)=rhsvar(ia)+r2(ia)*r2(ia)
         if(ia.gt.3) goto 25
         sqnorm(1)=sqnorm(1)+rhs(index(n))*rhs(index(n))
         r2(1)=rhs(index(n))-rhm(1)
         rhsvar(1)=rhsvar(1)+r2(1)*r2(1)
   25 continue
      write(16,1624) nit
      write(36,1624) nit
 1624 format(' iteration',i3,' rhs solution vector:')
      do 27 i=1,4
         if(nm(i).le.0) goto 27 
         snrm(i)=sqrt(sqnorm(i))
         rhsvar(i)=rhsvar(i)/float(nm(i))
         write(16,1625) solnam(i),rhm(i),rhsvar(i),sqnorm(i),snrm(i)
         write(36,1625) solnam(i),rhm(i),rhsvar(i),sqnorm(i),snrm(i)
   27 continue
 1625 format(9x,a11,': mean',f13.9,', variance',
     2 e14.4,', norm squared',e14.4,', norm',e14.4)
c check whether velocity solution norm is below cutoff value
      if(snrm(1).lt.snrmct) then
        nit=99
        write(16,1630) snrm(1),snrmct
 1630   format(/,' **** STOP since snrm',e14.4,' is below',
     2  ' snrmct',e14.4,' ****',/)
        return
      endif
c
c  apply adjustments to velocity model
      do 30 n=1,nparvi
c  gridpoint number in array of all nodes
         nn=mdexfx(n)
         vadj(nn)=zero
         if(khit(nn).eq.0) goto 30
         if(hit(nn).lt.hitct) goto 30
         rh=rhs(index(n))
c  calculate x and z indices of velocity grid
         k=(nn-1)/nxy2+2
         j=2+(nn-1+(2-k)*nxy2)/nx2
         i=1+nn+nx2*(2-j)+nxy2*(2-k)
         if(k.ge.nz) k=k+2      ! if s velocity node
       if(iuseq.eq.0) then
           delm=-vel(i,j,k)*rh/(1.0+rh)
       else
           delm=-vel(i,j,k+nz)*rh/(1.0+rh)
       endif
c**   Modification for S-P times
c    equations solve for delta-Vp/Vs directly
         if(k.ge.nz) delm=rh
         delma=abs(delm)
c  place upper bound on velocity adjustment
         if(k.gt.nz) then
            dvmax=dvsmx
         else
            dvmax=dvpmx
         end if
       if(iuseq.eq.0) then
            if (delma.gt.dvmax) delm=dvmax*delm/delma
            vadj(nn)=delm
       else
          delmaa = delma/vel(i,j,k)
            if (delmaa.gt.dvmax) delm=dvmax*vel(i,j,k)*delm/delma
            vadj(nn)=delm
       end if
          
c  apply adjustment to model
c** S-P CHANGE
         if (k.le.nz) then
          if(iuseq.eq.0) then
            vel(i,j,k)=vel(i,j,k)+delm
          else
            vel(i,j,k+nz)=vel(i,j,k+nz)+delm
            qnew=vel(i,j,k+nz)/vel(i,j,k)
            if(qnew.le.0.0) then
                qnew = 1.0
                vel(i,j,k+nz) = vel(i,j,k) * qval(i,j,k)
                vadj(nn) = delm-vel(i,j,k+nz)
            end if
            qadj(nn)=qnew-qval(i,j,k)
            qval(i,j,k)=qnew
          endif
       endif
         if(k.ge.nz) then
            vpvs(i,j,k-nz)=vpvs(i,j,k-nz)+delm
         endif
         ivadj=ivadj+1
   30 continue
c
      if(iuses.eq.1) goto 130
c  Compute Vs with new Vp and new Vp/Vs
        do 120 k=1,nz
           ks=k+nz
           do 115 j=1,ny
              do 110 i=1,nx
                 vel(i,j,ks)=vel(i,j,k)/vpvs(i,j,k)
  110         continue
  115      continue
  120   continue
c**
c
c Recalculate damping
c consider c1 to give relative size of 2 terms being minimized
c  min [ sum(data resid squared) + damping*(solution norm squared)]
  130 do 34 i=1,iuses
         if(i.eq.1) ssq=ssqrwp
         if(i.eq.2) ssq=ssqrws
         if(ssq.le.0) goto 34
         if(mbla(i).eq.0) goto 34
         if(nit.eq.1) c1(i)=vdamp(i)*sqnorm(i+1)/ssq
         if(nit.gt.1) dampc1(i)=c1(i)*ssq/sqnorm(i+1)
         write(16,1626) solnam(i+1),vdamp(i),c1(i),dampc1(i)
 1626    format('Damping: ',a11,' damp =',e12.3,', c1 =',e13.4,
     2      ', damp(c1) =',e12.3)
c Replace damping if idmp is turned on
c Do not change damping to be smaller
         if(dampc1(i).lt.vdamp(i)) goto 34
         if((idmp.eq.1).and.(nit.gt.1)) vdamp(i)=dampc1(i)
   34 continue
   36 continue
      isadj=0
      if(invdel.eq.0)goto 99
c  Station correction adjustments
      do 40 n=1,nsts
         nvn=n+nparv
         if(khit(nvn).eq.0) goto 40
         if((hit(nvn).lt.hitct).or.(nfixst(n).eq.1)) goto 40
         nv=ndexfx(nvn)
         vadj(nvn)=zero
         nin=index(nv)
         vadj(nvn)=rhs(nin)
         pdl(n)=pdl(n)+vadj(nvn)
         isadj=isadj+1
   40 continue
      if(iuses.eq.1) goto 99
c  S delay adjustments
      do 45 n=1,nsts
         nvn=n+nsts+nparv
         if(khit(nvn).eq.0) goto 45
         if((hit(nvn).lt.hitct).or.(nfixst(n).eq.1)) goto 45
         nv=ndexfx(nvn)
         vadj(nvn)=zero
         nin=index(nv)
         vadj(nvn)=rhs(nin)
         sdl(n)=sdl(n)+vadj(nvn)
         isadj=isadj+1
   45 continue
   99 continue
        write(16,1602) ivadj,isadj
 1602 format(i10,' velocity adjustments,',i10,' station corr. '
     2 'adjustments')
c
      return
c***** end of subroutine veladj *****
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine velbku(istop1)
c  This routine will backup the velocity and station parameters halfway
c  common block variables:
      include 'simulps_common.inc'
c
c  apply adjustments to velocity model
        ivadj=0
      if(nparvi.eq.0) goto 36
      do 30 n=1,nparvi
c  gridpoint number in array of all nodes
         nn=mdexfx(n)
         if(vadj(nn).eq.0.0) goto 30
c  calculate x and z indices of velocity grid
         k=(nn-1)/nxy2+2
         j=2+(nn-1+(2-k)*nxy2)/nx2
         i=1+nn+nx2*(2-j)+nxy2*(2-k)
         if(k.ge.nz) k=k+2   ! if S velocity node
c  compute half adjustment if first backup
         if(istop1.eq.0) vadj(nn)= -0.5*vadj(nn)
c  apply adjustment to model
c** S-P CHANGE
         if (k.le.nz) then
          if(iuseq.eq.0) then
            vel(i,j,k)=vel(i,j,k)+delm
       else
            vel(i,j,k+nz)=vel(i,j,k+nz)+delm
            qval(i,j,k)=vel(i,j,k+nz)/vel(i,j,k)
            if(qval(i,j,k).le.0.0) then
                qval(i,j,k) = 1.0
                vel(i,j,k+nz) = vel(i,j,k) * qval(i,j,k)
                vadj(nn) = delm-vel(i,j,k+nz)
            end if
          endif
       endif
         if(k.ge.nz) then
            vpvs(i,j,k-nz)=vpvs(i,j,k-nz)+vadj(nn)
         endif
         ivadj=ivadj+1
   30 continue
c
      if(iuses.eq.1) goto 36
c  Compute Vs with new Vp and new Vp/Vs
        do 120 k=1,nz
           ks=k+nz
           do 115 j=1,ny
              do 110 i=1,nx
                 vel(i,j,ks)=vel(i,j,k)/vpvs(i,j,k)
  110         continue
  115      continue
  120   continue
c**
c
   36 continue
        isadj=0
      if(invdel.eq.0)goto 99
c  Station correction adjustments
      do 40 n=1,nsts
         nvn=n+nparv
         if(khit(nvn).eq.0) goto 40
         if((hit(nvn).lt.hitct).or.(nfixst(n).eq.1)) goto 40
         if(istop1.eq.0) vadj(nvn)= -0.5*vadj(nvn)
         pdl(n)=pdl(n)+vadj(nvn)
         isadj=isadj+1
   40 continue
      if(iuses.eq.0) goto 99
c  S delay adjustments
      do 45 n=1,nsts
         nvn=n+nsts+nparv
         if(khit(nvn).eq.0) goto 45
         if((hit(nvn).lt.hitct).or.(nfixst(n).eq.1)) goto 45
         if(istop1.eq.0) vadj(nvn)= -0.5*vadj(nvn)
         sdl(n)=sdl(n)+vadj(nvn)
         isadj=isadj+1
   45 continue
   99 continue
        write(16,1602) ivadj,isadj
 1602 format(i10,' velocity adjustments,',i10,'station corr. '
     2 'adjustments')
c
      return
c***** end of subroutine velbku *****
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine veld(isp,xx,yy,zz,vx,vy,vz)
c
c*****this routine computes the derivatives of velocity
c     in x, y, and z directions
c
c  common block variables:
      include 'simulps_common.inc'
c
c  use Prothero's intmap here
      call intmap(xx,yy,zz,ip,jp,kp)
c
      ip1=ip+1
      jp1=jp+1
      kp1=kp+1
c
      xd=xn(ip1)-xn(ip)
      yd=yn(jp1)-yn(jp)
      zd=zn(kp1)-zn(kp)
c
      xf=(xx-xn(ip))/xd
      yf=(yy-yn(jp))/yd
      zf=(zz-zn(kp))/zd
c
      xf1=1.0-xf
      yf1=1.0-yf
      zf1=1.0-zf
c
c  S-velocity is stored in the 2nd half of the velocity array
      if(isp.eq.1) kp=kp+nz
      kp1=kp+1
c
c*****calculate derivatives of velocity
c
      vx=(yf1*zf1*(vel(ip1,jp,kp)-vel(ip,jp,kp))
     *+yf*zf1*(vel(ip1,jp1,kp)-vel(ip,jp1,kp))
     *+yf1*zf*(vel(ip1,jp,kp1)-vel(ip,jp,kp1))
     *+yf*zf*(vel(ip1,jp1,kp1)-vel(ip,jp1,kp1)))/xd
c
      vy=(xf1*zf1*(vel(ip,jp1,kp)-vel(ip,jp,kp))
     *+xf*zf1*(vel(ip1,jp1,kp)-vel(ip1,jp,kp))
     *+xf1*zf*(vel(ip,jp1,kp1)-vel(ip,jp,kp1))
     *+xf*zf*(vel(ip1,jp1,kp1)-vel(ip1,jp,kp1)))/yd
c
      vz=(xf1*yf1*(vel(ip,jp,kp1)-vel(ip,jp,kp))
     *+xf*yf1*(vel(ip1,jp,kp1)-vel(ip1,jp,kp))
     *+xf1*yf*(vel(ip,jp1,kp1)-vel(ip,jp1,kp))
     *+xf*yf*(vel(ip1,jp1,kp1)-vel(ip1,jp1,kp)))/zd
c
      return
c ***** end of subroutine veld *****
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine wthyp(ne,nobs,ah,rmswt,nwr,w)
c  routine to apply weighting to hypocenter matrix + residuals
c
c  common block variables:
      include 'simulps_common.inc'
c
c  declaration statements:
      real w(maxobs)
      double precision ah(maxobs,maxobs)
      character*1 phs(2)
      parameter (pi=3.1415927)
c
      data phs/'P','S'/
c
c  compute weighting
c
      nwr=0
      wnorm=0.0
      do 13 j=1,nobs
c  reading weight
         w(j)=wt(j,ne)
c  change relative weighting of S-P observations
         if(intsp(j,ne).eq.1) w(j)=w(j)*wtsp
c  residual weighting
c  downweighting(linear) 0 to 98% res1 to res2, 98 to 100% res2 to res3
         ares=abs(res(j))
         if(ares.le.res2) then
            wr=1.0-(ares-res1)*dres12
            if (wr.gt.1.0) wr=1.0
         else
            if(res3.gt.res2) then
               wr=0.02-(ares-res2)*dres23
               if (wr.lt.0.0) wr=0.0
            else
               wr=0.0
            endif
         endif
c  distance weighting
         wd=1.0-(dlta(j,ne)-delt1)*ddlt
         if (wd.gt.1.0) wd=1.0
         if (wd.lt.0.0) wd=0.0
c  unnormalized weight
         w(j)=w(j)*wr*wd
c
         if(i3d.eq.4.and.ides.gt.1) write(61,*)
     .    'in wthyp w(no) = ',w(j),' wr= ',wr,' wd= ',wd
         wnorm=wnorm+w(j)
         if (w(j).gt.0.0) nwr=nwr+1
  13  continue
c  check to be sure 4 or more readings with nonzero weight
      if(nwr.lt.4) goto 900
c  normalize weights
   12 wfac=nwr/wnorm
c
   20 continue
      wnorm=0.0
      rmswt=0.0
c  normalize weights, apply to hypocenter matrix, and place into
c  a-matrix for inversion
      do 30 i=1,nobs
         w(i)=w(i)*wfac
         wnorm=wnorm+w(i)*w(i)
         rmswt=rmswt+(res(i)*w(i))*(res(i)*w(i))
         do 35 j=1,4
            ah(i,j)=dth(i,j)*w(i)
   35    continue
         ah(i,5)=res(i)*w(i)
   30 continue
      rmswt=sqrt(rmswt/wnorm)
      return
  900 write(16,1601) ne
 1601 format(' in WTHYP, event',i5,' HAS LESS THAN 4 READINGS WITH ',
     2 '  NON-ZERO WEIGHT, SHOULD BE DELETED')
      write(16,51)
51    format(1x,4('sta ph   wt   res   ttime delta',2x))
      write(16,53) (stn(isto(j,ne)),phs(intsp(j,ne)+1),w(j),res(j),
     2 (secp(j,ne)-seco(ne)),dlta(j,ne),j=1,kobs(ne))
53    format(4(1x,a4,1x,a1,f6.2,f7.3,2f6.2,1x))
      write(16,1602)
 1602 format(/)
      return
c***** end of subroutine wthyp *****
      end
c **
cfh now follow paraxial raytracer subroutines
c **
c 
      subroutine input5
c routine to read in control parameters special for paraxial RT
c reads from file RAYTRAC
      include 'simulps_common.inc'
      include 'rayparax_common.inc'
      common/rayde/kmahi,ides,ampr,iterrai,itime
      common/heter2/iheter,iheter1,epsob,epsca,itsta
c
      open(unit=55,file='RAYTRAC',status='old')
      read(55,*) iheter,epsob,epsca,ides,ampr,iterrai
      read(55,*) dxrt,dyrt,dzrt
c
      write(16,'(//)')
      write(16,*)' Control parameters for paraxial raytracing'
      write(16,*)'   read from file RAYTRAC:'
      write(16,'(/)')
      write(16,*)'iheter, epsob, epsca, ides, ampr, iterrai'
      write(16,*) iheter,epsob,epsca,ides,ampr,iterrai
cfh --- this is for synthetics or with station residuals (previously was in preparax)
      iheter1=iheter
      if(iheter.eq.13.or.iheter.eq.23) then
        iheter1=3
        write(16,*)'iheter1 = 3: calculate synthetic data in this run'
      endif
c      
      write(16,*)'dxrt, dyrt, dzrt '
      write(16,*)dxrt, dyrt, dzrt 
c
      close(55)
      end
c end of subroutine input5
c
      subroutine bldrtgrd
c  routine to build regular grid with velocity values
c  from inversion grid (uneven, with velocities)
cc  Use greatest common divisor of gridpoint-distances along
cc  each direction for new sampling intervals in x,y,z.
cc  (or implement constant sampling interval)
c  sampling interval for rt-grid now user defined (dxrt, dyrt, dzrt)
c  read into array uu(i,j,k)
c
c  common block variables:
      include 'simulps_common.inc'
      include 'rayparax_common.inc'
      common/rayde/kmahi,ides,ampr,iterrai,itime
      common/cadre/xfmin,xfmax,yfmin,yfmax,zfmin,zfmax
c
      integer i,j,k,kv,k2
cfh      integer a,b,l,m,n
      real dx,dy,dz              ! these values should be whole integers if bld=1.0
cfh      integer gcd
cfh      integer xd(maxnx),yd(maxny),zd(maxnz)
cfh test
c      real uu2(200,200,100)
c      
      if (ides.gt.1) then
         write(61,*)' in routine bldrtgrd'
      endif
c 
cfh now use values read in from control-file RAYTRAC
cfh they are taken to be multiples of bld
      dx = dxrt
      dy = dyrt
      dz = dzrt
c
      if (ides.ge.1) then
         write(61,*)'raytracing grid sampling:'
         write(61,*)' dx, dy, dz'
         write(61,'(3f6.2)') dx, dy, dz
      endif
c compute dimensions of vectors [xyz]nu and array uu      
c
c   - reduce grid size in x and y: requires that the first non-boundary nodes (2, n[x,y]-1)
c     encompass all the information (stations, !!earthquakes!! and rays). then it is enough
c     to have 2 additional uu-nodes outside this last boundary (actually one would be enough,
c     but make it 2 for the rare case that there is information ON the node). this should nor-
c     mally be less than the distance to the fixed boundaries of the inversion grid...
c     new uu-grid size: (2 - n[x,y,z]-1) +- 2* d[x,y,z]1      
cfh for now only for x and y...
c      nxu = int((xn(nx) - xn(1))/dx) +1
c      nyu = int((yn(ny) - yn(1))/dy) +1
cfh ARTPB often needs really deep last node - makes grid too big..
cfh  so rely on nz-1 deep enough - if nzu .gt. 100...
      nzu = int((zn(nz) - zn(1))/dz) +1
      if (nzu.gt.100) nzu= int(((zn(nz-1)+2*dz) - zn(1))/dz) +1
      nxu = int(((xn(nx-1)+2*dx) - (xn(2)-2*dx))/dx) +1
      nyu = int(((yn(ny-1)+2*dy) - (yn(2)-2*dy))/dy) +1
c      nzu = int(((zn(nz-1)+4*dz1) - (zn(2)-4*dz1))/dz) +1
      if ((nxu*nyu*nzu*iuses).gt.maxparuu) then
         write(61,*)'size of squared slowness array uu too large'
         write(61,*)'maxparuu = ',maxparuu
         write(61,*)'required size = ',nxu*nyu*nzu*iuses
         write(61,*)'program stops in subroutine bldrtgrd'
         write(16,*)'size of squared slowness array uu too large'
         write(16,*)'maxparuu = ',maxparuu
         write(16,*)'required size = ',nxu*nyu*nzu*iuses
         write(16,*)'program stops in subroutine bldrtgrd'
         stop
      endif
      if (ides.ge.1) then
         write(61,*)' uu-grid dimensions:'
         write(61,*)' nxu, nyu, nzu '
         write(61,'(3i4)') nxu,nyu,nzu
      endif
cfh      xnu(1) = xn(1)
      xnu(1) = xn(2)-2*dx
      do i=2,nxu
        xnu(i) = xnu(i-1) + dx
      enddo
cfh      ynu(1) = yn(1)
      ynu(1) = yn(2)-2*dy
      do j=2,nyu
        ynu(j) = ynu(j-1) + dy
      enddo
      znu(1) = zn(1)
c      znu(1) = zn(1)-2*dz
      do k=2,nzu
        znu(k) = znu(k-1) + dz
      enddo
c get min and max values
      xfmin=xnu(1)
      xfmax=xnu(nxu)
      yfmin=ynu(1)
      yfmax=ynu(nyu)
      zfmin=znu(1) 
      zfmax=znu(nzu)
      if(ides.ge.1) then
        write(61,*)'xfmin, xfmax: ',xfmin, xfmax
        write(61,*)'yfmin, yfmax: ',yfmin, yfmax
        write(61,*)'zfmin, zfmax: ',zfmin, zfmax
      endif
c
c now fill array uu - does this take long?
      do kv=1,iuses
      do k=1,nzu
        k2 = k + (kv-1)*nzu
        v=0.
        do j=1,nyu
          do i=1,nxu
             call vel3(kv-1,xnu(i),ynu(j),znu(k),v) 
c             uu2(i,j,k2) = (1./v)**2
cfh here now we use velocities instead of slowness...
             uu(i,j,k2) = v
          enddo
cfh TEST 
c         write(99,'(25(f6.4,2x))') (uu(i,j,k2),i=1,nxu)
        enddo
c        write(99,*)'lay, vel = ',k2,v
c        write(99,'(//)')
      enddo
      enddo             
c***** end of soubroutine bldrtgrd
      end
c
      integer function gcd(a,b)
c to compute the greatest common divisor of 2 integers
c with euclidian algorithm
c a,b do not have to be sorted on call
c
      integer a,b,t
      if (a.gt.b) then
         t=b
         b=a
         a=t
      endif
 100  continue
      if (a.gt.0) then
         t=mod(b,a)
         b=a
         a=t
         goto 100
      endif
      gcd=b
c end of function gcd
      return
      end
c

c
      subroutine preparax(ne,nobs,iprep)
c to prepare stuff for paraxial ray tracing (read angles from file,
c map events
c fh. 9804
c
      include 'simulps_common.inc'
      include 'rayparax_common.inc'
      common/heter2/iheter,iheter1,epsob,epsca,itsta
      common/rayde/kmahi,ides,ampr,iterrai,itime
c
      real xev(3)
      real tet_sav(maxobs),phi_sav(maxobs),ji_sav(maxobs)
      common/prepare/xev,tet_sav,phi_sav,ji_sav
c
      integer no
      character*4 extension
      character*80 fichier
c
c-------------- test si deja angles initiaux
cfh --- test if there are already initial angles
      do no=1,nobs
         tet_sav(no)=-999.
         phi_sav(no)=-999.
      enddo
      if(iterrai.eq.2.or.iprep.eq.1) then
c--------------- il faut trouver le fichier des angles correspondant
c--------------- a ce seisme   kevent
cfh --- get the file which contains the angles for that event
         write(extension,'(i4)') ne
         do i=1,4
            if(extension(i:i).eq.' ') extension(i:i)='0'
         enddo
         fichier='angle.'//extension
         open(121,file=fichier,status='old')
         read(121,*) (tet_sav(no),phi_sav(no),
     .        ji_sav(no),no=1,nobs)
         close(121)
      endif
c-------------- a-t-on un event ?
cfh -- should be tested before - useless here.
c      if(kobs(neit).lt.0) then
c        return
c      endif
c----------------- est-on dans un cas test
cfh --- this is for synthetics or with station residuals
cfh -- better put that into input5 and write it to output for tests...
c      iheter1=iheter
c      if(iheter.eq.13.or.iheter.eq.23) iheter1=3
c
      xev(1)=evc(1,ne)
      xev(2)=evc(2,ne)
      xev(3)=evc(3,ne)
      if(ides.gt.1) then
         write(61,*) ' ::::::::::::::::::::::::::::::::'
         write(61,*) ' srce posi lat :',ltde(ne),eltm(ne),xev(2)
         write(61,*) '           lon :',lnde(ne),elnm(ne),xev(1)
         write(61,*) '             z :',xev(3)
         write(61,*) ' ne, kobs ',ne,kobs(ne)
      endif
c-------------- boucle generale sur les stations
cfh --- general loop over stations
cfh --- I don't understand for what he needs itsta (at the end used to set iflag, which
cfh ---  is never used in a sensible way...)
      itsta=0
c      if(iteres.eq.1) write(78,*) ne
cfhRT - that's it for now
      end
c end of subroutine preparax
c
c 
      subroutine finparax(ne,nobs)
c short soubroutine to write angles to file after paraxial raytracing
c
      include 'simulps_common.inc'
      include 'rayparax_common.inc'
c
      real xev(3)
      real tet_sav(maxobs),phi_sav(maxobs),ji_sav(maxobs)
      common/prepare/xev,tet_sav,phi_sav,ji_sav
      common/rayde/kmahi,ides,ampr,iterrai,itime
c
      integer no
      character*4 extension
      character*80 fichier
c
      write(extension,'(i4)') ne
      do i=1,4
         if(extension(i:i).eq.' ') extension(i:i)='0'
      enddo
      fichier='angle.'//extension
      if(ides.gt.1) write(61,*) ' event file for finparax',fichier
      open(121,file=fichier,status='unknown')
      if(ides.gt.1) write(61,*)' writing for ',
     . nobs,' observations to file in finparax'
      write(121,*) (tet_sav(no),phi_sav(no),
     .     ji_sav(no),no=1,nobs)
      close(121)
      end
c end of subroutine finparax
c
c
      subroutine trvhet(ne,no,ttime,iprep)
c routine taken from Jean Virieux's version of sim5, modified to
c take only one obs per time and to be comaptible with simulps code
c fh. 9803
c added iprep to check for existence of initial angles
c
c fh - 2206 now use ART from Thurber to get initial angles tet and phi
c    (call rayweb and then use 2 first raypoints for angle computation)
c**********************************************************************
c trace de rais en milieu heterogene avec recherche de la station     *  
c trace initial dans un milieu 1-D pour angles initials               *  
c determination du temps et du vecteur lenteur initial                *  
c attention version rka pour inversion stochastique
c version qui construit vecteurs locaux    pas de nparob              *  
c-------- nouvelle version pour stocker les angles initiaux           *  
c**********************************************************************
c decembre 1994     ne est l'increment des evenements selectionnes
c                  neit est le compteur des evenements depuis le debut
c             ne peut fluctuer, neit ne peut pas
**********************************************************************
c common/heter2 : isynt : construire les synthetiques (0/1)
c common/heter2 : iteres : construire les residus aux stations (0/1)
c common/rayde  : iterrai : flag pour les angles
c&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
c files 78 
c       79 pour les rais   (mais probleme)
c       35 pour les projections des residus....
c============================
cfh isynt and iteres are not used any more, as well as file 78
c  commons:
      include  'simulps_common.inc'
      include 'rayparax_common.inc'
c     common/obsiw/iw(npobs,npsei) !in simulps_common.inc
      common/heter2/iheter,iheter1,epsob,epsca,itsta
      common/rayde/kmahi,ides,ampr,iterrai,itime
c
      real xev(3)
      real tet_sav(maxobs),phi_sav(maxobs),ji_sav(maxobs)
      common/prepare/xev,tet_sav,phi_sav,ji_sav
      integer qzfc,qzfcold
c       
c xe(3) should be replaced, as xe,ye,ze are used in calling program
c     xev() is now in common/prepare/ (coming from subroutine preparax)
      dimension xo(3)
      dimension tete(50),phie(50),tetc(50),phic(50)
c these 3 are needed in the calling program, where the file is read
c       dimension tet_sav(120),phi_sav(120),ji_sav(120)
c
      data pi/3.141592/,epstea/1.e-3/
c jean uses neit for event counter ne, so just map it here
      neit=ne
c angle-file is opened before calling trvhet
c 
cfh the loop over all observations is now done in the calling routine
c      do 1000 no=1,kobs(neit)
      if (ides.gt.1) then
         write(61,*)'starting trvhet for event, obs: ',neit,no
      endif
      ttime = 0.
cfh set iflrai to 2 at the beginning - should be set to one if rkp-raytracing
c   is successful. 
      iflrai(no,neit)=2
      ji=isto(no,neit)
      isp=intsp(no,neit)
      itime=0                   ! generally no t-time calculation in ray2 
c
c-------------- calcul de la position de la station
cfh --- calculate station position
      xo(1)=stc(1,ji)
      xo(2)=stc(2,ji)
      xo(3)=stc(3,ji)
      if(ides.gt.1) then
         write(61,*) ' ::::::::::::::::::::::::::::::::',ne,no,ji,isp
         write(61,*) ' station position x :',xo(1)
         write(61,*) '                  y :',xo(2)
         write(61,*) '                  z :',xo(3)
      endif
cfh we need the epicentral distance to handle upward/downward shooting
cfh  (hopefully). dlta(no,neit) has to be the epicentral dist, (check on input4)
      edlt=dlta(no,neit)
c==================== c'est le cas ou pas d'angles connus que faire
cfh --- the case if we have no known angles
      if(iterrai.eq.1.and.iprep.eq.0) then
cfh use Thurbers ART+PB for at the beginning to find initial 
cfh  take-off angle and azimuth
         tmin=0.
         if(ides.gt.1) write(61,*)' using ART+PB '
cfh use rayweb here for thurber ART
cfh  ncrold, ndpold are not used if jflag=0
c        write(61,*)' calling rayweb '
         ncrold=nco(no,neit)
         ndpold=ndo(no,neit)
         call rayweb(no,isp,xev(1),xev(2),xev(3),xo(1),xo(2),xo(3),
     .        tmin,0,ncrold,ndpold)
        nco(no,neit)=ncrold
        ndo(no,neit)=ndpold
cfh ART is not good enough for initial angles in really heterogeneous 3D structure-
cfh  use pseudo-bending as well (nitpb fixed to 15?)
        call minima(no,isp,tmin,15,jpb)
        if(jpb.lt.15) goto 130
        write(61,2661)no,stn(ji)
 2661   format(' Minima: no=',i4,2x,a4,', used max. no PB iter')
  130   continue
        atim = tmin
        iflrai(no,neit)=2
c now compute tet (= azimuth) and phi (= take of angle) from
c first points of raypath
c         pxtir=-dx(1)
        pxtir=rp(1,2,no)-rp(1,1,no)
c         pytir=-dx(2)
        pytir=rp(2,2,no) -rp(2,1,no)
c         pztir=-dx(3)
        pztir=rp(3,2,no) -rp(3,1,no)
        if(ides.gt.1) write(61,*)'pxtir, pytir, pztir ',
     .     pxtir,pytir,pztir 
c-------------- recherche des angles
cfh --- calculate the angles
         rho=pxtir**2+pytir**2
         utir2=rho+pztir**2
         utir=sqrt(utir2)
         tet=acos(pztir/utir)
         rho=sqrt(rho)
         if(abs(pytir).gt.1.e-4) then
            if(rho.gt.1.e-5) then
               phi=acos(pxtir/rho)
               if(pytir.lt.0.) phi=2.*pi-phi
            else
               phi=0.
            endif
         else
            phi=0.
            if(pxtir.lt.0.) phi=pi
         endif
      else
c-------------- on a deja tire avant et on peut aller a la station
c-------------- on verifie qu'on est bien dans la meme station
cfh --- it was shot before (there should be initial angles) and we can look 
cfh --- up the station. verify that we aree still at the same station
cfh -- ?what is the use of that, how can we be not at the same station anymore??
c
cfh - in case we don't need ART any more, set atim to -99.999 to recognize in f19
cfh -- does that create the large residuals??
         atim = -99.999
c
         if(ji_sav(no).eq.ji) then
            if (ides.gt.1) write(61,*)'station found in angle file: '
     .      ,ji
            tet=tet_sav(no)
            phi=phi_sav(no)
         else
c-------------- on recherche dans les autres cas
cfh --- if not at same station
            isucc=0
            do no1=1,kobs(neit)
               if(ji_sav(no1).eq.ji) then
                  tet=tet_sav(no1)
                  phi=phi_sav(no1)
                  isucc=1
               endif
            enddo
            if(isucc.eq.0) then
               tet=-999.
               phi=-999.
            endif
         endif
c-------------- si tet=-999. alors erreur anterieure
cfh --- if tet =-999. there was an error before
cfh -- (either no ray found for that observation or ???)
         if(abs(tet+999.).lt.1.) then
            dist_sta_sav=1.e+24
c-------------- on recherche une station la plus proche
cfh --- look up the nearest station
cfh -- I am not sure whether this makes much sense, e.g
cfh -- if the stations are on either side of the eq, then the angles
cfh -- would be really different!!
            nox=-99
            do no1=1,kobs(neit)
               if(no1.ne.no.and.abs(tet_sav(no1)+999.).gt.1.) then
                  ji1=isto(no1,neit)
                  dist_sta=(stc(1,ji1)-xo(1))**2+(stc(2,ji1)-xo(2))**2
     1                 +(stc(3,ji1)-xo(3))**2
                  if(dist_sta.lt.dist_sta_sav.and.
     &                 isp.eq.intsp(no1,neit)) then
                     tet_sav(no)=tet_sav(no1)
                     phi_sav(no)=phi_sav(no1)
                     tet=tet_sav(no1)
                     phi=phi_sav(no1)
                     dist_sta_sav=dist_sta
                     nox=no1    !fh, to remember the station
                  endif
               endif
            enddo
            if (nox.gt.0) then
              jix = isto(nox,neit)
              if(ides.gt.1) then
                write(61,*) 'station not found in angle file'
                write(61,'(a)') 'using instead: ',stn(jix)
              endif
            endif
            if(dist_sta_sav.gt.0.99e+24) then
c-------------------- attention pas d'autres stations
c-------------------- alors faire le tir a partir milieu stratifie
cfh --- no other station!  (i.e every other station has either not observed this event
cfh -- or no ray has been found previously. very pathologic!
cfh --- so use ART+PB again 
               tmin=0.
cfh use nco from previous (at least on first iteration) rayweb runs to start from
               ncrold = nco(no,neit)
               ndpold = ndo(no,neit)
               call rayweb(no,isp,xev(1),xev(2),xev(3),xo(1),xo(2),
     .             xo(3),tmin,0,ncrold,ndpold)
               nco(no,neit)=ncrold
               ndo(no,neit)=ndpold
               call minima(no,isp,tmin,15,jpb)
               if(jpb.lt.15) goto 131
                 write(61,2661)no,stn(ji)
  131    continue
               atim = tmin
               iflrai(no,neit)=2
c now compute tet (= azimuth) and phi (= take of angle) from
c first points of raypath
         pxtir=rp(1,2,no)-rp(1,1,no)
         pytir=rp(2,2,no) -rp(2,1,no)
         pztir=rp(3,2,no) -rp(3,1,no)
c-------------- recherche des angles
               rho=pxtir**2+pytir**2
               utir2=rho+pztir**2
               utir=sqrt(utir2)
               tet=acos(pztir/utir)
               rho=sqrt(rho)
               if(abs(pytir).gt.1.e-4) then
                  if(rho.gt.1.e-5) then
                     phi=acos(pxtir/rho)
                     if(pytir.lt.0.) phi=2.*pi-phi
                  else
                     phi=0.
                  endif
               else
                  phi=0.
                  if(pxtir.lt.0.) phi=pi
               endif
            endif
c ------------- fin de recherche d'une station voisine ayant marchee
cfh --- end of look up for station ...
         endif
      endif
c------------------ on arrive ici ave tet et phi pour la station courante
c-------------- on peut tirer
cfh --- arrived here with tet and phi for the current station
cfh --- can shoot
      if(ides.gt.1) then
         write(61,*) ' angle tet,phi ',tet*57.29,phi*57.29
      endif
      iter=0
      dtet=5.e-3
      dphi=5.e-3
cfh ? something to be adjusted if source above station?
      qzfc= nint(sign(1.,(xev(3)-xo(3))))
      qzfcold = qzfc
      half=1.0
c      write(6,*)'qzfc = ',qzfc
cfh test - zero out rp-array again? 
cfh save ray from ART-pb in case shooting is not successful
      nrpt = nrp(no)
      do i=1,nrp(no)
        do j=1,3
           rtmp(j,i,no) = rp(j,i,no)
           rp(j,i,no) = 0
        enddo
      enddo
      nrp(no) = 0
c
cfh here start iteration loop for raytracing
  202 iter=iter+1
      tete(iter)=tet
      phie(iter)=phi
      dtet1=dtet
      dphi1=dphi
      zps=xo(3)
cfh 
c
c-------------- modification du maillage
cfh ---  modification of mesh (?)
cfh -- 5 tries to get a ray to the surface in the slowness grid
cfh -- if this is not successful apply weight of 4 to this observation
cfh -- and proceed with next obs.
      iloop=0
cfh test
c      write(61,*)'tet and phi for ray2 (deg), iter:'
c      write(61,'(2(f8.4,2x),i3)')tet*57.29, phi*57.29,iter
cfh test end
 666  call ray2(no,isp,tet,phi,xev(1),xev(2),xev(3),edlt,
     1     pxtir,pytir,pztir,
     2     xps,yps,zps,pxs,pys,pzs,ts,
     3     qq1x,qq1y,pp1x,pp1y,qq2x,qq2y,pp2x,pp2y,itest,ier,1,0,qzfc)
c
c-------------- piege pour positi mal passe ou perturb ou teq
cfh      xerr=(ier-100)
cfh      write(61,*)'xerr: ',xerr
cfh xerr.eq.0 means that ray did not hit surface - try again
cfh  also...
cfh      if(xerr.ge.0) then ! try with xerr instead of ierr...
      if(ier.ge.100) then 
cfh using ART on these grids makes curvature sometimes too big
cfh  -- tet too small. so adjust tet more here 
cfh very fancy, increase correction from iteration to iteration
cfh also use qzfc to adjust direction of correction 
         tet = tet + (iloop+1)*0.0088*qzfc
         phi=phi+0.0005
         iloop=iloop+1
         if (ides.gt.1) then
c            write(61,*) ' I try again '
            write(61,*)'new tet, phi (deg); iteration:'
            write(61,'(2(f8.4,2x),i3)')tet*57.29, phi*57.29,iter
         endif
         if(iloop.lt.5) goto 666
      endif
c-------------- trace mal passe : on garde les valeurs deja calculees
cfh --- bad ray. save dummy values 
      if(ier.ne.0) then
         if(ides.ne.0) write(61,6)
     1        ' station error ',stn(ji),no,sqrt(dist),iter,ier
 6       format(a15,' ! ',a4,' num ',i3,' : ',f10.5,' km ? ',2i4)
         tet_sav(no)=-999.
         phi_sav(no)=-999.
         ji_sav(no)=-999
cfh if iterrai.eq.2 for this case again use ART+PB to find a ray 
cfh  - was not yet done before because initial angle taken from file
         if(iterrai.eq.2.or.iprep.eq.1) then
           if(ides.ne.0) write(61,*)'shooting failed,using again ART+PB'
           ncrold = nco(no,neit)
           ndpold = ndo(no,neit)
           call rayweb(no,isp,xev(1),xev(2),xev(3),xo(1),xo(2),
     .         xo(3),tmin,0,ncrold,ndpold)
           nco(no,neit)=ncrold
           ndo(no,neit)=ndpold
           call minima(no,isp,tmin,15,jpb)
           if(jpb.lt.15) goto 138
             write(61,2661)no,stn(ji)
  138      continue
           atim = tmin
           iflrai(no,neit)=2
cfh save ray from ART-pb in case shooting is not successful
          nrpt = nrp(no)
          do i=1,nrp(no)
            do j=1,3
              rtmp(j,i,no) = rp(j,i,no)
              rp(j,i,no) = 0
            enddo
          enddo
         endif
c-------------- residus plus grand que res2
cfh --- don't need to compute residuals in here?!
c@         if(isp.eq.0) then
c@            res(no)=secp(no,neit)-seco(neit)-pdl(ji)-tmin
c############################### JEAN
c@            iw(no,neit)=4
c@         else
c@            res(no)=secp(no,neit)-seco(neit)-sdl(ji)-tmin
c############################### JEAN
c@            iw(no,neit)=4
c@         endif
         goto 1000
      endif
c-------------- test pour proximite
cfh --- test for closeness of ray surfacing point to station
      dist=(xps-xo(1))**2+(yps-xo(2))**2
      if(ides.gt.1) write(61,*) ' dist ',sqrt(dist),' xps, yps, zps: '
     . ,xps, yps, zps
c-------------- y-a-t-il convergence
cfh --- check for convergence
cfh -- if surfacing point not close enough (epsob) to station and still
cfh -- final number of ray tracing iterations not reached, get better
cfh -- angle and start again. if surfacing point after iheter iterations
cfh -- still far away (epsca) then station is missed.
      if(dist.gt.epsob.and.iter.lt.iheter) goto 201
      if(dist.gt.epsca.and.iter.eq.iheter) then
c-------------- convergence insuffisante
cfh --- station missed...
         if(ides.ne.0) write(61,6)
     1        ' station missed',stn(ji),no,sqrt(dist),iter
         tet_sav(no)=-999.
         phi_sav(no)=-999.
         ji_sav(no)=-999
cfh if iterrai.eq.2 for this case again use ART+PB to find a ray 
         if(iterrai.eq.2.or.iprep.eq.1) then
           if(ides.ne.0) write(61,*)'shooting failed,using again ART+PB'
           ncrold = nco(no,neit)
           ndpold = ndo(no,neit)
           call rayweb(no,isp,xev(1),xev(2),xev(3),xo(1),xo(2),
     .         xo(3),tmin,0,ncrold,ndpold)
           nco(no,neit)=ncrold
           ndo(no,neit)=ndpold
           call minima(no,isp,tmin,15,jpb)
           if(jpb.lt.15) goto 139
             write(61,2661)no,stn(ji)
  139      continue
           atim = tmin
           iflrai(no,neit)=2
cfh save ray from ART-pb in case shooting is not successful
          nrpt = nrp(no)
          do i=1,nrp(no)
            do j=1,3
              rtmp(j,i,no) = rp(j,i,no)
              rp(j,i,no) = 0
            enddo
          enddo
         endif
c-------------- residus plus grand que res2
cfh --- ???
c@         if(isp.eq.0) then
c@            res(no)=secp(no,neit)-seco(neit)-pdl(ji)-tmin
c############################### JEAN
c@            iw(no,neit)=4
cfh iw(no,neit) is not used any further in the inversion routines
cfh - as we now use the ART-PB for this ray, we keep the original weight
cfh -- need some information (on last iter) whether raytracing done with ART or RKP
c@         else
c@            res(no)=secp(no,neit)-seco(neit)-sdl(ji)-tmin
c############################### JEAN
c@            iw(no,neit)=4
c@         endif
         goto 1000
      endif
c////////////// tir reussi on calcule le temps et les derivees
cfh /// shooting ok, calculate time. derivatives not needed - they are 
cfh    calculated with the original simulps -routines. ideriv=1 means 
cfh     put out raypoints now
      itime=1
      call ray2(no,isp,tet,phi,xev(1),xev(2),xev(3),edlt,
     1        pxtir,pytir,pztir,xps,yps,zps,pxs,pys,pzs,ts,
     2        qq1x,qq1y,pp1x,pp1y,qq2x,qq2y,pp2x,pp2y,
     3        itest,ier,1,1,qzfc)
      itime=0
cfh -- now call tiraju again to get factors to adjust traveltime according
cfh     to ray surfacing point
      call tiraju(dtet1,dphi1,xo(1),xo(2),xps,yps,
     1     qq1x,qq1y,qq2x,qq2y)
      ddx=xo(1)-xps
      ddy=xo(2)-yps
c-------------- quadratic approximation of travel time
c------------- dtet from qq2 and dphi from qq1
c     qqx=dphi1*qq1x+dtet1*qq2x        this is  dx
c     qqy=dphi1*qq1y+dtet1*qq2y        this is  dy
      ppx=dphi1*pp1x+dtet1*pp2x
      ppy=dphi1*pp1y+dtet1*pp2y
      temps=ts+ddx*pxs+ddy*pys+0.5*(ddx*ppx+ddy*ppy)
c-------------- correction si tiraju va dans les choux
cfh --- correction if tiraju went bogus (?)
      if(abs(temps-ts).gt.1.5) then
         temps=ts
      endif
c-------------- output
      itsta=itsta+1
      tet_sav(no)=tet
      phi_sav(no)=phi
cfh describe
      if(ides.gt.1) then
        write(61,*)' final tet,phi saved for this obs:'
        write(61,'(2(f8.4,2x))')tet*57.29, phi*57.29
      endif
cfh describe end
      ji_sav(no)=ji
c
cfh -- iflrai is the flag to check that for this obs the found ray is ok for inversion
cfh -- in jean's code
      iflrai(no,neit)=1
c============================================= ajout ici
c============== calcul du probleme direct
c============== si iheter1=3
c=============================================
cfh --- calculation for synthetic ttimes
      if(iheter1.eq.3) then
         if(isp.eq.0) then
            secp(no,neit)=seco(neit)+pdl(ji)+temps
         else
            secp(no,neit)=seco(neit)+sdl(ji)+temps
         endif
c here set residuals for synthetics to '0'
         res(no)= 0.00
      endif
c-------------- residus comobs + comhyp
cfh --- residuals
cfh -- do we need them?
cfh (probably not, as they are computed in ttmder, but not for synthetics!)
c@ synthetics don't have residuals! so this is useless
c@      if(isp.eq.0) then
c@         res(no)=secp(no,neit)-seco(neit)-pdl(ji)-temps
c@      else
c@         res(no)=secp(no,neit)-seco(neit)-sdl(ji)-temps
c@      endif
cfh in the rare case where res(no) leads to format-boundary problems
c@      if(abs(res(no)).le.1e-8) then
c        if(ides.ne.0) write(61,*)'autsch'
c@         res(no)= 0.00
c@      endif
c
      if(ides.gt.1) write(61,7)
     1     ' sta dist,iter ',stn(ji),no,sqrt(dist),iter,
     2     iheter,temps,ts,tmin
 7    format(a15,' ! ',a4,' num ',i3,' : ',f10.5,' km ? ',2i4,
     1     f8.4,'s st',f8.4,'s ra',f8.4,'s la')
c     
cfh no more adjustment of ray 
      goto 1000
c-------------- on doit iterer on saute les caustiques
cfh --- one should iterate one should jump the caustics (??)
cfh -- what happens is that here we adjust the shooting angle 
cfh -- if we were not close enough to the station and have
cfh -- some raytracing iterations left
 201  continue
      call tiraju(dtet1,dphi1,xo(1),xo(2),xps,yps,
     1     qq1x,qq1y,qq2x,qq2y)
c--------------on limite a 10 degree
cfh ! limit that to 5 degrees - rays are too wrong...
cfh try original again
      if(dtet1.gt.0.) then
c         dtet1=amin1(dtet1,0.1745)
          dtet1=amin1(dtet1,0.0873)
      else
c         dtet1=amax1(dtet1,-0.1745)
          dtet1=amax1(dtet1,-0.0873)
      endif
      if(dphi1.gt.0.) then
c         dphi1=amin1(dphi1,0.1745)
          dphi1=amin1(dphi1,0.0873)
      else
c         dphi1=amax1(dphi1,-0.1745)
          dphi1=amax1(dphi1,-0.0873)
      endif
cfh - if qzfc has changed sign from the previous to this iteration, halve the
cfh    tet-adjustment - we are really close, and diving waves need smaller angles      
      if(qzfc-qzfcold.eq.0) half=0.5
      tet=tet+dtet1*half
      phi=phi+dphi1
c
c is the teas deconstruction really neccessary now??? (jean commented it out)    
c-------------- destruction de la suite dans teas
cfh --- try to find limit of tet, phi-sequences from
cfh --- previous iterations (only for iter.ge.10)
cfh -- try and take only last 10 tet,phi (hope for more stable convergence...)
      if(iter.ge.9) then
       do 311 ite=1,9
       tetc(ite)=tete(iter-9+ite)
       phic(ite)=phie(iter-9+ite)
  311  continue
       iterp=iter+1
       tetc(iterp)=tet
       phic(iterp)=phi
       call teas(tetc,iterp,tetf,epstea,ierte)
       call teas(phic,iterp,phif,epstea,ierpe)
       if(ierte.eq.0.and.ierpe.eq.0) then
cfh - around a caustic (jumping angles) teas does 'fix' an angle sometimes
cfh   so apply adjustment only if results differ from tet(iter),phi(iter)
         dtetf= abs(tetf-tete(iter))
         if(dtetf.gt.1e-3) tet=tetf
         dphif=abs(phif-phie(iter))
c         write(61,*)'dphif: ',dphif
         if(dphif.gt.1e-3) phi=phif
       endif
      endif
      qzfcold = qzfc
      goto 202                  !next raytracing iteration
 1000 continue                  
c
cfh  temps is the travel time ttime we are looking for
      if (iflrai(no,neit).eq.1) then    ! shooting successfull
         ttime= temps
      else if (iflrai(no,neit).eq.2) then 
cfh use results from ART-pb if RKP fails...
c        ttime= -99.9
         ttime = atim
         do i=1,nrpt
          do j=1,3
            rp(j,i,no)=rtmp(j,i,no)
            nrp(no) = nrpt
          enddo
         enddo
cfh --- calculation for synthetic ttimes
         if(iheter1.eq.3) then
            if(isp.eq.0) then
               secp(no,neit)=seco(neit)+pdl(ji)+ttime
            else
               secp(no,neit)=seco(neit)+sdl(ji)+ttime
            endif
         endif
c@ again - don't need residuals here!
c@         if(isp.eq.0) then
c@            res(no)=secp(no,neit)-seco(neit)-pdl(ji)-ttime
c@         else
cfh @@ problem - not solved: secp(no,neit) is an s-p time for s-reading?
c@            res(no)=secp(no,neit)-seco(neit)-sdl(ji)-ttime
c@         endif
cfh -- write information about failed RKP-shooting to file 80, rkpfail
         write(80,8001)neit,xev(1),xev(2),xev(3),stn(ji),
     .    xo(1),xo(2),xo(3),ttime
 8001    format('event',1x,i5,3(2x,f7.3),2x,'stn',1x,a4,1x,3(2x,f7.3),
     .    1x,f7.3)
         if(ides.ge.1) then
           write(61,*)'Shooting fails,using ART-PB for no,ne',
     .        no,neit
         endif
      endif
      if (ides.gt.1) then
         write(61,*)' number of raypoints for observation (no,ne) ',no,neit
         write(61,*)'   ',nrp(no)
         write(61,*)' travel time for this observation: ',ttime
      endif
cfh write station and travel time to file 19, no tdif makes sense
c     - instead write dist to be able to check surfacing distance 
      if (kout3.ne.0) then
        write(19,1919) ne,stn(isto(no,neit)),dlta(no,neit),atim,ttime,
     .       sqrt(dist)
 1919 format(i4,1x,a4,f7.2,2x,2(f8.4,2x),f6.3)
      endif
c
cfh write out new initial angle file for 
cfh all observations in calling program
c
c ****** end of subroutine trvhet
      end
c
cfh additional subroutines needed:
cfh tiraju, TEAS, ray2 (->vitest, latlon, milder, parsur)
c
c
c subroutine tiraju
c*********************************************************************
c ajustement du tir
c*********************************************************************
cfh -- shot adjustment routine from J. Virieux (his sim5)
cfh -- computes adjustments to the, phi
      subroutine tiraju(the,phi,xps,yps,xpi,ypi,qq1x,qq1y,qq2x,qq2y)
      dx=xps-xpi
      dy=yps-ypi
c%%%%%%%%%%%%%%%%%%%%%%%%%%% JEAN
c      dx=-dx
c      dy=-dy
c------------------- calcul des coefficients sur les deux paraxiaux
cfh --- calculate coefficients for the 2 paraxials
c
      det=qq1x*qq2y-qq2x*qq1y
      if(abs(det).gt.1.e-8) then
         phi=(dx*qq2y-dy*qq2x)/det
         the=(qq1x*dy-dx*qq1y)/det
      else
c------------------- les deux rais sont alignes strategie provisoire
c------------------- prendre un seul rai
cfh --- the two rays are almost the same (?? translation ??)
cfh --- take just one of the two
cfh ?? resulting phi,the are always 0. now ??
         if(abs(qq1x).gt.1.e-8) then
            phi=dx/qq1x
         elseif(abs(qq2x).gt.1.e-8) then
            phi=dx/qq2x
         else
            phi=0.
         endif
         if(abs(qq2y).gt.1.e-8) then
            the=dy/qq2y
         elseif(abs(qq1y).gt.1.e-8) then
            the=dy/qq1y
         else
            the=0.
         endif
      endif
      return
      end
c---
c end of subroutine tiraju
c
c
c@@@ here we have to add edlt (the epicentral distance) to the variables 
c    input to ray2 (for the qzfac handling...), and qzfac now is given back...
c    but has to be computed initally here on each call...
c subroutine ray2
cfh added current observation number 'no' for writing raypoint array in here
cfh -- also isp added to function call 
c***********************************************************************
c trace de rais a partir d'une position de tir et vecteur lenteur initial
c xtir,ytir,ztir,pxtir,pytir,pztir : source et vecteur directionnel
c edlt : epicentral distance (f.h)
c qxs,qys,qzs : position finale pour qzs donne en entree
cfh now given also at the beginning on first call
c pxs,pys,pzs : vecteur lenteur final au point de sortie
c qq1x,qq1y,pp1x,pp1y : vecteur canonique en phi sur z=qzs
c qq2x,qq2y,pp2x,pp2y : vecteur canonique en tet sur z=qzs
c itest : parametre de detection d'une caustique itest=0 caustique
c temps : temps de parcours
c ier   : code erreur
c iparax : calcul du paraxial    iparax=0 non iparax=1 oui
c qzfac : upward or downward ray (needed for reverse shooting) (f.h)
c-----------------------------------------------------------------------
c code d'erreurs ier=0   : le calcul s'est bien passe
c                ier=100 : le rai ne rencontre pas z=qzs
c                ier=101 : le rai sort du milieu defini
c                ier=102 : le rai nre remplit pas les conditions requise
c                ier=103 : le rai transmis n'existe pas ( angle critique
c                ier=104 : on se retrouve a la source
c                ier=105 : rai incident rasant a l'interface
c-----------------------------------------------------------------------
c***********************************************************************
c ray2 trace cinematique et dynamique avec derivees
c***********************************************************************
cfh --  kinematic and dynamic raytracing (shooting) in 3D medium with
cfh -- paraxial rays, taken from Jean's sim5
cfhRT change made so that raypath is written to rp(3,n_points,no)
cfhRT simulps12 uses a max. no of 130 (128) points per ray. prob. this
cfhRT should be changed (dependence on ampr -value?). 
cfhRT 'ideriv' does now only mean to put raypath to array rp
      subroutine ray2(no,isp,the,phi,xtir,ytir,ztir,edlt,
     1                pxtir,pytir,pztir,
     2                qxs,qys,qzs,pxs,pys,pzs,temps,
     3                qq1x,qq1y,pp1x,pp1y,qq2x,qq2y,pp2x,pp2y,
     4                itest,ier,iparax,ideriv,qzfac)
      common/cadre/xfmin,xfmax,yfmin,yfmax,zfmin,zfmax
      common/rayde/kmahi,ides,ampr,iterrai,itime
      common/raypos/qxold,qyold,qzold,qx,qy,qz
      common/heter2/iheter,iheter1,epsob,epsca,itsta
cfh      data ntu/8000/
      data ntu/1000/
      integer qzfac
cfhRT needed for rp-array
      include 'simulps_common.inc'
cfhRT end
c
      nt=0
      ier=0
      qq1x=0.
      qq1y=0.
      qq2x=0.
      qq2y=0.
      pp1x=0.
      pp1y=0.
      pp2x=0.
      pp2y=0.
      itest=1
      qxa=xtir
      qya=ytir
      qza=ztir
cfh define qzfac for station below shot
      qzfac=1
      if ((ztir - qzs).lt.-0.00001) qzfac = -1
c------------- calcul du vecteur lenteur a la source xtir,ytir,ztir
cfh --- calculate slowness vector at the source
cfh -- returns error if we get outside the grid
      call vitest(isp,qxa,qya,qza,u2,u2x,u2y,u2z,
     1     u2xx,u2yy,u2zz,u2xy,u2xz,u2yz,0,ier1)
      if(ier1.ne.0) then
         if(ides.gt.1) then
            write(61,*) ' vitesse source erreur  ier: ',ier1
         endif
         ier=102
         return
      endif 
      us=sqrt(u2)
cfh test
c      write(61,*)'slowness at source: ',us    
c------------ systeme canonique initial
cfh --- initial system of canonics
      cphi=cos(phi)
      sphi=sin(phi)
      cthe=cos(the)
      sthe=sin(the)
      pxi=sthe*cphi
      pyi=sthe*sphi
      pzi=cthe
      pxi2=pxi*pxi
      pyi2=pyi*pyi
      pzi2=pzi*pzi
      qx=xtir
      qy=ytir
      qz=ztir
      px=pxi*us
      py=pyi*us
      pz=pzi*us
      pxtir=px
      pytir=py
      pztir=pz
c------------ debut du calcul du rai par perturbation
cfh --- beginning of ray calculation by perturbation
      qxold=qx
      qyold=qy
      qzold=qz
      pxold=px
      pyold=py
      pzold=pz
c----------- harmonique x
      if(iparax.eq.1) then
         qx1=0.
         qy1=0.
         qz1=0.
         px1=us
         py1=0.
         pz1=0.
c----------- harmonique y
         qx2=0.
         qy2=0.
         qz2=0.
         px2=0.
         py2=us
         pz2=0.
c----------- harmonique z
         qx3=0.
         qy3=0.
         qz3=0.
         px3=0.
         py3=0.
         pz3=us
c----------- initialisation
         qx1old=qx1
         qy1old=qy1
         qz1old=qz1
         px1old=px1
         py1old=py1
         pz1old=pz1
         qx2old=qx2
         qy2old=qy2
         qz2old=qz2
         px2old=px2
         py2old=py2
         pz2old=pz2
         qx3old=qx3
         qy3old=qy3
         qz3old=qz3
         px3old=px3
         py3old=py3
         pz3old=pz3
c-------------- initialisation pour indice kmah
         qq1old=0.
         qq2old=0.
         qq3old=0.
         qqqold=0.
         kma=0
      endif
c-------------- initialisation
cfh -- for first ray segment
      nt=1
      dt=ampr
      dtn=dt
      dtn2=0.5*dtn
      istop=0
      fact=1.
      temps=0.
c@@ temps1,2 not needed any more! replace temps2 with temps?
cfh another test
c@      temps1=0.
c@      temps2=0.
      u2old=u2
cfhRT -- fill rp-array only when this is the raytracing for
cfhRT -- the final ray! ideriv=1
cfhRT -- now put the first raypoint to array rp
cfh -- no must be known to ray2
cfh -- if the ray segment length from this raytracing
cfh -- is too small to be handled with the inversion routines
cfh -- one could think of accumulating segment lenght and giving
cfh -- out raypoints only for larger ray segments...
      if (ideriv.ge.1) then
         rp(1,nt,no) = qxold
         rp(2,nt,no) = qyold
         rp(3,nt,no) = qzold
      endif
c
 101  nt=nt+1
      if(nt.gt.ntu) goto 201    ! max number of raypoints reached
      call vitest(isp,qxold,qyold,qzold,u2,u2xa,u2ya,u2za,
     1              u2xx,u2yy,u2zz,u2xy,u2xz,u2yz,1,ier1)
      if(ier1.ne.0) then
         if(ides.gt.1) then
            write(61,*) ' vitesse erreur premiere passe',ier1,' rp: ',nt
         endif
         ier=102
         return
      endif
 111  continue
c--------------- premiere passe
cfh --- first step
      qx=qxold+pxold*dtn2
      qy=qyold+pyold*dtn2
      qz=qzold+pzold*dtn2
      px=pxold-dtn2*(-0.5*u2xa)
      py=pyold-dtn2*(-0.5*u2ya)
      pz=pzold-dtn2*(-0.5*u2za)          
c------------------- deuxieme passe
cfh --- second step
      call vitest(isp,qx,qy,qz,u2,u2x,u2y,u2z,
     1     u2xx,u2yy,u2zz,u2xy,u2xz,u2yz,iparax+1,ier1)
      if(ier1.ne.0) then
         if(ides.gt.1) then
            write(61,*) ' vit. erreur deuxieme passe ',ier1,' rp: ',nt
c            write(61,*) ' x,y,z = :',qx,qy,qz
c            write(61,*)'qzold = ',qzold
         endif
         ier=102
         return
      endif
      qx=qxold+dtn*px
      qy=qyold+dtn*py
      qz=qzold+dtn*pz
      px=pxold-dtn*(-0.5*u2x)
      py=pyold-dtn*(-0.5*u2y)
      pz=pzold-dtn*(-0.5*u2z)
cJean/fh use correct (= current) point for paraxials
      call vitest(isp,qx,qy,qz,u2,u2x,u2y,u2z,
     1     u2xx,u2yy,u2zz,u2xy,u2xz,u2yz,iparax+1,ier1)
c
c-------------------- paraxiaux
cfh --- paraxials
      if(iparax.eq.1) then
c------------------- x
         qx1=qx1old+dtn*px1old
         qy1=qy1old+dtn*py1old
         qz1=qz1old+dtn*pz1old
         px1=px1old+dtn*0.5*(u2xx*qx1old+u2xy*qy1old+u2xz*qz1old)
         py1=py1old+dtn*0.5*(u2xy*qx1old+u2yy*qy1old+u2yz*qz1old)
         pz1=pz1old+dtn*0.5*(u2xz*qx1old+u2yz*qy1old+u2zz*qz1old)
c------------------- y
         qx2=qx2old+dtn*px2old
         qy2=qy2old+dtn*py2old
         qz2=qz2old+dtn*pz2old
         px2=px2old+dtn*0.5*(u2xx*qx2old+u2xy*qy2old+u2xz*qz2old)
         py2=py2old+dtn*0.5*(u2xy*qx2old+u2yy*qy2old+u2yz*qz2old)
         pz2=pz2old+dtn*0.5*(u2xz*qx2old+u2yz*qy2old+u2zz*qz2old)
c------------------- z
         qx3=qx3old+dtn*px3old
         qy3=qy3old+dtn*py3old
         qz3=qz3old+dtn*pz3old
         px3=px3old+dtn*0.5*(u2xx*qx3old+u2xy*qy3old+u2xz*qz3old)
         py3=py3old+dtn*0.5*(u2xy*qx3old+u2yy*qy3old+u2yz*qz3old)
         pz3=pz3old+dtn*0.5*(u2xz*qx3old+u2yz*qy3old+u2zz*qz3old)
      endif
c------------------- test avec la surface z=qzs
cfh --- surface test.
cfh -- if ray jumps into the air with this step, reduce dtn,dtn2; if they are
cfh --  already smaller than 0.1 *ampr then finish.
cfh -- (raypoints near the surface are really close therefore ...)
cfh -- reverse shooting goes garbage if station is below shot. maybe
cfh   -- the reason is here? use qzfac defined at the beginning of the
cfh   -- subroutine (qzfac = -1 if station below source, 1 else)
c
cfh if qzfac .lt.0 we came from above.
cfh      if(qz.le.qzs) then
c      if((qzfac*(qz-qzs)).le.0) then
      if(qz.le.qzs.and.qzfac.eq.1) then   !ray comes from below
         dtn=0.5*dtn
         dtn2=dtn2*0.5
         if(dtn.lt.0.1*dt) goto 112
         goto 111
      else if (qz.gt.qzs.and.qzfac.eq.-1) then ! ray comes from above
           dist = sqrt((xtir-qx)**2+(ytir-qy)**2) ! this is the distance from the source 
           if ((edlt-dist).lt.0.5) then ! delta is the epicentral distance, this means no diving wave
              dtn=0.5*dtn                ! so treat this point as ray surfacing point...
              dtn2=dtn2*0.5
              if(dtn.lt.0.1*dt) goto 112
              goto 111
           else if((edlt-dist).ge.0.5) then ! possible diving wave, continue raytracing but
              qzfac = 1                     ! now the ray will come from below- change qzfac
           endif
      endif
      goto 113
 112  istop=1
 113  continue
c////////////////////////////////////////////////
c-------------- calcul du temps de parcours par trapeze
c////////////////////////////////////////////////
      temps = temps +0.5*(u2old+u2)*dtn         ! original
      u2old = u2
c
c////////////////////////////////////////////////
c-------------- derivee par rapport aux noeuds du milieu
c////////////////////////////////////////////////
c-------------- reinitialisation
      qxold=qx
      qyold=qy
      qzold=qz
      pxold=px
      pyold=py
      pzold=pz
      qxa=qx
      qya=qy
      qza=qz
      if(iparax.eq.1) then
         qx1old=qx1
         qy1old=qy1
         qz1old=qz1
         px1old=px1
         py1old=py1
         pz1old=pz1
         qx2old=qx2
         qy2old=qy2
         qz2old=qz2
         px2old=px2
         py2old=py2
         pz2old=pz2
         qx3old=qx3
         qy3old=qy3
         qz3old=qz3
         px3old=px3
         py3old=py3
         pz3old=pz3
c--------------- indice kmah
c-------------- qq1,qq2,qq3 sous-determinant suivant pxi,pyi,pzi
cfh --- kmah is used to look for caustics
         qq1=px*(qy2*qz3-qz2*qy3)-py*(qx2*qz3-qz2*qx3)+
     1        pz*(qx2*qy3-qy2*qx3)
         qq2=-(px*(qy1*qz3-qz1*qy3)-py*(qx1*qz3-qz1*qx3)+
     1        pz*(qx1*qy3-qy1*qx3))
         qq3=px*(qy1*qz2-qz1*qy2)-py*(qx1*qz2-qz1*qx2)+
     1        pz*(qx1*qy2-qy1*qx2)
         qqq=    px*(pxi*(qy2*qz3-qz2*qy3)-pyi*(qy1*qz3-qz1*qy3)+
     1        pzi*(qy1*qz2-qz1*qy2))
     2        -py*(pxi*(qx2*qz3-qz2*qx3)-pyi*(qx1*qz3-qz1*qx3)+
     3        pzi*(qx1*qz2-qz1*qx2))
     4        +pz*(pxi*(qx2*qy3-qy2*qx3)-pyi*(qx1*qy3-qy1*qx3)+
     5        pzi*(qx1*qy2-qy1*qx2))
         qq1=qq1*fact
         qq2=qq2*fact
         qq3=qq3*fact
         qqq=qqq*fact
         kmao=kma
         if(nt.eq.1) goto 150
         pro0=qqq*qqqold
         if(pro0.lt.0.) kma=kma+1
         if(kma.ne.kmao) goto 150
         pro1=qq1*qq1old
         pro2=qq2*qq2old
         pro3=qq3*qq3old
c-------------- attention si chang de signe des sous-determinants point
c-------------- il n'y a pas de chang de signe de qqq pour un point foca
cfh --- sic.
         if(pro1.lt.0..and.pro2.lt.0..and.pro3.lt.0.) kma=kma+2
 150     qq1old=qq1
         qq2old=qq2
         qq3old=qq3
         qqqold=qqq
c     write(61,*) ' qq1,qq2,qq3,qqq,kmah '
c     write(61,*) qq1,qq2,qq3,qqq,kma
c     if(kmao.ne.kma) then
c       write(61,*) ' tet,phi ',the*57.29,phi*57.29
c     write(61,*) ' kmao,kma ',kmao,kma,nt
c       write(61,*) ' qqq ',qqq,qq1,qq2,qq3
c     endif
         kmao=kma
      endif
c--------------- test sur le bord superieur
cfh --- if we reached the station elevation
      if(istop.eq.1) goto 200
c--------------- test sur le cadre du milieu
cfh --- test if we are still within the grid
      iflag=0
      if(qx.lt.xfmin) iflag=1
      if(qx.gt.xfmax) iflag=1
      if(qy.lt.yfmin) iflag=1
      if(qy.gt.yfmax) iflag=1
      if(qz.lt.zfmin) iflag=1
      if(qz.gt.zfmax) iflag=1
      if(iflag.ne.0) then
         ier=101
         if(ides.gt.1) then
            write(61,*) ' rai sort du cadre xfmin,xfmax,zfmin,zfmax '
            write(61,*) xfmin,xfmax,yfmin,yfmax,zfmin,zfmax
            write(61,*) ' x,y,z ',qx,qy,qz
         endif
         return
      endif
c------------------------ write raypoint to rp-array
      if(ideriv.ge.1) then
         rp(1,nt,no) = qxold
         rp(2,nt,no) = qyold
         rp(3,nt,no) = qzold
      endif
c------------------------ verbose output
c      if(ideriv.ge.1.and.ides.gt.2) then
c         write(61,*) 'nt,x,y,z,px,py,pz',
c     1        nt,qxold,qyold,qzold,pxold,pyold,pzold
c      endif
      goto 101                  ! next ray segment
c--------------- epuisement de nt > ntu
cfh --- max. number of ray segments used without hitting surface
 201  ier=100
      if(ides.gt.1) then
         write(61,*) ' rai ne rencontre pas le plan z=qzs '
         write(61,*) qz,qzs,nt
      endif
      return
c--------------- on sort quand on a rencontre le plan z=qzs
cfh --- leave subroutine if surface is hit
 200  continue
      qxs=qx
      qys=qy
      pxs=px
      pys=py
      pzs=pz
      if(iparax.eq.1) then
         if(kmahi.ne.kma) then
            itest=0
            kmahi=kma
         endif
c---------------- valeur qq1 suivant phi et qq2 suivant the
c---------------- pour les rayons paraxiaux
cfh --- qq1 follows phi and qq2 follows the for the paraxial beams
         q1x=(-sphi*qx1+cphi*qx2)*sthe
         q1y=(-sphi*qy1+cphi*qy2)*sthe
         q1z=(-sphi*qz1+cphi*qz2)*sthe
         p1x=(-sphi*px1+cphi*px2)*sthe
         p1y=(-sphi*py1+cphi*py2)*sthe
         p1z=(-sphi*pz1+cphi*pz2)*sthe
c---------------- prolongement sur surface libre
cfh -- if we hit the surface we have to adjust the paraxials
cfh -- following the continuation conditions
         iert=0
cfh here pp1., qq1. are adjusted
         call parsur(q1x,q1y,q1z,p1x,p1y,p1z,
     1        qq1x,qq1y,qq1z,pp1x,pp1y,pp1z,
     2        qxs,qys,qzs,px,py,pz,isp,iert)
         if(iert.ne.0) then
            if(ides.gt.1) then
               write(61,*) ' ray3 prolong surface libre erreur '
            endif
            ier=102
            return
         endif
         q2x=cthe*(cphi*qx1+sphi*qx2)-sthe*qx3
         q2y=cthe*(cphi*qy1+sphi*qy2)-sthe*qy3
         q2z=cthe*(cphi*qz1+sphi*qz2)-sthe*qz3
         p2x=cthe*(cphi*px1+sphi*px2)-sthe*px3
         p2y=cthe*(cphi*py1+sphi*py2)-sthe*py3
         p2z=cthe*(cphi*pz1+sphi*pz2)-sthe*pz3
c---------------- prolongement sur surface libre
         iert=0
cfh here pp2., qq2. are adjusted
         call parsur(q2x,q2y,q2z,p2x,p2y,p2z,
     1        qq2x,qq2y,qq2z,pp2x,pp2y,pp2z,
     2        qxs,qys,qzs,px,py,pz,isp,iert)
         if(iert.ne.0) then
            if(ides.gt.1) then
               write(61,*) ' ray3 prolong surface libre erreur '
            endif
            ier=102
            return
         endif
      endif
c
c
c--------------- un petit test sur la source
cfh ---  test if surface and event coordinates are equal
      rho=(qxs-xtir)**2+(qys-ytir)**2+(qzs-ztir)**2
      if(rho.lt.1.e-5) then
         ier=104
         if(ides.gt.1) then
            write(61,*) ' obser=source ',qxs,qys,qzs,xtir,ytir,ztir
         endif
c--------------- on a decolle de la source pourtant
cfh --- anyway we started from the source (??)
cfh -- if traveltime is greater than .1 sec it is ok.
cfh -- again I don't understand...
         if(temps.gt..1) ier=0
         return
      endif
cfh -- put number of raypoints to nrp(no) if this is the final raypath
cfh --   and don't forget final raypoint...
      if (ideriv.ge.1) then
         rp(1,nt,no) = qxold
         rp(2,nt,no) = qyold
         rp(3,nt,no) = qzold
         nrp(no)=nt
      endif
c
cfh @@ test
c      write(6,*)'qzfac = ',qzfac
cfh test end
      return
      end
c
c end of subroutine ray2
c
c
c subroutine parsur
c*********************************************************************
c conditions de continuite des paraxiaux a un interface horizontal   *
c code erreur  ier=105 rai rasant a l'interface                      *
c              ne doit pas arriver puisque solution rasante refusee  *
c*********************************************************************
cfh -- this routine applies continuation conditions to paraxial rays
cfh -- hitting an interface (slowness discontinuity). 
cfh -- here the only interface allowed is the surface.
cfh -- error if ray is parallel to the surface. (= horizontal)
cfh -- taken from Jean's sim5
cfh also add isp to function call
      subroutine parsur(xp1e,yp1e,zp1e,px1e,py1e,pz1e,
     1     xp1s,yp1s,zp1s,px1s,py1s,pz1s,
     2     xe,ye,ze,pxe,pye,pze,isp,ier)
      data epsras/1.e-8/
      ier=0
c-------------------- recherche du rai rasant i.e. horizontal
cfh --- check if the ray is horizontal
      if(abs(pze).lt.epsras) then
         ier=105
         return
      endif
      deni=1./pze
c-------------------- prolongement sur la surface horizontale
cfh --- continuation along horizontal surface (??)
      dtau=-zp1e*deni
      xp1s=xp1e+pxe*dtau
      yp1s=yp1e+pye*dtau
      zp1s=0.
      call vitest(isp,xe,ye,ze,u2,u2x,u2y,u2z,
     1     u2xx,u2yy,u2zz,u2xy,u2xz,u2yz,1,ier1)
c-------------------- prolongement sur la surface horizontale
      px1s=px1e+0.5*u2x*dtau
      py1s=py1e+0.5*u2y*dtau
      pz1s=pz1e+0.5*u2z*dtau
      return
      end
c end of subroutine parsur
c
c
c subroutine vitest
c*********************************************************************
c calcul de la lenteur au carre et derivees dans le milieu           *   
c                                                                    *
c iopt=0 calcul de u2                                                *
c iopt=1 calcul de u2 u2x u2y,u2z                                    *
c iopt=2 calcul de u2xx,u2yy,u2zz,u2xy,u2xz,u2yz                     *   
c                                                                    *
c code d'erreurs   ier=108  x trop petit                             *
C                  ier=109  x trop grand                             *
c                  ier=110  y trop petit                             *
c                  ier=111  y trop grand                             *
c                  ier=112  z trop petit                             *
c                  ier=113  z trop grand                             *
c variable imodps determine la lenteur au carre des ondes P ou S     *   
c*********************************************************************
cfh calculation of slowness and slowness derivatives on the grid
cfh taken from Jean's sim5
cfh 1906 change here now for velocities: the values in the uu-array
cfh      are velocities, therefore we get the interpolated velocities
cfh      and its derivatives from the bsp -routines and compute
cfh      squared slowness resp its derivatives here.
cfh      the return values are still squared slowness and derivatives!!
cfh 0310 -- need isp as function parameter, it is NOT in common!
      subroutine vitest(isp,x,y,z,u2,u2x,u2y,u2z,
     1     u2xx,u2yy,u2zz,u2xy,u2xz,u2yz,iopt,ier)
      include 'rayparax_common.inc'
      dimension pp(4,4,4)
      ier=0 
      lnx2=nxu-2
      lny2=nyu-2
      lnz2=nzu-2
c---------- localisation des points
cfh --- find the points on the grid
c      if(ier.eq.-1) then
c         ier=108
c         return
c      endif 
c      if(ier.eq.1) then
c         ier=109
c         return
c      endif
cfh try using intmaprt here
c - error handling: nto .eq.-1 if o.lt.onu(1)
c                   nto .eq. 0 if o.gt. onu(nou)
      call intmaprt(x,y,z,ntx,nty,ntz)
      if(ntx.eq.-1) then
         ier=108
         return
      endif 
      if(ntx.eq.0) then
         ier=109
         return
      endif
      xu=xnu(ntx)
      dxu=xnu(ntx+1)-xu
c      if(ier.eq.-1) then
      if(nty.eq.-1) then
         ier=110
         return
      endif
c      if(ier.eq.0) then
      if(nty.eq.0) then
         ier=111
         return
      endif
      yv=ynu(nty)
      dyv=ynu(nty+1)-yv
c      if(ier.eq.-1) then
      if(ntz.eq.-1) then
         ier=112
         return
      endif
c      if(ier.eq.1) then
      if(ntz.eq.0) then
         ier=113
         return
      endif
      zw=znu(ntz)
      dzw=znu(ntz+1)-zw
c----------- calcul de la lenteur au carre
cfh --- slowness value for these coords
cfh  first get relative distances
      ux=(x-xu)/dxu
      vy=(y-yv)/dyv
      wz=(z-zw)/dzw
cfh 
c 
c//////////////////////////////////////////////////////////
c remplissage de la matrice lenteur au carre isp=0 P isp=1 S
c//////////////////////////////////////////////////////////
cfh --- filling of the slowness matrix for interpolation
cfh               now  velocity 
cfh --- (needs 2 neighbors in every direction)
      koff=0
      if(isp.eq.1) koff=nzu
      do 400 k=1,4
      do 400 j=1,4
      do 400 i=1,4
      pp(i,j,k)=uu(ntx-2+i,nty-2+j,ntz-2+k+koff)
  400 continue
cfh -- pure slowness calc.
c      u2=bsp3(ux,vy,wz,pp)
cfh         velocity
      v1=bsp3(ux,vy,wz,pp)
      v2 = v1*v1
      u2 = 1./(v2)
      if(iopt.eq.0) then
         ier=0
         return
      endif
c---------- calcul des derivees premieres
cfh -- first derivs
      u3 = u2/v1
      u3x2 = u3*2.
      v2x=bsp3s1(ux,vy,wz)/dxu
      v2y=bsp3t1(ux,vy,wz)/dyv
      v2z=bsp3v1(ux,vy,wz,pp)/dzw
      u2x= -u3x2*v2x
      u2y= -u3x2*v2y
      u2z= -u3x2*v2z
c      write(61,*) ' u2,x,y,z = ',u2,u2x,u2y,u2z
      if(iopt.eq.1) then
         ier=0
         return
      endif
c---------- calcul des derivees secondes
cfh -- second derivs
      u1x3 = (1./v1) * 3.
      v2xx= ( bsp3s2(ux,vy,wz)/dxu ) /dxu
      v2yy= ( bsp3t2(ux,vy,wz)/dyv ) /dyv
      v2zz= ( bsp3v2(ux,vy,wz,pp)/dzw ) /dzw
      v2xy= ( bsp3st(ux,vy,wz)/dxu ) /dyv
      v2xz= ( bsp3sv(ux,vy,wz)/dxu ) /dzw
      v2yz= ( bsp3tv(ux,vy,wz)/dyv ) /dzw
      u2xx= u3x2* ((v2x*v2x)*u1x3 - v2xx)
      u2yy= u3x2* ((v2y*v2y)*u1x3 - v2yy)
      u2zz= u3x2* ((v2z*v2z)*u1x3 - v2zz)
      u2xy= u3x2* ((v2x*v2y)*u1x3 - v2xy)
      u2xz= u3x2* ((v2x*v2z)*u1x3 - v2xz)
      u2yz= u3x2* ((v2y*v2z)*u1x3 - v2yz)
      ier=0
      return
      end
c end of subroutine vitest
c
c
c** spline functions now. 
c** hardwired to work on equidistant grid in x,y,z
c
cfh this is an optimized version of Jean's original spline-calculation
c    routines.
c    uses common blocks for multiple used variables 
c  !! THE ORDER IN WHICH THE FUNCTIONS ARE CALLED IS IMPORTANT !!
c the functions are in the order in which they are called from vitest:
c  (ordered by first occurrence)
c 1) iopt=0
c    bsp3 -> bsp2 -> bsp1
c 2) iopt=1
c    bsp3s1 -> bsp2s1 -> bsp1x1
c    bsp3t1 -> bsp2t1 -> bsp1
c    bsp3v1 -> bsp2 -> bsp1
c 3) iopt=2
c    bsp3s2 -> bsp2s2 -> bsp1x2
c    bsp3t2 -> bsp2t2 -> bsp1
c    bsp3v2 -> bsp2 -> bsp1
c    bsp3st -> bsp2st -> bsp1x1
c    bsp3sv -> bsp2s1 -> bsp1x1
c    bsp3tv -> bsp2t1 -> bsp1
c 
c****************
c subroutine to calculate cardinal cubic b-splines in 3D:
c bsp3(s,t,v) with s,t and v between 0 and 1
c***********
c
c  bsp3 is always called at the beginning - in here calculate
c    v**2 (vv2), v**3 (vv3) and pv1 to be used in other bsp3. functions
c    pv2 is only computed later
c
      function bsp3(s,t,v,pp)
      dimension pp(4,4,4),pv1(4,4),pv2(4,4)
      real vv2, vv3
      common/splines1/ vv2,vv3,pv1,pv2
      vv2 = v*v
      vv3 = vv2*v
c
c      q1=0.166666666667*(1.+v*(-3.+v*( 3.-    v)))
      q1=0.166666666667*(1. - 3.*v + 3.*vv2 - vv3)
c      q2=0.166666666667*(4.+v*     v*(-6.+3.*v))
      q2=0.166666666667*(4. - 6.*vv2 + 3.*vv3)
c      q3=0.166666666667*(1.+v*( 3.+v*( 3.-3.*v)))
      q3=0.166666666667*(1.+ 3.*v + 3.*vv2 - 3.*vv3)
c      q4=0.166666666667*v*v*v
      q4=0.166666666667*vv3
      do 10 j=1,4
      do 10 i=1,4
      pv1(i,j)=pp(i,j,1)*q1+pp(i,j,2)*q2+pp(i,j,3)*q3+pp(i,j,4)*q4
   10 continue
      bsp3=bsp2(s,t,pv1)
      return
      end
c
c*************
c subroutine to calculate cardinal cubic b-splines in 2D:
c bsp2(s,t) with s,t between 0 and 1
c************
c  here calculate qt11, qt12, qtt3, qt14 to be used in other bsp2. routines
c  and also t**2 (tt2) and t**3 (tt3)
c
      function bsp2(s,t,pv)
      dimension pv(4,4)
      real tt2, tt3, qt11,qt12,qt13,qt14
      common/splines2/ tt2,tt3,qt11,qt12,qt13,qt14
c
      tt2 = t*t
      tt3 = tt2*t
c      q1=0.166666666667*(1.+t*(-3.+t*( 3.-    t)))
      qt11=0.166666666667*(1. - 3.*t + 3.*tt2 -  tt3)
c      q2=0.166666666667*(4.+t*     t*(-6.+3.*t))
      qt12=0.166666666667*(4.    -   6.*tt2 + 3.*tt3)
c      q3=0.166666666667*(1.+t*( 3.+t*( 3.-3.*t)))
      qt13=0.166666666667*(1.+ 3.*t + 3.*tt2 - 3.*tt3)
c      q4=0.166666666667*t*t*t
      qt14=0.166666666667*tt3
      p1=pv(1,1)*qt11+pv(1,2)*qt12+pv(1,3)*qt13+pv(1,4)*qt14
      p2=pv(2,1)*qt11+pv(2,2)*qt12+pv(2,3)*qt13+pv(2,4)*qt14
      p3=pv(3,1)*qt11+pv(3,2)*qt12+pv(3,3)*qt13+pv(3,4)*qt14
      p4=pv(4,1)*qt11+pv(4,2)*qt12+pv(4,3)*qt13+pv(4,4)*qt14
      bsp2=bsp1(s,p1,p2,p3,p4)
      return
      end
c
c************
c subroutine to calculate cardinal cubic b-splines in 1D:
c bsp1(s)
c***********
      function bsp1(s,p1,p2,p3,p4)
c
      bsp1=0.166666666667*(    p1+4.*p2+   p3+
     1                   s*(-3.*p1      +3.*p3+
     2                   s*( 3.*p1-6.*p2+3.*p3+
     3                   s*(   -p1+3.*p2-3.*p3+p4))))
      return
      end
c
c**********
c subroutine to calculate first derivatives in s with b-splines in 3D:
c bsp3s1(s,t,v)
c********
c actually, as the pv1 is the same as in bsp3, to create single functions
c is maybe not very intelligent...
      function bsp3s1(s,t,v)
      dimension pv1(4,4),pv2(4,4)
      common/splines1/ vv2,vv3,pv1,pv2
c
c      bsp3s1=bsp2s1(s,t,pv1)
      bsp3s1=bsp2s1(s,pv1)
      return
      end
c
c*************
c subroutine to calculate first derivatives in s with b-splines in 2D:
c bsp2s1(s,pv1)
c**********
      function bsp2s1(s,p)
      dimension p(4,4)
      common/splines2/ tt2,tt3,qt11,qt12,qt13,qt14
c
      p1=p(1,1)*qt11+p(1,2)*qt12+p(1,3)*qt13+p(1,4)*qt14
      p2=p(2,1)*qt11+p(2,2)*qt12+p(2,3)*qt13+p(2,4)*qt14
      p3=p(3,1)*qt11+p(3,2)*qt12+p(3,3)*qt13+p(3,4)*qt14
      p4=p(4,1)*qt11+p(4,2)*qt12+p(4,3)*qt13+p(4,4)*qt14      
      bsp2s1=bsp1x1(s,p1,p2,p3,p4)
      return
      end
c
c********
c subroutine bsp1x1 to calculate first derivatives with b-splines in 1D
c *******
      function bsp1x1(s,p1,p2,p3,p4)
c
      bsp1x1=0.5*(  -p1      +   p3+
     1                    s*(2.*p1-4.*p2+2.*p3+
     2                    s*(  -p1+3.*p2-3.*p3+p4)))
      return
      end
c
c*********
c subroutine to calculate the first derivative in t with b-splines in 3D:
c bsp3t1(s,t,v)
c*********
      function bsp3t1(s,t,v)
      dimension pv1(4,4), pv2(4,4)
      common/splines1/ vv2,vv3,pv1,pv2
c
      bsp3t1=bsp2t1(s,t,pv1)
      return
      end
c
c***********
c subroutine to calculate first derivatives in t with b-splines in 2D:
c bsp2t1(s,t)
c**********
c  here comes a new qt: qt21,qt22,qt23,qt24
      function bsp2t1(s,t,pv)
      dimension pv(4,4)
      common/splines2/ tt2,tt3,qt11,qt12,qt13,qt14
      real qt21,qt22,qt23,qt24
      common/splines3/ qt21,qt22,qt23,qt24
c
c     q1=0.5*(-1.+t*( 2.-1.*t))
      qt21=0.5*(-1. + 2.*t - 1.*tt2)
c     q2=0.5*(   +t*(-4.+3.*t))
      qt22=0.5*(    -4.*t + 3.*tt2)
c     q3=0.5*( 1.+t*( 2.-3.*t))
      qt23=0.5*( 1. + 2.*t - 3.*tt2)
c     q4=0.5*     t*        t
      qt24=0.5*     tt2       
      p1=pv(1,1)*qt21+pv(1,2)*qt22+pv(1,3)*qt23+pv(1,4)*qt24
      p2=pv(2,1)*qt21+pv(2,2)*qt22+pv(2,3)*qt23+pv(2,4)*qt24
      p3=pv(3,1)*qt21+pv(3,2)*qt22+pv(3,3)*qt23+pv(3,4)*qt24
      p4=pv(4,1)*qt21+pv(4,2)*qt22+pv(4,3)*qt23+pv(4,4)*qt24
      bsp2t1=bsp1(s,p1,p2,p3,p4)
      return
      end
cc    a variant bsp2t1a exists, to be used when bsp2t1 has already been called
cc     ( qt2. are known)
c
c********
c subroutine to calculate first derivatives in v with b-splines in 3D:
c bsp3v1(s,t,v)
c*********
c here comes a new pv: pv2    
      function bsp3v1(s,t,v,pp)
      dimension pp(4,4,4),pv1(4,4),pv2(4,4)
      common/splines1/ vv2,vv3,pv1,pv2
c
c     q1=0.5*(-1.+v*( 2.-1.*v))
      q1=0.5*(-1.+ 2.*v - 1.*vv2)
c      q2=0.5*(   +v*(-4.+3.*v))
      q2=0.5*(   -4.*v + 3.*vv2)
c     q3=0.5*( 1.+v*( 2.-3.*v))
      q3=0.5*( 1.+ 2.*v - 3.*vv2)
c      q4=0.5*     v*        v
      q4=0.5*     vv2
      do 10 j=1,4
      do 10 i=1,4
      pv2(i,j)=pp(i,j,1)*q1+pp(i,j,2)*q2+pp(i,j,3)*q3+pp(i,j,4)*q4
 10   continue
      bsp3v1=bsp2a(s,pv2)
      return
      end

c
c******
c  bsp2a is a derivative of bsp2 - qt1. are already computed for this one
c*****
      function bsp2a(s,pv)
      dimension pv(4,4)
      common/splines2/ tt2,tt3,qt11,qt12,qt13,qt14
c
      p1=pv(1,1)*qt11+pv(1,2)*qt12+pv(1,3)*qt13+pv(1,4)*qt14
      p2=pv(2,1)*qt11+pv(2,2)*qt12+pv(2,3)*qt13+pv(2,4)*qt14
      p3=pv(3,1)*qt11+pv(3,2)*qt12+pv(3,3)*qt13+pv(3,4)*qt14
      p4=pv(4,1)*qt11+pv(4,2)*qt12+pv(4,3)*qt13+pv(4,4)*qt14
      bsp2a=bsp1(s,p1,p2,p3,p4)
      return
      end
c
c*********
c subroutine to calculate the second derivatives in s with b-splines in 3D:
c bsp3s2(s,t,v)
c**********
      function bsp3s2(s,t,v)
      dimension pv1(4,4),pv2(4,4)
      common/splines1/ vv2,vv3,pv1,pv2
c
      bsp3s2=bsp2s2(s,pv1)
      return
      end
c
c************
c subroutine to calculate second derivatives in s with b-splines in 2D:
c bsp2s2(s,t)
c************
      function bsp2s2(s,pv)
      dimension pv(4,4)
      common/splines2/ tt2,tt3,qt11,qt12,qt13,qt14
c
      p1=pv(1,1)*qt11+pv(1,2)*qt12+pv(1,3)*qt13+pv(1,4)*qt14
      p2=pv(2,1)*qt11+pv(2,2)*qt12+pv(2,3)*qt13+pv(2,4)*qt14
      p3=pv(3,1)*qt11+pv(3,2)*qt12+pv(3,3)*qt13+pv(3,4)*qt14
      p4=pv(4,1)*qt11+pv(4,2)*qt12+pv(4,3)*qt13+pv(4,4)*qt14
      bsp2s2=bsp1x2(s,p1,p2,p3,p4)
      return
      end
c
c********
c subroutine to calculate the second derivative with b-splines in 1D
c bsp1x2(s)
c******
      function bsp1x2(s,p1,p2,p3,p4)
c
      bsp1x2=    p1-2.*p2+   p3+
     1       s*(-p1+3.*p2-3.*p3+p4)
      return
      end
c
c*********
c subroutine to calculate the second derivative in t with b-splines in 3D:
c bsp3t2(s,t,v)
c*******
      function bsp3t2(s,t,v)
      dimension pv1(4,4),pv2(4,4)
      common/splines1/ vv2,vv3,pv1,pv2
c
      bsp3t2=bsp2t2(s,t,pv1)
      return
      end
c
c**********
c subroutine to calculate the second derivative in t with b-splines in 2D:
c bsp2t2(s,t)
c*********
      function bsp2t2(s,t,pv)
      dimension pv(4,4)
c
      q1= 1.-   t
      q2=-2.+3.*t
      q3= 1.-3.*t
      q4=       t
      p1=pv(1,1)*q1+pv(1,2)*q2+pv(1,3)*q3+pv(1,4)*q4
      p2=pv(2,1)*q1+pv(2,2)*q2+pv(2,3)*q3+pv(2,4)*q4
      p3=pv(3,1)*q1+pv(3,2)*q2+pv(3,3)*q3+pv(3,4)*q4
      p4=pv(4,1)*q1+pv(4,2)*q2+pv(4,3)*q3+pv(4,4)*q4
      bsp2t2=bsp1(s,p1,p2,p3,p4)
      return
      end      
c
c********
c subroutine to calculate second derivatives in v with b-splines in 3D:
c bsp3v2(s,t,v)
c********
      function bsp3v2(s,t,v,pp)
      dimension pp(4,4,4),p(4,4)
c
      q1= 1.-   v
      q2=-2.+3.*v
      q3= 1.-3.*v
      q4=       v
      do 10 j=1,4
      do 10 i=1,4
      p(i,j)=pp(i,j,1)*q1+pp(i,j,2)*q2+pp(i,j,3)*q3+pp(i,j,4)*q4
 10   continue
      bsp3v2=bsp2a(s,p)
      return
      end
c
c********
c subroutine to calculate secnod derivatives in s,t with b-splines in 3D:
c bsp3st(s,t,v)
c********
      function bsp3st(s,t,v)
      dimension pv1(4,4),pv2(4,4)
      common/splines1/ vv2,vv3,pv1,pv2
c
      bsp3st=bsp2st(s,pv1)
      return
      end
c
c******
c subroutine to calculate second derivatives in s,t with b-splines in 2D:
c bsp2st(s,t)
c******
      function bsp2st(s,pv)
      dimension pv(4,4)
      common/splines3/ qt21,qt22,qt23,qt24
c
      p1=pv(1,1)*qt21+pv(1,2)*qt22+pv(1,3)*qt23+pv(1,4)*qt24
      p2=pv(2,1)*qt21+pv(2,2)*qt22+pv(2,3)*qt23+pv(2,4)*qt24
      p3=pv(3,1)*qt21+pv(3,2)*qt22+pv(3,3)*qt23+pv(3,4)*qt24
      p4=pv(4,1)*qt21+pv(4,2)*qt22+pv(4,3)*qt23+pv(4,4)*qt24
      bsp2st=bsp1x1(s,p1,p2,p3,p4)
      return
      end
c
c*******
c subroutine to calculate second derivatives in s,v with b-splines in 3D:
c bsp3sv(s,t,v)
c*******
      function bsp3sv(s,t,v)
      dimension pv1(4,4),pv2(4,4)
      common/splines1/ vv2,vv3,pv1,pv2
c
      bsp3sv=bsp2s1(s,pv2)
      return
      end
c
c*********
c subroutine to calculate second derivatives in t,v with b-splines in 3D:
c bsp3tv(s,t,v)
c********
      function bsp3tv(s,t,v)
      dimension pv1(4,4),pv2(4,4)
      common/splines1/ vv2,vv3,pv1,pv2
c
      bsp3tv=bsp2t1a(s,pv2)
      return
      end
c
c*******      
c bsp2t1a to be used after bsp2t1 has been called
c******
      function bsp2t1a(s,pv)
      dimension pv(4,4)
      common/splines3/ qt21,qt22,qt23,qt24
c
      p1=pv(1,1)*qt21+pv(1,2)*qt22+pv(1,3)*qt23+pv(1,4)*qt24
      p2=pv(2,1)*qt21+pv(2,2)*qt22+pv(2,3)*qt23+pv(2,4)*qt24
      p3=pv(3,1)*qt21+pv(3,2)*qt22+pv(3,3)*qt23+pv(3,4)*qt24
      p4=pv(4,1)*qt21+pv(4,2)*qt22+pv(4,3)*qt23+pv(4,4)*qt24
      bsp2t1a=bsp1(s,p1,p2,p3,p4)
      return
      end
c
c
c** end of spline functions
c
c
c subroutine teas
C        SUBROUTINE TEAS
C
C        PURPOSE
C           CALCULATE THE LIMIT OF A GIVEN SEQUENCE BY MEANS OF THE
C           EPSILON-ALGORITHM.
C
C        USAGE
C           CALL TEAS(X,N,FIN,EPS,IER)
C
C        DESCRIPTION OF PARAMETERS
C           X      - VECTOR WHOSE COMPONENTS ARE TERMS OF THE GIVEN
C                    SEQUENCE. ON RETURN THE COMPONENTS OF VECTOR X
C                    ARE DESTROYED.
C           N      - DIMENSION OF INPUT VECTOR X.
C           FIN    - RESULTANT SCALAR CONTAINING ON RETURN THE LIMIT
C                    OF THE GIVEN SEQUENCE.
C           EPS    - AN INPUT VALUE, WHICH SPECIFIES THE UPPER BOUND
C                    OF THE RELATIVE (ABSOLUTE) ERROR IF THE COMPONENTS
C                    OF X ARE ABSOLUTELY GREATER (LESS) THAN ONE.
C                    CALCULATION IS TERMINATED AS SOON AS THREE TIMES IN
C                    SUCCESSION THE RELATIVE (ABSOLUTE) DIFFERENCE
C                    BETWEEN NEIGHBOURING TERMS IS NOT GREATER THAN EPS.
C           IER    - RESULTANT ERROR PARAMETER CODED IN THE FOLLOWING
C                    FORM
C                     IER=0  - NO ERROR
C                     IER=1  - REQUIRED ACCURACY NOT REACHED WITH
C                              MAXIMAL NUMBER OF ITERATIONS
C                     IER=-1 - INTEGER N IS LESS THAN TEN.
C
C        REMARKS
C           NO ACTION BESIDES ERROR MESSAGE IN CASE N LESS THAN TEN.
C           THE CHARACTER OF THE GIVEN INFINITE SEQUENCE MUST BE
C           RECOGNIZABLE BY THOSE N COMPONENTS OF THE INPUT VECTOR X.
C
C        SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED
C           NONE
C
C        METHOD
C           THE CONVERGENCE OF THE GIVEN SEQUENCE IS ACCELERATED BY
C           MEANS OF THE E(2)-TRANSFORMATION, USED IN AN ITERATIVE WAY.
C           FOR REFERENCE, SEE
C           ALGORITHM 215,SHANKS, CACM 1963, NO. 11, PP. 662. AND
C           P. WYNN, SINGULAR RULES FOR CERTAIN NON-LINEAR ALGORITHMS
C           BIT VOL. 3, 1963, PP. 175-195.
C
C     .................................................................
C
      SUBROUTINE TEAS(X,N,FIN,EPS,IER)
C
cfh dimension x(4) is garbage, as x(i), i=5,N
cfh is used further down...
cfh      DIMENSION X(4)
      DIMENSION X(40)
C
C        TEST ON WRONG INPUT PARAMETER N
C
      NEW=N
      IF(NEW-10) 1,2,2
    1 IER=-1
      RETURN
C
C        CALCULATE INITIAL VALUES FOR THE EPSILON ARRAY
C
    2 ISW1=0
      ISW2=0
      W1=1.E35
      W7=X(4)-X(3)
      IF(W7)3,4,3
    3 W1=1./W7
C
    4 W5=1.E35
      W7=X(2)-X(1)
      IF(W7)5,6,5
    5 W5=1./W7
C
    6 W4=X(3)-X(2)
      IF(W4)9,7,9
    7 W4=1.E35
      T=X(2)
      W2=X(3)
    8 W3=1.E35
      GO TO 17
C
    9 W4=1./W4
C
      T=1.E35
      W7=W4-W5
      IF(W7)10,11,10
   10 T=X(2)+1./W7
C
   11 W2=W1-W4
      IF(W2)15,12,15
   12 W2=1.E35
      IF(T-1.E35)13,14,14
   13 ISW2=1
   14 W3=W4
      GO TO 17
C
   15 W2=X(3)+1./W2
      W7=W2-T
      IF(W7)16,8,16
   16 W3=W4+1./W7
C
   17 ISW1=ISW2
      ISW2=0
      IMIN=4
C
C        CALCULATE DIAGONALS OF THE EPSILON ARRAY IN A DO-LOOP
C
      DO 40 I=5,NEW
      IAUS=I-IMIN
      W4=1.E35
      W5=X(I-1)
      W7=X(I)-X(I-1)
      IF(W7)18,24,18
   18 W4=1./W7
C
      IF(W1-1.E35)19,25,25
   19 W6=W4-W1
C
C        TEST FOR NECESSITY OF A SINGULAR RULE
C
      IF(ABS(W6)-ABS(W4)*1.E-4)20,20,22
   20 ISW2=1
      IF(W6)22,21,22
   21 W5=1.E35
      W6=W1
      IF(W2-1.E35)28,26,26
   22 W5=X(I-1)+1./W6
C
C        FIRST TEST FOR LOSS OF SIGNIFICANCE
C
      IF(ABS(W5)-ABS(X(I-1))*1.E-5)23,24,24
   23 IF(W5)36,24,36
C
   24 W7=W5-W2
      IF(W7)27,25,27
   25 W6=1.E35
   26 ISW2=0
      X(IAUS)=W2
      GO TO 37
   27 W6=W1+1./W7
   28 IF(ISW1-1)33,29,29
C
C        CALCULATE X(IAUS) WITH HELP OF SINGULAR RULE
C
   29 IF(W2-1.E35)30,32,32
   30 W7=W5/(W2-W5)+T/(W2-T)+X(I-2)/(X(I-2)-W2)
      IF(1.+W7)31,38,31
   31 X(IAUS)=W7*W2/(1.+W7)
      GO TO 39
C
   32 X(IAUS)=W5+T-X(I-2)
      GO TO 39
C
   33 W7=W6-W3
      IF(W7)34,38,34
   34 X(IAUS)=W2+1./W7
C
C        SECOND TEST FOR LOSS OF SIGNIFICANCE
C
      IF(ABS(X(IAUS))-ABS(W2)*1.E-5)35,37,37
   35 IF(X(IAUS))36,37,36
C
   36 NEW=IAUS-1
      ISW2=0
      GO TO 41
C
   37 IF(W2-1.E35)39,38,38
   38 X(IAUS)=1.E35
      IMIN=I
C
   39 W1=W4
      T=W2
      W2=W5
      W3=W6
      ISW1=ISW2
   40 ISW2=0
C
      NEW=NEW-IMIN
C
C        TEST FOR ACCURACY
C
   41 IEND=NEW-1
      DO 47 I=1,IEND
      W1=ABS(X(I)-X(I+1))
      W2=ABS(X(I+1))
      IF(W1-EPS)44,44,42
   42 IF(W2-1.)46,46,43
   43 IF(W1-EPS*W2)44,44,46
   44 ISW2=ISW2+1
      IF(3-ISW2)45,45,47
   45 FIN=X(I)
      IER=0
      RETURN
C
   46 ISW2=0
   47 CONTINUE
C
      if(new.le.0.or.new.gt.50) then
c      write(61,*) new,' teas probleme:new>50 ou new<1'
      ier=-1
      return
      endif
      if(new-6)48,2,2
   48 FIN=X(NEW)
      IER=1
      RETURN
      END
c end of teas
cc
c
      subroutine bldmaprt
cfh the same as bldmap but for rt-grid
c  common block variables:
      include 'simulps_common.inc'
      include 'rayparax_common.inc'
c
c     array size limits
c     ixkms=iykms=izkms=1500
c
      write(61,400)
 400  format(' subroutine bldmaprt')
      xlrt=bld-xnu(1)
      ixmax=(xnu(nxu)+xlrt)/bld
      ylrt=bld-ynu(1)
      iymax=(ynu(nyu)+ylrt)/bld
      zlrt=bld-znu(1)
      izmax=(znu(nzu)+zlrt)/bld
      write(61,402)ixmax,iymax,izmax
 402  format(' array sizes: ',3i5)
c
c  Check for array size overflow
      if(ixmax.gt.ixkmrt.or.iymax.gt.iykmrt.or.izmax.gt.izkmrt)goto 330
      ix=1
      do 10 i=1,ixmax
c
         ix1=ix+1
c
         xnow=float(i)*bld-xlrt
         if (xnow.ge.xnu(ix1)) ix=ix1
c
         ixlort(i)=ix
   10 continue
c  Fill remainder of array with zeroes.
cfh ?? why does he start from i=ixmax (this is the last element above) 
cfh      do 12 i=ixmax,ixkms
      do 12 i=ixmax+1,ixkmrt
         ixlort(i)=0
   12 continue
c
c
      iy=1
      do 15 i=1,iymax
c
         iy1=iy+1
c
         ynow=float(i)*bld-ylrt
         if (ynow.ge.ynu(iy1)) iy=iy1
c
         iylort(i)=iy
   15 continue
c
c  Fill rest of array with zeroes.
cfh      do 17 i=iymax,iykms
      do 17 i=iymax+1,iykmrt
         iylort(i)=0
 17   continue
c
      iz=1
      do 20 i=1,izmax
c
         iz1=iz+1
c
         znow=float(i)*bld-zlrt
         if (znow.ge.znu(iz1)) iz=iz1
c
         izlort(i)=iz
   20 continue
c
c  Fill remainder of array with zeroes.
cfh      do 22 i=izmax,izkms
      do 22 i=izmax+1,izkmrt
         izlort(i)=0
  22  continue
      return
 330   continue
      write(16,331)ixkmrt,iykmrt,izkmrt
 331  format(' ***** error in array size in common/rtlocate/',/,
     *' maximum map dimensions (km)=',/,' x=',i5,' y=',i5,' z=',i5)
      write(16,332)ixmax,iymax,izmax
  332 format(' Actual map size (km): ',/,' x=',i5,' y=',i5,' z=',i5)
      stop
c***** end of subroutine bldmaprt *****
      end
c
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c - - -- - - - - - - - - - - - - - - - - - - - - - - - - -
c
      subroutine intmaprt(x,y,z,ip,jp,kp)
c  same as intmap but for rt -grid and different off-map handling
c
c do error handling for point in between  [3,nmax-2], so
c that pp(nx,ny,nz) does not give an error...
      include 'simulps_common.inc'
      include 'rayparax_common.inc'
c
      ip=int((x+xlrt)/bld)
      ip=ixlort(ip)
c      if (x.lt.xnu(1)) ip=-1
      if (x.lt.xnu(3)) ip=-1
c      if (x.gt.xnu(nxu)) ip=0
      if (x.gt.xnu(nxu-2)) ip=0
      jp=int((ylrt+y)/bld)
      jp=iylort(jp)
      if (y.lt.ynu(3)) jp=-1
      if (y.gt.ynu(nyu-2)) jp=0
      kp=int((z+zlrt)/bld)
      kp=izlort(kp)
      if (z.lt.znu(3)) kp=-1
      if (z.gt.znu(nzu-2)) kp=0
c  If an array element=0, the position is off the map.
      return
c***** end of subroutine intmaprt *****
      end
c

************************************************************************

      Subroutine DateTime(DateToday,TimeNow)

c
c F90 subroutine to retrieve Date and Time for the PC version of Simulps14.
c
c Freddy Aldersons  March 2006.
c

      Implicit  None
      
c Output variables:

      Character DateToday*11,TimeNow*8
      
c Local variables:

      Character Date*8,Time*10
 
      DateToday = ' '
      TimeNow   = ' '

      Call DATE_AND_TIME(Date,Time)

      DateToday( 1: 2) = Date( 7:8)
      DateToday( 3: 3) = '-'
      If (Date(5:6).eq.'01') Then 
          DateToday( 4: 6) = 'Jan'
      Else If (Date(5:6).eq.'02') Then 
          DateToday( 4: 6) = 'Feb'
      Else If (Date(5:6).eq.'03') Then 
          DateToday( 4: 6) = 'Mar'
      Else If (Date(5:6).eq.'04') Then 
          DateToday( 4: 6) = 'Apr'
      Else If (Date(5:6).eq.'05') Then 
          DateToday( 4: 6) = 'May'
      Else If (Date(5:6).eq.'06') Then 
          DateToday( 4: 6) = 'Jun'
      Else If (Date(5:6).eq.'07') Then 
          DateToday( 4: 6) = 'Jul'
      Else If (Date(5:6).eq.'08') Then 
          DateToday( 4: 6) = 'Aug'
      Else If (Date(5:6).eq.'09') Then 
          DateToday( 4: 6) = 'Sep'
      Else If (Date(5:6).eq.'10') Then 
          DateToday( 4: 6) = 'Oct'
      Else If (Date(5:6).eq.'11') Then 
          DateToday( 4: 6) = 'Nov'
      Else If (Date(5:6).eq.'12') Then 
          DateToday( 4: 6) = 'Dec'
          Endif
      DateToday( 7: 7) = '-'
      DateToday( 8:11) = Date(1:4)
      TimeNow  ( 1: 2) = Time(1:2)
      TimeNow  ( 3: 3) = ':'
      TimeNow  ( 4: 5) = Time(3:4)
      TimeNow  ( 6: 6) = ':'
      TimeNow  ( 7: 8) = Time(5:6)

      End

************************************************************************

